@@ -1,5 +1,7 @@
 /*-
- * morph3d.c - Shows 3D morphing objects (TK Version)
+ * morph3d.c - Shows 3D morphing objects
+ *
+ * Converted to GLUT by brianp on 1/1/98
  *
  * This program was inspired on a WindowsNT(R)'s screen saver. It was written 
  * from scratch and it was not based on any other source code. 
@@ -129,8 +131,10 @@ So the angle is:
 
 #include <stdio.h>
 #include <stdlib.h>
+#if !defined(_MSC_VER) && !defined(_WIN32)
 #include <unistd.h>
-#include <gltk.h>
+#endif
+#include <GL/glut.h>
 #include <math.h>
 #include <string.h>
 
@@ -674,14 +678,14 @@ static void draw ( void ) {
 
   glFlush();
 
-  tkSwapBuffers();
+  glutSwapBuffers();
 
   step+=0.05;
 }
 
 static void idle_( void )
 {
-  draw();
+   glutPostRedisplay();
 }
 
 static void reshape( int width, int height )
@@ -695,21 +699,20 @@ static void reshape( int width, int height )
 
 static void pinit(void);
 
-static GLenum key(int k, GLenum mask)
+static void key( unsigned char k, int x, int y )
 {
   switch (k) {
-    case TK_1: object=1; break;
-    case TK_2: object=2; break;
-    case TK_3: object=3; break;
-    case TK_4: object=4; break;
-    case TK_5: object=5; break;
-    case TK_SPACE: mono^=1; break;
-    case TK_RETURN: smooth^=1; break;
-    case TK_ESCAPE:
-      tkQuit();
+    case '1': object=1; break;
+    case '2': object=2; break;
+    case '3': object=3; break;
+    case '4': object=4; break;
+    case '5': object=5; break;
+    case ' ': mono^=1; break;
+    case 13: smooth^=1; break;
+    case 27:
+       exit(0);
   }
   pinit();
-  return GL_FALSE;
 }
 
 static void pinit(void)
@@ -818,12 +821,13 @@ void INIT(void)
 
   object=1;
 
-  tkInitPosition(0,0,640,480);
+  glutInitWindowPosition(0,0);
+  glutInitWindowSize(640,480);
 
-  tkInitDisplayMode( TK_DOUBLE | TK_DIRECT | TK_RGB );
+  glutInitDisplayMode( GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB );
 
-  if (tkInitWindow("Morph 3D - Shows morphing platonic polyhedra") == GL_FALSE) {
-    tkQuit();
+  if (glutCreateWindow("Morph 3D - Shows morphing platonic polyhedra") <= 0) {
+     exit(0);
   }
 
   glClearDepth(1.0);
@@ -832,7 +836,7 @@ void INIT(void)
 
   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
   glFlush();
-  tkSwapBuffers();
+  glutSwapBuffers();
 
   glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
@@ -857,12 +861,11 @@ void INIT(void)
 
   pinit();
 
-  tkExposeFunc( reshape );
-  tkReshapeFunc( reshape );
-  tkKeyDownFunc( key );
-  tkIdleFunc( idle_ );
-  tkDisplayFunc( draw );
-  tkExec();
+  glutReshapeFunc( reshape );
+  glutKeyboardFunc( key );
+  glutIdleFunc( idle_ );
+  glutDisplayFunc( draw );
+  glutMainLoop();
   
 }
 
