@@ -5,6 +5,8 @@
  * Also, this demo utilizes Mesa's fast texture map path.
  *
  * Brian Paul   June 1996
+ *
+ * This file is in the public domain.
  */
 
 
@@ -13,15 +15,16 @@
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
-#include "gltk.h"
+#include "GL/glut.h"
 
+static GLuint Window = 0;
 
 static GLuint TexObj[2];
 static GLfloat Angle = 0.0f;
 static GLboolean HaveTexObj = GL_FALSE;
 
 
-#if defined(GL_VERSION_1_1_fo)
+#if defined(GL_VERSION_1_1)
 #  define TEXTURE_OBJECT 1
 #elif defined(GL_EXT_texture_object)
 #  define TEXTURE_OBJECT 1
@@ -35,6 +38,7 @@ static GLboolean HaveTexObj = GL_FALSE;
 
 static void draw( void )
 {
+   glDepthFunc(GL_EQUAL);
    /*   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );*/
    glClear( GL_COLOR_BUFFER_BIT );
 
@@ -80,7 +84,7 @@ static void draw( void )
    glEnd();
    glPopMatrix();
 
-   tkSwapBuffers();
+   glutSwapBuffers();
 }
 
 
@@ -88,22 +92,22 @@ static void draw( void )
 static void idle( void )
 {
    Angle += 2.0;
-   draw();
+   glutPostRedisplay();
 }
 
 
 
 /* change view Angle, exit upon ESC */
-static GLenum key(int k, GLenum mask)
+static void key(unsigned char k, int x, int y)
 {
    switch (k) {
-      case TK_ESCAPE:
+      case 27:
 #ifdef TEXTURE_OBJECT
          glDeleteTextures( 2, TexObj );
 #endif
-	 tkQuit();
+         glutDestroyWindow(Window);
+         exit(0);
    }
-   return GL_FALSE;
 }
 
 
@@ -239,11 +243,13 @@ static void init( void )
 
 int main( int argc, char *argv[] )
 {
-   tkInitPosition(0, 0, 300, 300);
-   tkInitDisplayMode( TK_RGB | TK_DEPTH | TK_DOUBLE | TK_DIRECT );
+   glutInitWindowPosition(0, 0);
+   glutInitWindowSize(300, 300);
+   glutInitDisplayMode( GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE );
 
-   if (tkInitWindow("Texture Objects") == GL_FALSE) {
-      tkQuit();
+   Window = glutCreateWindow("Texture Objects");
+   if (!Window) {
+      exit(1);
    }
 
    /* check that renderer has the GL_EXT_texture_object extension
@@ -262,11 +268,10 @@ int main( int argc, char *argv[] )
 
    init();
 
-   tkExposeFunc( reshape );
-   tkReshapeFunc( reshape );
-   tkKeyDownFunc( key );
-   tkIdleFunc( idle );
-   tkDisplayFunc( draw );
-   tkExec();
+   glutReshapeFunc( reshape );
+   glutKeyboardFunc( key );
+   glutIdleFunc( idle );
+   glutDisplayFunc( draw );
+   glutMainLoop();
    return 0;
 }
