@@ -4,6 +4,8 @@
  * Example of how to use the GL_MESA_window_pos extension.
  *
  * Brian Paul
+ *
+ * This file is in the public domain.
  */
 
 
@@ -11,15 +13,22 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include "gltk.h"
+#include "GL/glut.h"
+
+#include "../util/readtex.c"  /* a hack, I know */
 
 
 #ifndef M_PI
 #  define M_PI 3.14159265
 #endif
 
+#define IMAGE "../samples/1.rgb"
 
-static TK_RGBImageRec *image;
+
+
+static GLubyte *Image;
+static int ImgWidth, ImgHeight;
+static GLenum ImgFormat;
 
 
 
@@ -44,21 +53,19 @@ static void draw( void )
 #ifdef GL_MESA_window_pos
       glWindowPos2fMESA( x, y );
 #endif
-      glDrawPixels( image->sizeX, image->sizeY, GL_RGB,
-                    GL_UNSIGNED_BYTE, image->data );
+      glDrawPixels( ImgWidth, ImgHeight, ImgFormat, GL_UNSIGNED_BYTE, Image );
    }
 }
 
 
 
 
-static GLenum key(int k, GLenum mask)
+static void key( unsigned char key, int x, int y )
 {
-   switch (k) {
-      case TK_ESCAPE:
-	 tkQuit();
+   switch (key) {
+      case 27:
+         exit(0);
    }
-   return GL_FALSE;
 }
 
 
@@ -72,31 +79,29 @@ static void reshape( int width, int height )
 
 static void init( void )
 {
-   char *filename = "../samples/1.rgb";
-   image = tkRGBImageLoad( filename );
-
-   if (!image) {
-      printf("Error: couldn't load image file: %s\n", filename );
-      exit(1);
+   Image = LoadRGBImage( IMAGE, &ImgWidth, &ImgHeight, &ImgFormat );
+   if (!Image) {
+      printf("Couldn't read %s\n", IMAGE);
+      exit(0);
    }
 }
 
 
 int main( int argc, char *argv[] )
 {
-   tkInitPosition(0, 0, 500, 500);
-   tkInitDisplayMode( TK_RGB | TK_DIRECT );
+   glutInitWindowPosition(0, 0);
+   glutInitWindowSize(500, 500);
+   glutInitDisplayMode( GLUT_RGB );
 
-   if (tkInitWindow("winpos") == GL_FALSE) {
-      tkQuit();
+   if (glutCreateWindow("winpos") <= 0) {
+      exit(0);
    }
 
    init();
 
-   tkExposeFunc( reshape );
-   tkReshapeFunc( reshape );
-   tkKeyDownFunc( key );
-   tkDisplayFunc( draw );
-   tkExec();
+   glutReshapeFunc( reshape );
+   glutKeyboardFunc( key );
+   glutDisplayFunc( draw );
+   glutMainLoop();
    return 0;
 }
