@@ -1,9 +1,9 @@
-/* $Id: osmesa.h,v 1.1 1996/09/13 01:26:41 brianp Exp $ */
+/* $Id: osmesa.h,v 1.8 1997/12/07 17:28:03 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.0
- * Copyright (C) 1995-1996  Brian Paul
+ * Version:  2.6
+ * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +23,27 @@
 
 /*
  * $Log: osmesa.h,v $
+ * Revision 1.8  1997/12/07 17:28:03  brianp
+ * updated version to 2.6
+ *
+ * Revision 1.7  1997/09/29 23:22:16  brianp
+ * updated version to 2.5
+ *
+ * Revision 1.6  1997/07/11 23:07:16  brianp
+ * added OSMesaGetDepthBuffer function (Randy Frank)
+ *
+ * Revision 1.5  1997/02/19 10:13:54  brianp
+ * now test for __QUICKDRAW__ like for __BEOS__ (Randy Frank)
+ *
+ * Revision 1.4  1997/02/10 20:31:41  brianp
+ * added OSMESA_RGB and OSMESA_BGR, per Randy Frank
+ *
+ * Revision 1.3  1997/02/03 20:07:40  brianp
+ * patches for BeOS
+ *
+ * Revision 1.2  1996/10/30 03:12:30  brianp
+ * incremented version to 2.1
+ *
  * Revision 1.1  1996/09/13 01:26:41  brianp
  * Initial revision
  *
@@ -70,7 +91,7 @@ extern "C" {
 
 
 #define OSMESA_MAJOR_VERSION 2
-#define OSMESA_MINOR_VERSION 0
+#define OSMESA_MINOR_VERSION 6
 
 
 
@@ -82,6 +103,8 @@ extern "C" {
 #define OSMESA_RGBA		GL_RGBA
 #define OSMESA_BGRA		0x1
 #define OSMESA_ARGB		0x2
+#define OSMESA_RGB		GL_RGB
+#define OSMESA_BGR		0x4
 
 
 /*
@@ -105,13 +128,17 @@ extern "C" {
 typedef struct osmesa_context *OSMesaContext;
 
 
+#if defined(__BEOS__) || defined(__QUICKDRAW__)
+#pragma export on
+#endif
+
 
 /*
  * Create an Off-Screen Mesa rendering context.  The only attribute needed is
  * an RGBA vs Color-Index mode flag.
  *
  * Input:  format - one of OSMESA_COLOR_INDEX, OSMESA_RGBA, OSMESA_BGRA,
- *                  or OSMESA_ARGB
+ *                  OSMESA_ARGB, OSMESA_RGB, or OSMESA_BGR.
  *         sharelist - specifies another OSMesaContext with which to share
  *                     display lists.  NULL indicates no sharing.
  * Return:  an OSMesaContext or 0 if error
@@ -204,6 +231,28 @@ extern void OSMesaGetIntegerv( GLint pname, GLint *value );
 
 
 
+/*
+ * Return the depth buffer associated with an OSMesa context.
+ * Input:  c - the OSMesa context
+ * Output:  width, height - size of buffer in pixels
+ *          bytesPerValue - bytes per depth value (2 or 4)
+ *          buffer - pointer to depth buffer values
+ * Return:  GL_TRUE or GL_FALSE to indicate success or failure.
+ *
+ * New in Mesa 2.4.
+ */
+extern GLboolean OSMesaGetDepthBuffer( OSMesaContext c,
+                                       GLint *width, GLint *height,
+                                       GLint *bytesPerValue, void **buffer );
+
+
+
+
+#if defined(__BEOS__) || defined(__QUICKDRAW__)
+#pragma export off
+#endif
+
+
 #ifdef __cplusplus
 }
 #endif
