@@ -1,9 +1,9 @@
-/* $Id: xmesa.h,v 1.3 1996/10/30 03:13:01 brianp Exp $ */
+/* $Id: xmesa.h,v 1.12 1998/01/29 02:41:36 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.1
- * Copyright (C) 1995-1996  Brian Paul
+ * Version:  2.6
+ * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +23,33 @@
 
 /*
  * $Log: xmesa.h,v $
+ * Revision 1.12  1998/01/29 02:41:36  brianp
+ * XMesaSetFXmode() now returns GLboolean
+ *
+ * Revision 1.11  1998/01/27 03:28:10  brianp
+ * added XMesaSetFXmode()
+ *
+ * Revision 1.10  1997/12/08 04:03:13  brianp
+ * added XMesaCopySubBuffer() function
+ *
+ * Revision 1.9  1997/12/07 17:28:03  brianp
+ * updated version to 2.6
+ *
+ * Revision 1.8  1997/09/29 23:22:16  brianp
+ * updated version to 2.5
+ *
+ * Revision 1.7  1997/07/11 23:06:55  brianp
+ * added XMesaGetDepthBuffer function (Randy Frank)
+ *
+ * Revision 1.6  1997/05/16 01:54:28  brianp
+ * edited a comment
+ *
+ * Revision 1.5  1997/05/14 03:28:37  brianp
+ * added XMesaDitherColor()
+ *
+ * Revision 1.4  1997/04/27 03:21:34  brianp
+ * added XMesaFindBuffer() and XMesaGarbageCollect
+ *
  * Revision 1.3  1996/10/30 03:13:01  brianp
  * incremented version to 2.1
  *
@@ -38,6 +65,9 @@
 /*
  * Mesa/X11 interface.  This header file serves as the documentation for
  * the Mesa/X11 interface functions.
+ *
+ * Note: this interface isn't intended for user programs.  It's primarily
+ * just for implementing the pseudo-GLX interface.
  */
 
 
@@ -59,7 +89,7 @@ and create a window, you must do the following to use the X/Mesa interface:
 
 5. Make gl* calls to render your graphics.
 
-6. Use XMesaSwapBuffers() when double buffering to update the buffer.
+6. Use XMesaSwapBuffers() when double buffering to swap front/back buffers.
 
 7. Before the X window is destroyed, call XMesaDestroyBuffer().
 
@@ -91,7 +121,7 @@ extern struct Library *XLibBase;
 
 
 #define XMESA_MAJOR_VERSION 2
-#define XMESA_MINOR_VERSION 1
+#define XMESA_MINOR_VERSION 6
 
 
 
@@ -102,6 +132,13 @@ extern struct Library *XLibBase;
 #define XMESA_EXTENSIONS 2
 
 
+/*
+ * Values passed to XMesaSetFXmode:
+ */
+#define XMESA_FX_WINDOW       1
+#define XMESA_FX_FULLSCREEN   2
+
+
 
 typedef struct xmesa_context *XMesaContext;
 
@@ -183,6 +220,15 @@ extern XMesaBuffer XMesaCreatePixmapBuffer( XMesaVisual v, Pixmap p,
 extern void XMesaDestroyBuffer( XMesaBuffer b );
 
 
+/*
+ * Return the XMesaBuffer handle which corresponds to an X drawable, if any.
+ *
+ * New in Mesa 2.3.
+ */
+extern XMesaBuffer XMesaFindBuffer( Display *dpy, Drawable d );
+
+
+
 /*
  * Bind a buffer to a context and make the context the current one.
  */
@@ -208,6 +254,15 @@ extern XMesaBuffer XMesaGetCurrentBuffer( void );
 extern void XMesaSwapBuffers( XMesaBuffer b );
 
 
+/*
+ * Copy a sub-region of the back buffer to the front buffer.
+ *
+ * New in Mesa 2.6
+ */
+extern void XMesaCopySubBuffer( XMesaBuffer b,
+                                int x, int y, int width, int height );
+
+
 /*
  * Return a pointer to the the Pixmap or XImage being used as the back
  * color buffer of an XMesaBuffer.  This function is a way to get "under
@@ -223,6 +278,22 @@ extern GLboolean XMesaGetBackBuffer( XMesaBuffer b,
 
 
 
+/*
+ * Return the depth buffer associated with an XMesaBuffer.
+ * Input:  b - the XMesa buffer handle
+ * Output:  width, height - size of buffer in pixels
+ *          bytesPerValue - bytes per depth value (2 or 4)
+ *          buffer - pointer to depth buffer values
+ * Return:  GL_TRUE or GL_FALSE to indicate success or failure.
+ *
+ * New in Mesa 2.4.
+ */
+extern GLboolean XMesaGetDepthBuffer( XMesaBuffer b,
+                                      GLint *width, GLint *height,
+                                      GLint *bytesPerValue, void **buffer );
+
+
+
 /*
  * Flush/sync a context
  */
@@ -238,6 +309,45 @@ extern const char *XMesaGetString( XMesaContext c, int name );
 
 
 
+/*
+ * Scan for XMesaBuffers whose window/pixmap has been destroyed, then free
+ * any memory used by that buffer.
+ *
+ * New in Mesa 2.3.
+ */
+extern void XMesaGarbageCollect( void );
+
+
+
+/*
+ * Return a dithered pixel value.
+ * Input:  c - XMesaContext
+ *         x, y - window coordinate
+ *         red, green, blue, alpha - color components in [0,1]
+ * Return:  pixel value
+ *
+ * New in Mesa 2.3.
+ */
+extern unsigned long XMesaDitherColor( XMesaContext c, GLint x, GLint y,
+                                       GLfloat red, GLfloat green,
+                                       GLfloat blue, GLfloat alpha );
+
+
+
+/*
+ * 3Dfx Glide driver only!
+ * Set 3Dfx/Glide full-screen or window rendering mode.
+ * Input:  mode - either XMESA_FX_WINDOW (window rendering mode) or
+ *                XMESA_FX_FULLSCREEN (full-screen rendering mode)
+ * Return:  GL_TRUE if success
+ *          GL_FALSE if invalid mode or if not using 3Dfx driver
+ *
+ * New in Mesa 2.6.
+ */
+extern GLboolean XMesaSetFXmode( GLint mode );
+
+
+
 #ifdef __cplusplus
 }
 #endif
