@@ -1,9 +1,9 @@
-/* glaux.h */
+/* $Id: glaux.h,v 1.2 1998/01/16 02:28:47 brianp Exp $ */
 
 /*
  * Mesa - a 3-D graphics library
- * Version:  1.0 beta
- * Copyright (C) 1995  Brian Paul  (brianp@ssec.wisc.edu)
+ * Version:  2.6
+ * Copyright (C) 1995-1996  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -35,12 +35,17 @@ extern "C" {
 #endif
 
 
-#if !defined(AMIGA) && !defined(__WIN32__) && !defined(NeXT) && !defined(__QUICKDRAW__)
+#if !defined(AMIGA) && !defined(__WIN32__) && !defined(NeXT) && !defined(__QUICKDRAW__) && !defined(DOSVGA) && !defined(__BEOS__)
 #  include <X11/Xlib.h>
 #  include <X11/Xutil.h>
 #endif
+#ifdef __BEOS__
+#include "GL/gl.h"
+#include "GL/glu.h"
+#else
 #include <GL/gl.h>
 #include <GL/glu.h>
+#endif
 
 /*
 ** ToolKit Window Types
@@ -232,7 +237,7 @@ enum {
     AUX_WHITE
 };
 
-#ifdef __WIN32__
+#if defined(__WIN32__) || defined(DOSVGA)
    extern float auxRGBMap[17][3];
 #else
    extern float auxRGBMap[8][3];
@@ -254,6 +259,10 @@ typedef struct _AUX_RGBImageRec {
 ** Prototypes
 */
 
+#ifdef __BEOS__
+#pragma export on
+#endif
+
 extern void auxInitDisplayMode(GLbitfield);
 extern void auxInitPosition(int, int, int, int);
 extern GLenum auxInitWindow(char *);
@@ -261,16 +270,16 @@ extern void auxCloseWindow(void);
 extern void auxQuit(void);
 extern void auxSwapBuffers(void);
 
-#if !defined(AMIGA) && !defined(__WIN32__) && !defined(NeXT) && !defined(__QUICKDRAW__)
+#if !defined(AMIGA) && !defined(__WIN32__) && !defined(NeXT) && !defined(__QUICKDRAW__) && !defined(DOSVGA) && !defined(__BEOS__)
 extern Display *auxXDisplay(void);
 extern Window auxXWindow(void);
 #endif
 
-extern void auxMainLoop(void (*)());
+extern void auxMainLoop(void (*)(void));
 extern void auxExposeFunc(void (*)(int, int));
 extern void auxReshapeFunc(void (*)(int, int));
-extern void auxIdleFunc(void (*)());
-extern void auxKeyFunc(int, void (*)());
+extern void auxIdleFunc(void (*)(void));
+extern void auxKeyFunc(int, void (*)(void));
 extern void auxMouseFunc(int, int, void (*)(AUX_EVENTREC *));
 
 extern void auxDeleteMouseFunc( int, int , void (*)(AUX_EVENTREC *));
@@ -321,6 +330,9 @@ extern void auxGetScreenSize( GLint *width, GLint *height );
 
 extern void auxAnimation( GLint flag );
 
+#ifdef __BEOS__
+#pragma export off
+#endif
 
 #ifdef __cplusplus
 }
