@@ -4,12 +4,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(AMIGA) && !defined(__WIN32__) && !defined(__QUICKDRAW__)
+#if !defined(AMIGA) && !defined(__WIN32__) && !defined(__QUICKDRAW__) && !defined(DOSVGA) && !defined(NeXT) && !defined(__BEOS__)
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 #endif
-#include <GL/gl.h>
+#include "GL/gl.h"
 #include "gltk.h"
 #include "glaux.h"
 
@@ -44,7 +44,7 @@ static int displayModeID = 0;
 static int animate = 0;
 
 
-#ifdef __WIN32__
+#if defined(__WIN32__) || defined(DOSVGA)
 #define NCOLORS 17
 float auxRGBMap[NCOLORS][3] = {
     {0,0,0},
@@ -369,7 +369,7 @@ void auxSwapBuffers(void)
     tkSwapBuffers();
 }
 
-#if !defined(AMIGA) && !defined(__WIN32__) && !defined(__QUICKDRAW__)
+#if !defined(AMIGA) && !defined(__WIN32__) && !defined(__QUICKDRAW__) && !defined(DOSVGA) && !defined(NeXT) && !defined(__BEOS__)
 /* for systems with X only... */
 Display *auxXDisplay(void)
 {
@@ -439,9 +439,14 @@ void auxGetMouseLoc(int *x, int *y)
 
 void auxGetScreenSize( GLint *width, GLint *height )
 {
+#if defined(DOSVGA)
    /* This is a kludge! */
    *width = 1280;
    *height = 1024;
+#else
+   *width = 320;
+   *height = 200;
+#endif
 }
 
 
