@@ -1,9 +1,9 @@
-/* nurbsutl.c */
+/* $Id: nurbsutl.c,v 1.6 1997/10/29 02:02:20 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  1.2
- * Copyright (C) 1995  Brian Paul  (brianp@ssec.wisc.edu)
+ * Version:  2.4
+ * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,31 +22,23 @@
 
 
 /*
-$Id: nurbsutl.c,v 1.8 1996/02/20 20:48:43 brianp Exp $
-
-$Log: nurbsutl.c,v $
- * Revision 1.8  1996/02/20  20:48:43  brianp
- * applied Bogdan's February 20th patches (fixes patch cracks)
+ * $Log: nurbsutl.c,v $
+ * Revision 1.6  1997/10/29 02:02:20  brianp
+ * various MS Windows compiler changes (David Bucciarelli, v20 3dfx driver)
  *
- * Revision 1.7  1996/01/26  15:32:08  brianp
- * added Bogdan Sikorski's January 25th and 26th patches
+ * Revision 1.5  1997/07/24 01:28:44  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
  *
- * Revision 1.6  1995/12/12  21:33:36  brianp
- * fixed a few C+ errors/warnings in castings
+ * Revision 1.4  1997/05/28 02:29:38  brianp
+ * added support for precompiled headers (PCH), inserted APIENTRY keyword
  *
- * Revision 1.5  1995/11/03  14:15:13  brianp
- * Bogdan's November 3, 1995 updates
+ * Revision 1.3  1997/05/27 03:19:54  brianp
+ * minor clean-up
  *
- * Revision 1.4  1995/09/20  18:25:57  brianp
- * removed Bogdan's old email address
+ * Revision 1.2  1997/05/27 03:00:16  brianp
+ * incorporated Bogdan's new NURBS code
  *
- * Revision 1.3  1995/08/04  13:09:59  brianp
- * include gluP.h to define NULL, just in case
- *
- * Revision 1.2  1995/07/28  21:37:30  brianp
- * updates from Bogdan on July 28
- *
- * Revision 1.1  1995/07/28  14:45:10  brianp
+ * Revision 1.1  1996/09/27 01:19:39  brianp
  * Initial revision
  *
  */
@@ -58,8 +50,15 @@ $Log: nurbsutl.c,v $
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <math.h>
+#include <stdlib.h>
+#include "gluP.h"
 #include "nurbs.h"
+#endif
+
 
 GLenum
 test_knot(GLint nknots, GLfloat *knot, GLint order)
@@ -87,8 +86,11 @@ test_knot(GLint nknots, GLfloat *knot, GLint order)
 	return GLU_NO_ERROR;
 }
 
-/* qsort function */
 static int
+/* qsort function */
+#if defined(FX) && defined(__WIN32__)
+_cdecl
+#endif
 knot_sort(const void *a, const void *b)
 {
 	GLfloat x,y;
@@ -616,6 +618,197 @@ calc_factor(GLfloat *pts,GLint order,GLint indx,GLint stride,GLfloat tolerance,
 	return ((GLint)len+1);
 }
 
+/* we can't use the Mesa evaluators - no way to get the point coords */
+/* so we use our own Bezier point calculus routines */
+/* because I'm lazy, I reuse the ones from eval.c */
+
+static void
+bezier_curve(GLfloat *cp, GLfloat *out, GLfloat t,
+	GLuint dim, GLuint order, GLint offset)
+{
+	GLfloat s, powert;
+	GLuint i, k, bincoeff;
+
+	if(order >= 2)
+	{ 
+		bincoeff = order-1;
+		s = 1.0-t;
+
+		for(k=0; k<dim; k++)
+			out[k] = s*cp[k] + bincoeff*t*cp[offset+k];
+
+		for(i=2, cp+=2*offset, powert=t*t; i<order; i++, powert*=t, cp +=offset)
+		{
+			bincoeff *= order-i;
+			bincoeff /= i;
+
+			for(k=0; k<dim; k++)
+				out[k] = s*out[k] + bincoeff*powert*cp[k];
+		}
+	}
+	else /* order=1 -> constant curve */
+	{ 
+		for(k=0; k<dim; k++)
+			out[k] = cp[k];
+	} 
+}
+
+static GLint
+calc_parametric_factor(GLfloat *pts,GLint order,GLint indx,GLint stride,
+	GLfloat tolerance,GLint dim)
+{
+	GLdouble model[16],proj[16];
+	GLint viewport[4];
+	GLdouble x,y,z,w,x1,y1,z1,x2,y2,z2,x3,y3,z3;
+	GLint i;
+	GLint P;
+	GLfloat bez_pt[4];
+	GLdouble len=0.0,tmp,z_med;
+
+	P = 2*(order+2);
+	glGetDoublev(GL_MODELVIEW_MATRIX,model);
+	glGetDoublev(GL_PROJECTION_MATRIX,proj);
+	glGetIntegerv(GL_VIEWPORT,viewport);
+	z_med = (viewport[2] + viewport[3]) * 0.5;
+	switch(dim)
+	{
+		case 4:
+			for(i=1;i<P;i++)
+			{
+				bezier_curve(pts+indx, bez_pt, (GLfloat)i/(GLfloat)P, 4,
+					order,stride);
+				w = (GLdouble)bez_pt[3];
+				x = (GLdouble)bez_pt[0] / w;
+				y = (GLdouble)bez_pt[1] / w;
+				z = (GLdouble)bez_pt[2] / w;
+				gluProject(x,y,z,model,proj,viewport,&x3,&y3,&z3);
+				z3 *= z_med;
+				bezier_curve(pts+indx, bez_pt, (GLfloat)(i-1)/(GLfloat)P, 4,
+					order,stride);
+				w = (GLdouble)bez_pt[3];
+				x = (GLdouble)bez_pt[0] / w;
+				y = (GLdouble)bez_pt[1] / w;
+				z = (GLdouble)bez_pt[2] / w;
+				gluProject(x,y,z,model,proj,viewport,&x1,&y1,&z1);
+				z1 *= z_med;
+				bezier_curve(pts+indx, bez_pt, (GLfloat)(i+1)/(GLfloat)P, 4,
+					order,stride);
+				w = (GLdouble)bez_pt[3];
+				x = (GLdouble)bez_pt[0] / w;
+				y = (GLdouble)bez_pt[1] / w;
+				z = (GLdouble)bez_pt[2] / w;
+				gluProject(x,y,z,model,proj,viewport,&x2,&y2,&z2);
+				z2 *= z_med;
+				/* calc distance between point (x3,y3,z3) and line segment */
+				/* <x1,y1,z1><x2,y2,z2> */
+				x = x2-x1;
+				y = y2-y1;
+				z = z2-z1;
+				tmp = sqrt(x*x+y*y+z*z);
+				x /= tmp;
+				y /= tmp;
+				z /= tmp;
+				tmp = x3*x+y3*y+z3*z-x1*x-y1*y-z1*z;
+				x = x1+x*tmp-x3;
+				y = y1+y*tmp-y3;
+				z = z1+z*tmp-z3;
+				tmp = sqrt(x*x+y*y+z*z);
+				if(tmp > len)
+					len = tmp;
+			}
+			break;
+		case 3:
+			for(i=1;i<P;i++)
+			{
+				bezier_curve(pts+indx, bez_pt, (GLfloat)i/(GLfloat)P, 3,
+					order,stride);
+				x = (GLdouble)bez_pt[0];
+				y = (GLdouble)bez_pt[1];
+				z = (GLdouble)bez_pt[2];
+				gluProject(x,y,z,model,proj,viewport,&x3,&y3,&z3);
+				z3 *= z_med;
+				bezier_curve(pts+indx, bez_pt, (GLfloat)(i-1)/(GLfloat)P, 3,
+					order,stride);
+				x = (GLdouble)bez_pt[0];
+				y = (GLdouble)bez_pt[1];
+				z = (GLdouble)bez_pt[2];
+				gluProject(x,y,z,model,proj,viewport,&x1,&y1,&z1);
+				z1 *= z_med;
+				bezier_curve(pts+indx, bez_pt, (GLfloat)(i+1)/(GLfloat)P, 3,
+					order,stride);
+				x = (GLdouble)bez_pt[0];
+				y = (GLdouble)bez_pt[1];
+				z = (GLdouble)bez_pt[2];
+				gluProject(x,y,z,model,proj,viewport,&x2,&y2,&z2);
+				z2 *= z_med;
+				/* calc distance between point (x3,y3,z3) and line segment */
+				/* <x1,y1,z1><x2,y2,z2> */
+				x = x2-x1;
+				y = y2-y1;
+				z = z2-z1;
+				tmp = sqrt(x*x+y*y+z*z);
+				x /= tmp;
+				y /= tmp;
+				z /= tmp;
+				tmp = x3*x+y3*y+z3*z-x1*x-y1*y-z1*z;
+				x = x1+x*tmp-x3;
+				y = y1+y*tmp-y3;
+				z = z1+z*tmp-z3;
+				tmp = sqrt(x*x+y*y+z*z);
+				if(tmp > len)
+					len = tmp;
+			}
+			break;
+		case 2:
+			for(i=1;i<P;i++)
+			{
+				bezier_curve(pts+indx, bez_pt, (GLfloat)i/(GLfloat)P, 2,
+					order,stride);
+				x = (GLdouble)bez_pt[0];
+				y = (GLdouble)bez_pt[1];
+				z = 0.0;
+				gluProject(x,y,z,model,proj,viewport,&x3,&y3,&z3);
+				z3 *= z_med;
+				bezier_curve(pts+indx, bez_pt, (GLfloat)(i-1)/(GLfloat)P, 2,
+					order,stride);
+				x = (GLdouble)bez_pt[0];
+				y = (GLdouble)bez_pt[1];
+				z = 0.0;
+				gluProject(x,y,z,model,proj,viewport,&x1,&y1,&z1);
+				z1 *= z_med;
+				bezier_curve(pts+indx, bez_pt, (GLfloat)(i+1)/(GLfloat)P, 2,
+					order,stride);
+				x = (GLdouble)bez_pt[0];
+				y = (GLdouble)bez_pt[1];
+				z = 0.0;
+				gluProject(x,y,z,model,proj,viewport,&x2,&y2,&z2);
+				z2 *= z_med;
+				/* calc distance between point (x3,y3,z3) and line segment */
+				/* <x1,y1,z1><x2,y2,z2> */
+				x = x2-x1;
+				y = y2-y1;
+				z = z2-z1;
+				tmp = sqrt(x*x+y*y+z*z);
+				x /= tmp;
+				y /= tmp;
+				z /= tmp;
+				tmp = x3*x+y3*y+z3*z-x1*x-y1*y-z1*z;
+				x = x1+x*tmp-x3;
+				y = y1+y*tmp-y3;
+				z = z1+z*tmp-z3;
+				tmp = sqrt(x*x+y*y+z*z);
+				if(tmp > len)
+					len = tmp;
+			}
+			break;
+
+	}
+	if(len < tolerance)
+		return (order);
+	else
+		return (GLint)(sqrt(len/tolerance)*(order+2)+1);
+}
+
 static GLenum
 calc_sampling_3D(new_ctrl_type *new_ctrl, GLfloat tolerance, GLint dim,
 	GLint uorder, GLint vorder, GLint **ufactors, GLint **vfactors)
@@ -636,7 +829,7 @@ calc_sampling_3D(new_ctrl_type *new_ctrl, GLfloat tolerance, GLint dim,
 	if((*vfactors=(GLint *)malloc(sizeof(GLint)*vfactor_cnt*3))
 			==NULL)
 	{
-		free(ufactors);
+		free(*ufactors);
 		return GLU_OUT_OF_MEMORY;
 	}
 	ctrl=new_ctrl->geom_ctrl;
@@ -689,6 +882,79 @@ calc_sampling_3D(new_ctrl_type *new_ctrl, GLfloat tolerance, GLint dim,
 	return GL_NO_ERROR;
 }
 
+static GLenum
+calc_sampling_param_3D(new_ctrl_type *new_ctrl, GLfloat tolerance, GLint dim,
+	GLint uorder, GLint vorder, GLint **ufactors, GLint **vfactors)
+{
+	GLfloat		*ctrl;
+	GLint		tmp_factor1,tmp_factor2;
+	GLint		ufactor_cnt,vfactor_cnt;
+	GLint		offset1,offset2,offset3;
+	GLint		i,j;
+
+	ufactor_cnt=new_ctrl->s_bezier_cnt;
+	vfactor_cnt=new_ctrl->t_bezier_cnt;
+	if((*ufactors=(GLint *)malloc(sizeof(GLint)*ufactor_cnt*3))
+			==NULL)
+	{
+		return GLU_OUT_OF_MEMORY;
+	}
+	if((*vfactors=(GLint *)malloc(sizeof(GLint)*vfactor_cnt*3))
+			==NULL)
+	{
+		free(*ufactors);
+		return GLU_OUT_OF_MEMORY;
+	}
+	ctrl=new_ctrl->geom_ctrl;
+	offset1=new_ctrl->geom_t_stride*vorder;
+	offset2=new_ctrl->geom_s_stride*uorder;
+	for(j=0;j<vfactor_cnt;j++)
+	{
+		*(*vfactors+j*3+1)=tmp_factor1=calc_parametric_factor(ctrl,vorder,
+			j*offset1,dim,tolerance,dim);
+		/* loop ufactor_cnt-1 times */
+		for(i=1;i<ufactor_cnt;i++)
+		{
+			tmp_factor2=calc_parametric_factor(ctrl,vorder,
+				j*offset1+i*offset2,dim,tolerance,dim);
+			if(tmp_factor2>tmp_factor1)
+				tmp_factor1=tmp_factor2;
+		}
+		/* last time for the opposite edge */
+		*(*vfactors+j*3+2)=tmp_factor2=calc_parametric_factor(ctrl,vorder,
+			j*offset1+i*offset2-new_ctrl->geom_s_stride,
+			dim,tolerance,dim);
+		if(tmp_factor2>tmp_factor1)
+			*(*vfactors+j*3)=tmp_factor2;
+		else
+			*(*vfactors+j*3)=tmp_factor1;
+	}
+	offset3=new_ctrl->geom_s_stride;
+	offset2=new_ctrl->geom_s_stride*uorder;
+	for(j=0;j<ufactor_cnt;j++)
+	{
+		*(*ufactors+j*3+1)=tmp_factor1=calc_parametric_factor(ctrl,uorder,
+			j*offset2,offset3,tolerance,dim);
+		/* loop vfactor_cnt-1 times */
+		for(i=1;i<vfactor_cnt;i++)
+		{
+			tmp_factor2=calc_parametric_factor(ctrl,uorder,
+				j*offset2+i*offset1,offset3,tolerance,dim);
+			if(tmp_factor2>tmp_factor1)
+				tmp_factor1=tmp_factor2;
+		}
+		/* last time for the opposite edge */
+		*(*ufactors+j*3+2)=tmp_factor2=calc_parametric_factor(ctrl,uorder,
+			j*offset2+i*offset1-new_ctrl->geom_t_stride,
+			offset3,tolerance,dim);
+		if(tmp_factor2>tmp_factor1)
+			*(*ufactors+j*3)=tmp_factor2;
+		else
+			*(*ufactors+j*3)=tmp_factor1;
+	}
+	return GL_NO_ERROR;
+}
+
 static GLenum
 calc_sampling_2D(GLfloat *ctrl, GLint cnt, GLint order,
 	GLfloat tolerance, GLint dim, GLint **factors)
@@ -773,15 +1039,58 @@ glu_do_sampling_3D( GLUnurbsObj *nobj, new_ctrl_type *new_ctrl,
 }
 
 GLenum
-glu_do_sampling_2D( GLUnurbsObj *nobj, GLfloat *ctrl, GLint cnt, GLint order,
-	GLint dim, GLint **factors)
+glu_do_sampling_uv( GLUnurbsObj *nobj, new_ctrl_type *new_ctrl,
+	GLint **sfactors, GLint **tfactors)
 {
-	GLenum err;
+	GLint	s_cnt, t_cnt, i;
+	GLint	u_steps, v_steps;
 
-	*factors=NULL;
+	s_cnt = new_ctrl->s_bezier_cnt;
+	t_cnt = new_ctrl->t_bezier_cnt;
+	*sfactors=NULL;
+	*tfactors=NULL;
+	if((*sfactors=(GLint *)malloc(sizeof(GLint)*s_cnt*3))
+			==NULL)
+	{
+		return GLU_OUT_OF_MEMORY;
+	}
+	if((*tfactors=(GLint *)malloc(sizeof(GLint)*t_cnt*3))
+			==NULL)
+	{
+		free(*sfactors);
+		return GLU_OUT_OF_MEMORY;
+	}
+	u_steps = nobj->u_step;
+	v_steps = nobj->v_step;
+	for(i=0; i<s_cnt; i++)
+	{
+		*(*sfactors+i*3) = u_steps;
+		*(*sfactors+i*3+1) = u_steps;
+		*(*sfactors+i*3+2) = u_steps;
+	}
+	for(i=0; i<t_cnt; i++)
+	{
+		*(*tfactors+i*3) = v_steps;
+		*(*tfactors+i*3+1) = v_steps;
+		*(*tfactors+i*3+2) = v_steps;
+	}
+	return GLU_NO_ERROR;
+}
+
+GLenum
+glu_do_sampling_param_3D( GLUnurbsObj *nobj, new_ctrl_type *new_ctrl,
+	GLint **sfactors, GLint **tfactors)
+{
+	GLint	dim;
+	GLenum	err;
+
+	*sfactors=NULL;
+	*tfactors=NULL;
+	dim=nobj->surface.geom.dim;
 	set_sampling_and_culling(nobj);
-	if((err=calc_sampling_2D(ctrl,cnt,order,nobj->sampling_tolerance,dim,
-		factors))==GLU_ERROR)
+	if((err=calc_sampling_param_3D(new_ctrl,nobj->parametric_tolerance,dim,
+		nobj->surface.geom.sorder,nobj->surface.geom.torder,
+		sfactors,tfactors))==GLU_ERROR)
 	{
 		revert_sampling_and_culling(nobj);
 		call_user_error(nobj,err);
@@ -791,6 +1100,107 @@ glu_do_sampling_2D( GLUnurbsObj *nobj, GLfloat *ctrl, GLint cnt, GLint order,
 	return GLU_NO_ERROR;
 }
 
+GLenum
+glu_do_sampling_2D( GLUnurbsObj *nobj, GLfloat *ctrl, GLint cnt, GLint order,
+	GLint dim, GLint **factors)
+{
+	GLenum err;
+
+	set_sampling_and_culling(nobj);
+	err=calc_sampling_2D(ctrl,cnt,order,nobj->sampling_tolerance,dim,
+		factors);
+	revert_sampling_and_culling(nobj);
+	return err;
+}
+
+
+GLenum
+glu_do_sampling_u( GLUnurbsObj *nobj, GLfloat *ctrl, GLint cnt, GLint order,
+	GLint dim, GLint **factors)
+{
+	GLint	i;
+	GLint	u_steps;
+
+	cnt /= order;
+	if((*factors=(GLint *)malloc(sizeof(GLint)*cnt))
+			==NULL)
+	{
+		return GLU_OUT_OF_MEMORY;
+	}
+	u_steps = nobj->u_step;
+	for(i=0; i<cnt; i++)
+		(*factors)[i] = u_steps;
+	return GLU_NO_ERROR;
+}
+
+GLenum
+glu_do_sampling_param_2D( GLUnurbsObj *nobj, GLfloat *ctrl, GLint cnt,
+	GLint order, GLint dim, GLint **factors)
+{
+	GLint	i;
+	GLint	u_steps;
+	GLfloat	tolerance;
+
+	set_sampling_and_culling(nobj);
+	tolerance = nobj->parametric_tolerance;
+	cnt /= order;
+	if((*factors=(GLint *)malloc(sizeof(GLint)*cnt))
+			==NULL)
+	{
+		revert_sampling_and_culling(nobj);
+		return GLU_OUT_OF_MEMORY;
+	}
+	u_steps = nobj->u_step;
+	for(i=0; i<cnt; i++)
+	{
+		(*factors)[i] = calc_parametric_factor(ctrl,order,0,
+			dim,tolerance,dim);
+
+	}
+	revert_sampling_and_culling(nobj);
+	return GLU_NO_ERROR;
+}
+
+GLenum
+glu_do_sampling_crv( GLUnurbsObj *nobj, GLfloat *ctrl, GLint cnt, GLint order,
+	GLint dim, GLint **factors)
+{
+	GLenum err;
+
+	*factors=NULL;
+	switch(nobj->sampling_method)
+	{
+		case GLU_PATH_LENGTH:
+			if((err=glu_do_sampling_2D(nobj,ctrl,cnt,order,dim,factors))!=
+				GLU_NO_ERROR)
+			{
+				call_user_error(nobj,err);
+				return GLU_ERROR;
+			}
+			break;
+		case GLU_DOMAIN_DISTANCE:
+			if((err=glu_do_sampling_u(nobj,ctrl,cnt,order,dim,factors))!=
+				GLU_NO_ERROR)
+			{
+				call_user_error(nobj,err);
+				return GLU_ERROR;
+			}
+			break;
+		case GLU_PARAMETRIC_ERROR:
+			if((err=glu_do_sampling_param_2D(nobj,ctrl,cnt,order,dim,factors))!=
+				GLU_NO_ERROR)
+			{
+				call_user_error(nobj,err);
+				return GLU_ERROR;
+			}
+			break;
+		default:
+			abort();
+	}
+
+	return GLU_NO_ERROR;
+}
+
 /* TODO - i don't like this culling - this one just tests if at least one */
 /* ctrl point lies within the viewport . Also the point_in_viewport() */
 /* should be included in the fnctions for efficiency reasons */
