@@ -1,8 +1,8 @@
-/* $Id: quadric.c,v 1.2 1997/03/12 02:15:38 brianp Exp $ */
+/* $Id: quadric.c,v 1.13 1998/02/07 14:28:34 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.6
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,39 @@
 
 /*
  * $Log: quadric.c,v $
+ * Revision 1.13  1998/02/07 14:28:34  brianp
+ * another change to gluQuadricCallback(), this time for StormC compiler
+ *
+ * Revision 1.12  1998/02/05 00:43:19  brianp
+ * Yes, still another change to gluQuadricCallback()!
+ *
+ * Revision 1.11  1998/02/04 00:27:43  brianp
+ * yet another change to gluQuadricCallback()!
+ *
+ * Revision 1.10  1998/02/04 00:23:23  brianp
+ * fixed CALLBACK problem in gluQuadricCallback() (Stephane Rehel)
+ *
+ * Revision 1.9  1998/02/04 00:20:09  brianp
+ * added missing (int) in ErrorFunc cast
+ *
+ * Revision 1.8  1998/01/16 03:37:51  brianp
+ * fixed another assignment warning in gluQuadricCallback()
+ *
+ * Revision 1.7  1998/01/16 03:35:26  brianp
+ * fixed Windows compilation warnings (Theodore Jump)
+ *
+ * Revision 1.6  1997/10/29 02:02:20  brianp
+ * various MS Windows compiler changes (David Bucciarelli, v20 3dfx driver)
+ *
+ * Revision 1.5  1997/09/17 01:51:48  brianp
+ * changed glu*Callback() functions to match prototype in glu.h
+ *
+ * Revision 1.4  1997/07/24 01:28:44  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.3  1997/05/28 02:29:38  brianp
+ * added support for precompiled headers (PCH), inserted APIENTRY keyword
+ *
  * Revision 1.2  1997/03/12 02:15:38  brianp
  * fixed problem in gluPartialDisk() reported by Kenneth H. Carpenter
  *
@@ -40,10 +73,14 @@
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include "gluP.h"
+#endif
 
 
 
@@ -77,7 +114,7 @@ struct GLUquadricObj {
 	GLenum Orientation;		/* GLU_INSIDE or GLU_OUTSIDE */
 	GLboolean TextureFlag;		/* Generate texture coords? */
 	GLenum Normals;		/* GLU_NONE, GLU_FLAT, or GLU_SMOOTH */
-	void (*ErrorFunc)(GLenum err);	/* Error handler callback function */
+	void (CALLBACK *ErrorFunc)(GLenum err);	/* Error handler callback function */
 };
 
 
@@ -100,7 +137,7 @@ static void quadric_error( GLUquadricObj *qobj, GLenum error, const char *msg )
 
 
 
-GLUquadricObj *gluNewQuadric( void )
+GLUquadricObj * APIENTRY gluNewQuadric( void )
 {
    GLUquadricObj *q;
 
@@ -117,7 +154,7 @@ GLUquadricObj *gluNewQuadric( void )
 
 
 
-void gluDeleteQuadric( GLUquadricObj *state )
+void APIENTRY gluDeleteQuadric( GLUquadricObj *state )
 {
    if (state) {
       free( (void *) state );
@@ -127,9 +164,10 @@ void gluDeleteQuadric( GLUquadricObj *state )
 
 
 /*
- * Set the drawing style to be GLU_FILL, GLU_LINE, GLU_SILHOUETTE, or GLU_POINT.
+ * Set the drawing style to be GLU_FILL, GLU_LINE, GLU_SILHOUETTE,
+ * or GLU_POINT.
  */
-void gluQuadricDrawStyle( GLUquadricObj *quadObject, GLenum drawStyle )
+void APIENTRY gluQuadricDrawStyle( GLUquadricObj *quadObject, GLenum drawStyle )
 {
    if (quadObject && (drawStyle==GLU_FILL || drawStyle==GLU_LINE
 		   || drawStyle==GLU_SILHOUETTE || drawStyle==GLU_POINT)) {
@@ -145,7 +183,8 @@ void gluQuadricDrawStyle( GLUquadricObj *quadObject, GLenum drawStyle )
 /*
  * Set the orientation to GLU_INSIDE or GLU_OUTSIDE.
  */
-void gluQuadricOrientation( GLUquadricObj *quadObject, GLenum orientation )
+void APIENTRY gluQuadricOrientation( GLUquadricObj *quadObject,
+                                     GLenum orientation )
 {
    if (quadObject && (orientation==GLU_INSIDE || orientation==GLU_OUTSIDE)) {
       quadObject->Orientation = orientation;
@@ -160,16 +199,27 @@ void gluQuadricOrientation( GLUquadricObj *quadObject, GLenum orientation )
 /*
  * Set the error handler callback function.
  */
-void gluQuadricCallback( GLUquadricObj *qobj,
-			 GLenum which, void (*fn)(GLenum) )
+void APIENTRY gluQuadricCallback( GLUquadricObj *qobj,
+                                  GLenum which, void (CALLBACK *fn)() )
 {
+   /*
+    * UGH, this is a mess!  I thought ANSI was a standard.
+    */
    if (qobj && which==GLU_ERROR) {
-      qobj->ErrorFunc = fn;
+#ifdef __CYGWIN32__
+      qobj->ErrorFunc = (void(*)(int))fn;
+#elif defined(_WIN32)
+      qobj->ErrorFunc = (void(CALLBACK*)(int))fn;
+#elif defined(__STORM__)
+      qobj->ErrorFunc = (void(CALLBACK*)(GLenum))fn;
+#else
+      qobj->ErrorFunc = (void(CALLBACK*)())fn;
+#endif
    }
 }
 
 
-void gluQuadricNormals( GLUquadricObj *quadObject, GLenum normals )
+void APIENTRY gluQuadricNormals( GLUquadricObj *quadObject, GLenum normals )
 {
    if (quadObject
          && (normals==GLU_NONE || normals==GLU_FLAT || normals==GLU_SMOOTH)) {
@@ -178,8 +228,8 @@ void gluQuadricNormals( GLUquadricObj *quadObject, GLenum normals )
 }
 
 
-void gluQuadricTexture( GLUquadricObj *quadObject,
-		        GLboolean textureCoords )
+void APIENTRY gluQuadricTexture( GLUquadricObj *quadObject,
+                                 GLboolean textureCoords )
 {
    if (quadObject) {
       quadObject->TextureFlag = textureCoords;
@@ -207,9 +257,9 @@ static void normal3f( GLfloat x, GLfloat y, GLfloat z )
 
 
 
-void gluCylinder( GLUquadricObj *qobj,
-                  GLdouble baseRadius, GLdouble topRadius, GLdouble height,
-                  GLint slices, GLint stacks )
+void APIENTRY gluCylinder( GLUquadricObj *qobj,
+                           GLdouble baseRadius, GLdouble topRadius,
+                           GLdouble height, GLint slices, GLint stacks )
 {
    GLdouble da, r, dr, dz;
    GLfloat x, y, z, nz, nsign;
@@ -339,8 +389,8 @@ void gluCylinder( GLUquadricObj *qobj,
 
 
 
-void gluSphere( GLUquadricObj *qobj,
-                GLdouble radius, GLint slices, GLint stacks )
+void APIENTRY gluSphere( GLUquadricObj *qobj,
+                         GLdouble radius, GLint slices, GLint stacks )
 {
    GLfloat rho, drho, theta, dtheta;
    GLfloat x, y, z;
@@ -503,9 +553,9 @@ void gluSphere( GLUquadricObj *qobj,
 
 
 
-void gluDisk( GLUquadricObj *qobj,
-              GLdouble innerRadius, GLdouble outerRadius,
-              GLint slices, GLint loops )
+void APIENTRY gluDisk( GLUquadricObj *qobj,
+                       GLdouble innerRadius, GLdouble outerRadius,
+                       GLint slices, GLint loops )
 {
    GLdouble a, da;
    GLfloat r, dr;
@@ -623,9 +673,9 @@ void gluDisk( GLUquadricObj *qobj,
 
 
 
-void gluPartialDisk( GLUquadricObj *qobj, GLdouble innerRadius,
-		     GLdouble outerRadius, GLint slices, GLint loops,
-		     GLdouble startAngle, GLdouble sweepAngle )
+void APIENTRY gluPartialDisk( GLUquadricObj *qobj, GLdouble innerRadius,
+                              GLdouble outerRadius, GLint slices, GLint loops,
+                              GLdouble startAngle, GLdouble sweepAngle )
 {
    if (qobj->Normals!=GLU_NONE) {
       if (qobj->Orientation==GLU_OUTSIDE) {
