@@ -1,3 +1,26 @@
+/* $Id */
+
+/*
+ * $Log: tkwndws.c,v $
+ * Revision 1.4  1997/08/01 20:00:00  brianp
+ * applied more Cynus-Win32 changes from Stephane Rehel
+ *
+ * Revision 1.3  1997/05/03 03:05:07  brianp
+ * added & to tkhpalette parameter to WMesaCreateContext()
+ *
+ * Revision 1.2  1997/04/14 21:58:35  brianp
+ * fixed C++ style comments
+ * added Cygnus-Win32 changes from Stephane
+ *
+ */
+
+
+/*
+ * This is the MS Window version of tk.
+ * Based on SGI code.  See copyright below.
+ */
+
+
 /*
  * (c) Copyright 1993, Silicon Graphics, Inc.
  * ALL RIGHTS RESERVED
@@ -35,14 +58,14 @@
  * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
  */
 
-// Mesa Tweaking by: Mark E. Peterson (markp@ic.mankato.mn.us)
+/* Mesa Tweaking by: Mark E. Peterson (markp@ic.mankato.mn.us) */
 
 #include <windows.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "gltk.h"
-#include "gl\wmesa.h"
+#include "GL/wmesa.h"
 
 #define static
 
@@ -62,6 +85,10 @@ if ( !(x) ) {                                           \
 
 /******************************************************************************/
 
+#ifdef __CYGWIN32__
+#define WNDCLASSW WNDCLASS
+#endif
+
 static struct _WINDOWINFO {
     int x, y;
     int width, height;
@@ -79,7 +106,7 @@ static HDC      tkhdc      = NULL;
 static HPALETTE tkhpalette = NULL;
 GLboolean tkPopupEnable = TRUE;
 
-// Fixed palette support.
+/* Fixed palette support.*/
 
 #define BLACK   PALETTERGB(0,0,0)
 #define WHITE   PALETTERGB(255,255,255)
@@ -97,10 +124,16 @@ static void (*IdleFunc)(void)                    = NULL;
 static char     *lpszClassName = "tkLibWClass";
 static WCHAR    *lpszClassNameW = L"tkLibWClass";
 
-long FAR PASCAL _export tkWndProc(HWND hWnd, UINT message, DWORD wParam, LONG lParam);
+#if defined(_MSC_VER) && defined(_WIN32)
+#define TKWNDPROCEXPORT __declspec(dllexport) FAR PASCAL
+#else
+#define TKWNDPROCEXPORT FAR PASCAL _export
+#endif
+
+long TKWNDPROCEXPORT tkWndProc(HWND hWnd, UINT message, DWORD wParam, LONG lParam);
 static unsigned char ComponentFromIndex(int i, int nbits, int shift );
 static void PrintMessage( const char *Format, ... );
-//static PALETTEENTRY *FillRgbPaletteEntries( PIXELFORMATDESCRIPTOR *Pfd, PALETTEENTRY *Entries, UINT Count );
+/*static PALETTEENTRY *FillRgbPaletteEntries( PIXELFORMATDESCRIPTOR *Pfd, PALETTEENTRY *Entries, UINT Count );*/
 static HPALETTE CreateCIPalette( HDC Dc );
 static HPALETTE CreateRGBPalette( HDC hdc );
 static void DestroyThisWindow( HWND Window );
@@ -122,7 +155,7 @@ static void FreeMemory( void *Chunk );
 static void DbgPrintf( const char *Format, ... );
 static void pwi( void );
 static void pwr(RECT *pr);
-//static void ShowPixelFormat(HDC hdc);
+/*static void ShowPixelFormat(HDC hdc);*/
 
 #endif
 #define NCOLORS 17
@@ -161,10 +194,16 @@ void tkErrorPopups(GLboolean bEnable)
 void tkCloseWindow(void)
 {
     DestroyThisWindow(tkhwnd);
-	/* if (w.cMain) {
-	   XMesaDestroyContext(w.cMain);
-	}*/
-	WMesaDestroyContext(NULL);
+    /*
+    if (w.cMain) {
+       XMesaDestroyContext(w.cMain);
+    }
+    */
+#ifdef FX
+    fxMesaDestroyContext(NULL);
+#else
+    WMesaDestroyContext();
+#endif
 }
 
 
@@ -298,16 +337,56 @@ GLenum tkInitDisplayModeID(GLint ipfd)
 
 GLenum tkInitWindowAW(char *title, BOOL bUnicode)
 {
+#ifdef FX
+    fxMesaContext Cur;
+	GLint attribs[100];
+	int i;
+#else
     WMesaContext Cur;
+#endif
     WNDCLASS wndclass;
     RECT     WinRect;
     HANDLE   hInstance;
     ATOM     aRegister;
     GLenum   Result = GL_FALSE,RGB_Flag=GL_TRUE,DB_Flag=GL_FALSE;
 
+#ifdef FX
+   /* Build fxMesa attribute list */
+   i = 0;
+   if (TK_IS_DOUBLE(windInfo.type)) {
+      attribs[i] = FXMESA_DOUBLEBUFFER;
+      i++;
+   }
+   if (TK_HAS_DEPTH(windInfo.type)) {
+      attribs[i] = FXMESA_DEPTH_SIZE;
+      i++;
+      attribs[i] = 1;
+      i++;
+   }
+   if (TK_HAS_ALPHA(windInfo.type)) {
+      attribs[i] = FXMESA_ALPHA_SIZE;
+      i++;
+      attribs[i] = 1;
+      i++;
+   }
+   if (TK_HAS_ACCUM(windInfo.type)) {
+      attribs[i] = FXMESA_ACCUM_SIZE;
+      i++;
+      attribs[i] = 1;
+      i++;
+   }
+   if (TK_HAS_STENCIL(windInfo.type)) {
+      attribs[i] = FXMESA_STENCIL_SIZE;
+      i++;
+      attribs[i] = 1;
+      i++;
+   }
+   attribs[i] = FXMESA_NONE;  /* end of list */
+#endif
+
     hInstance = GetModuleHandle(NULL);
 
-    // Must not define CS_CS_PARENTDC style.
+    /* Must not define CS_CS_PARENTDC style.*/
     wndclass.style         = CS_HREDRAW | CS_VREDRAW;
     wndclass.lpfnWndProc   = (WNDPROC)tkWndProc;
     wndclass.cbClsExtra    = 0;
@@ -393,9 +472,9 @@ GLenum tkInitWindowAW(char *title, BOOL bUnicode)
 
     if ( NULL != tkhwnd )
     {
-        // If default window positioning used, find out window position and fix
-        // up the windInfo position info.
-
+       /* If default window positioning used, find out window position and fix
+        * up the windInfo position info.
+        */
         if (windInfo.bDefPos)
         {
             GetWindowRect(tkhwnd, &WinRect);
@@ -421,12 +500,18 @@ GLenum tkInitWindowAW(char *title, BOOL bUnicode)
     }
     if (windInfo.type & TK_DOUBLE)
       DB_Flag=GL_TRUE;
-    Cur=WMesaCreateContext(tkhwnd,tkhpalette,RGB_Flag,DB_Flag);
+#ifdef FX
+    /* XXX something's wrong here! */
+    Cur=fxMesaCreateContext(tkhwnd,tkhpalette,RGB_Flag,DB_Flag, attribs);
+    fxMesaMakeCurrent(Cur);
+#else
+    Cur=WMesaCreateContext(tkhwnd,&tkhpalette,RGB_Flag,DB_Flag);
     WMesaMakeCurrent(Cur);
+#endif
     return GL_TRUE;
 }
 
-// Initialize a window, create a rendering context for that window
+/* Initialize a window, create a rendering context for that window*/
 GLenum tkInitWindow(char *title)
 {
     TKASSERT( NULL==tkhwnd      );
@@ -489,31 +574,33 @@ void tkSetFogRamp(int density, int startIndex)
                     k = i * fogValues + j;
 
                     intensity = i * fogValues + j * colorValues;
-                    //mf: not sure what they're trying to do here
-                    //intensity = (intensity << 8) | intensity; ???
+                    /*mf: not sure what they're trying to do here*/
+                    /*intensity = (intensity << 8) | intensity; ???*/
 
-                // This is a workaround for a GDI palette "feature".  If any of
-                // the static colors are repeated in the palette, those colors
-                // will map to the first occurance.  So, for our case where there
-                // are only two static colors (black and white), if a white
-                // color appears anywhere in the palette other than in the last
-                // entry, the static white will remap to the first white.  This
-                // destroys the nice one-to-one mapping we are trying to achieve.
-                //
-                // There are two ways to workaround this.  The first is to
-                // simply not allow a pure white anywhere but in the last entry.
-                // Such requests are replaced with an attenuated white of
-                // (0xFE, 0xFE, 0xFE).
-                //
-                // The other way is to mark these extra whites with PC_RESERVED
-                // which will cause GDI to skip these entries when mapping colors.
-                // This way the app gets the actual colors requested, but can
-                // have side effects on other apps.
-                //
-                // Both solutions are included below.  The PC_RESERVED solution is
-                // the one currently enabled.  It may have side effects, but taking
-                // over the static colors as we are is a really big side effect that
-                // should swamp out the effects of using PC_RESERVED.
+                    /*
+                      This is a workaround for a GDI palette "feature".  If any of
+                      the static colors are repeated in the palette, those colors
+                      will map to the first occurance.  So, for our case where there
+                      are only two static colors (black and white), if a white
+                      color appears anywhere in the palette other than in the last
+                      entry, the static white will remap to the first white.  This
+                      destroys the nice one-to-one mapping we are trying to achieve.
+
+                      There are two ways to workaround this.  The first is to
+                      simply not allow a pure white anywhere but in the last entry.
+                      Such requests are replaced with an attenuated white of
+                      (0xFE, 0xFE, 0xFE).
+
+                      The other way is to mark these extra whites with PC_RESERVED
+                      which will cause GDI to skip these entries when mapping colors.
+                      This way the app gets the actual colors requested, but can
+                      have side effects on other apps.
+
+                      Both solutions are included below.  The PC_RESERVED solution is
+                      the one currently enabled.  It may have side effects, but taking
+                      over the static colors as we are is a really big side effect that
+                      should swamp out the effects of using PC_RESERVED.
+                    */
                 if (intensity > 0xFF)
                   intensity = 0xFF;
                 pPalEntry[k].peRed =pPalEntry[k].peGreen = pPalEntry[k].peBlue = (BYTE) intensity;
@@ -575,7 +662,11 @@ void tkSetRGBMap( int Size, float *Values )
 
 void tkSwapBuffers(void)
 {
+#ifdef FX
+  fxMesaSwapBuffers();
+#else
   WMesaSwapBuffers();
+#endif
 }
 
 /******************************************************************************/
@@ -639,7 +730,7 @@ GLenum tkGetDisplayMode(void)
  *                                                                     *
  ***********************************************************************/
 
-long FAR PASCAL _export
+long TKWNDPROCEXPORT
 tkWndProc(HWND hWnd, UINT message, DWORD wParam, LONG lParam)
 {
     int key;
@@ -695,9 +786,9 @@ tkWndProc(HWND hWnd, UINT message, DWORD wParam, LONG lParam)
         return (0);
     case WM_QUERYNEWPALETTE:
 
-    // In the foreground!  Let RealizePaletteNow do the work--
-    // if management of the static system color usage is needed,
-    // RealizePaletteNow will take care of it.
+       /* In the foreground!  Let RealizePaletteNow do the work--*/
+       /* if management of the static system color usage is needed,*/
+       /* RealizePaletteNow will take care of it.*/
 
         if ( NULL != tkhpalette )
         {
@@ -711,27 +802,31 @@ tkWndProc(HWND hWnd, UINT message, DWORD wParam, LONG lParam)
 
     case WM_ACTIVATE:
 
-    // If the window is going inactive, the palette must be realized to
-    // the background.  Cannot depend on WM_PALETTECHANGED to be sent since
-    // the window that comes to the foreground may or may not be palette
-    // managed.
+       /*
+         If the window is going inactive, the palette must be realized to
+         the background.  Cannot depend on WM_PALETTECHANGED to be sent since
+         the window that comes to the foreground may or may not be palette
+         managed.
+         */
 
         if ( LOWORD(wParam) == WA_INACTIVE )
         {
             if ( NULL != tkhpalette )
             {
-            // Realize as a background palette.  Need to call
-            // RealizePaletteNow rather than RealizePalette directly to
-            // because it may be necessary to release usage of the static
-            // system colors.
+               /*
+                 Realize as a background palette.  Need to call
+                 RealizePaletteNow rather than RealizePalette directly to
+                 because it may be necessary to release usage of the static
+                 system colors.
+                 */
 
                 if ( RealizePaletteNow( tkhdc, tkhpalette) > 0 )
                     ForceRedraw( hWnd );
             }
         }
 
-    // Allow DefWindowProc() to finish the default processing (which includes
-    // changing the keyboard focus).
+        /* Allow DefWindowProc() to finish the default processing (which includes*/
+        /* changing the keyboard focus).*/
 
         break;
 
@@ -924,7 +1019,7 @@ static HPALETTE CreateCIPalette( HDC Dc )
 
     if ( (Dc != NULL) && (NULL == tkhpalette) )
     {
-                PaletteSize = 256; //(Pfd.cColorBits >= 8) ? 256 : (1 << Pfd.cColorBits);
+                PaletteSize = 256; /*(Pfd.cColorBits >= 8) ? 256 : (1 << Pfd.cColorBits);*/
 
                 LogicalPalette = AllocateZeroedMemory( sizeof(LOGPALETTE) +
                                         (PaletteSize * sizeof(PALETTEENTRY)) );
@@ -1013,7 +1108,9 @@ static long RealizePaletteNow( HDC Dc, HPALETTE Palette)
     if ( NULL != SelectPalette( Dc, Palette, FALSE ) )
     {
       Result = RealizePalette( Dc );
+#ifndef FX
       WMesaPaletteChange(Palette);
+#endif
     }
     return( Result );
 }
@@ -1046,22 +1143,22 @@ CleanUp( void )
 {
     HPALETTE hStock;
 
-// Cleanup the palette.
+    /*Cleanup the palette.*/
 
     if ( NULL != tkhpalette )
     {
-    // If static system color usage is set, restore the system colors.
+       /* If static system color usage is set, restore the system colors.*/
 
         if ((hStock = GetStockObject( DEFAULT_PALETTE ))!=NULL)
           SelectPalette( tkhdc, hStock, FALSE );
         DeleteObject( tkhpalette );
     }
 
-// Cleanup the DC.
+    /* Cleanup the DC.*/
 
     if ( NULL != tkhdc )
         ReleaseDC( tkhwnd, tkhdc );
-// Be really nice and reset global values.
+    /* Be really nice and reset global values.*/
     tkhwnd        = NULL;
     tkhdc         = NULL;
     tkhpalette    = NULL;
