@@ -1,8 +1,8 @@
-/* $Id: attrib.c,v 1.5 1997/01/28 22:13:02 brianp Exp $ */
+/* $Id: attrib.c,v 1.9 1997/07/24 01:24:28 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.4
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,18 @@
 
 /*
  * $Log: attrib.c,v $
+ * Revision 1.9  1997/07/24 01:24:28  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.8  1997/05/28 03:23:09  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.7  1997/04/20 19:49:57  brianp
+ * replaced abort() calls with gl_problem()
+ *
+ * Revision 1.6  1997/04/01 04:18:28  brianp
+ * removed #include "draw.h"
+ *
  * Revision 1.5  1997/01/28 22:13:02  brianp
  * now there's separate state for CI and RGBA logic op enabled
  *
@@ -41,15 +53,18 @@
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <stdlib.h>
 #include <string.h>
 #include "attrib.h"
 #include "context.h"
-#include "draw.h"
 #include "dlist.h"
 #include "macros.h"
 #include "misc.h"
 #include "types.h"
+#endif
 
 
 #define MALLOC_STRUCT(T)  (struct T *) malloc( sizeof(struct T) )
@@ -489,7 +504,7 @@ void gl_PopAttrib( GLcontext* ctx )
 		    sizeof(struct gl_viewport_attrib) );
             break;
          default:
-            abort();
+            gl_problem( ctx, "Bad attrib flag in PopAttrib");
             break;
       }
 
@@ -594,7 +609,7 @@ void gl_PopClientAttrib( GLcontext *ctx )
 		    sizeof(struct gl_array_attrib) );
             break;
          default:
-            abort();
+            gl_problem( ctx, "Bad attrib flag in PopClientAttrib");
             break;
       }
 
