@@ -1,9 +1,9 @@
-/* $Id: bitmap.c,v 1.3 1996/11/06 04:23:18 brianp Exp $ */
+/* $Id: bitmap.c,v 1.9 1998/02/03 23:45:02 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.0
- * Copyright (C) 1995-1996  Brian Paul
+ * Version:  2.5
+ * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +23,24 @@
 
 /*
  * $Log: bitmap.c,v $
+ * Revision 1.9  1998/02/03 23:45:02  brianp
+ * added casts to prevent warnings with Amiga StormC compiler
+ *
+ * Revision 1.8  1997/10/02 03:06:42  brianp
+ * added #include <assert.h>
+ *
+ * Revision 1.7  1997/09/27 00:15:39  brianp
+ * changed parameters to gl_unpack_image()
+ *
+ * Revision 1.6  1997/07/24 01:24:45  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.5  1997/06/20 02:18:09  brianp
+ * replaced Current.IntColor with Current.ByteColor
+ *
+ * Revision 1.4  1997/05/28 03:23:48  brianp
+ * added precompiled header (PCH) support
+ *
  * Revision 1.3  1996/11/06 04:23:18  brianp
  * replaced 0 with GL_COLOR_INDEX in gl_unpack_bitmap()
  *
@@ -35,6 +53,10 @@
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
+#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 #include "bitmap.h"
@@ -44,7 +66,7 @@
 #include "macros.h"
 #include "pb.h"
 #include "types.h"
-
+#endif
 
 
 
@@ -56,8 +78,7 @@ struct gl_image *gl_unpack_bitmap( GLcontext* ctx,
                                    const GLubyte *bitmap )
 {
    return gl_unpack_image( ctx, width, height,
-                           GL_COLOR_INDEX, GL_BITMAP, GL_BITMAP,
-                           bitmap, GL_FALSE );
+                           GL_COLOR_INDEX, GL_BITMAP, bitmap );
 }
 
 
@@ -77,6 +98,10 @@ void gl_render_bitmap( GLcontext* ctx,
    GLint px, py, pz;  /* pixel position */
    GLubyte *ptr;
 
+   assert(bitmap);
+   assert(bitmap->Type == GL_BITMAP);
+   assert(bitmap->Format == GL_COLOR_INDEX);
+
    if (ctx->NewState) {
       gl_update_state(ctx);
       PB_INIT( PB, GL_BITMAP );
@@ -167,16 +192,16 @@ void gl_Bitmap( GLcontext* ctx,
    }
    else if (ctx->RenderMode==GL_FEEDBACK) {
       GLfloat color[4], texcoord[4], invq;
-      color[0] = ctx->Current.IntColor[0] * ctx->Visual->InvRedScale;
-      color[1] = ctx->Current.IntColor[1] * ctx->Visual->InvGreenScale;
-      color[2] = ctx->Current.IntColor[2] * ctx->Visual->InvBlueScale;
-      color[3] = ctx->Current.IntColor[3] * ctx->Visual->InvAlphaScale;
+      color[0] = ctx->Current.ByteColor[0] * ctx->Visual->InvRedScale;
+      color[1] = ctx->Current.ByteColor[1] * ctx->Visual->InvGreenScale;
+      color[2] = ctx->Current.ByteColor[2] * ctx->Visual->InvBlueScale;
+      color[3] = ctx->Current.ByteColor[3] * ctx->Visual->InvAlphaScale;
       invq = 1.0F / ctx->Current.TexCoord[3];
       texcoord[0] = ctx->Current.TexCoord[0] * invq;
       texcoord[1] = ctx->Current.TexCoord[1] * invq;
       texcoord[2] = ctx->Current.TexCoord[2] * invq;
       texcoord[3] = ctx->Current.TexCoord[3];
-      FEEDBACK_TOKEN( ctx, (GLfloat) GL_BITMAP_TOKEN );
+      FEEDBACK_TOKEN( ctx, (GLfloat) (GLint) GL_BITMAP_TOKEN );
       /* TODO: Verify XYZW values are correct: */
       gl_feedback_vertex( ctx, ctx->Current.RasterPos[0] - xorig,
 			  ctx->Current.RasterPos[1] - yorig,
