@@ -1,8 +1,8 @@
-/* $Id: clip.c,v 1.11 1997/02/13 21:16:09 brianp Exp $ */
+/* $Id: clip.c,v 1.16 1998/02/03 23:45:36 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.6
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,21 @@
 
 /*
  * $Log: clip.c,v $
+ * Revision 1.16  1998/02/03 23:45:36  brianp
+ * added space parameter to clip interpolation functions
+ *
+ * Revision 1.15  1998/01/06 02:40:52  brianp
+ * added DavidB's clipping interpolation optimization
+ *
+ * Revision 1.14  1997/07/24 01:24:45  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.13  1997/05/28 03:23:48  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.12  1997/04/02 03:10:06  brianp
+ * call gl_analyze_modelview_matrix instead of gl_compute_modelview_inverse
+ *
  * Revision 1.11  1997/02/13 21:16:09  brianp
  * if too many vertices in polygon return VB_SIZE-1, not VB_SIZE
  *
@@ -60,6 +75,9 @@
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <string.h>
 #include "clip.h"
 #include "context.h"
@@ -69,6 +87,7 @@
 #include "types.h"
 #include "vb.h"
 #include "xform.h"
+#endif
 
 
 
@@ -77,11 +96,10 @@
 #define LINTERP( T, A, B )   ( (A) + (T) * ( (B) - (A) ) )
 
 
+/* Clipping coordinate spaces */
 #define EYE_SPACE 1
 #define CLIP_SPACE 2
 
-static GLuint Space;
-
 
 
 /*
@@ -89,13 +107,14 @@ static GLuint Space;
  * coordinates when clipping has to be done.  In general, we compute
  *     aux[dst] = aux[in] + t * (aux[out] - aux[in])
  * where aux is the quantity to be interpolated.
- * Input:  dst - index of array position to store interpolated value
+ * Input:  space - either EYE_SPACE or CLIP_SPACE
+ *         dst - index of array position to store interpolated value
  *         t - a value in [0,1]
  *         in - index of array position corresponding to 'inside' vertex
  *         out - index of array position corresponding to 'outside' vertex
  */
-static void interpolate_aux( GLcontext* ctx,
-                             GLuint dst, GLfloat t, GLuint in, GLuint out )
+void interpolate_aux( GLcontext* ctx, GLuint space,
+                      GLuint dst, GLfloat t, GLuint in, GLuint out )
 {
    struct vertex_buffer* VB = ctx->VB;
 
@@ -123,7 +142,7 @@ static void interpolate_aux( GLcontext* ctx,
 
    if (ctx->ClipMask & CLIP_TEXTURE_BIT) {
       /* TODO: is more sophisticated texture coord interpolation needed?? */
-      if (Space==CLIP_SPACE) {
+      if (space==CLIP_SPACE) {
 	 /* also interpolate eye Z component */
 	 VB->Eye[dst][2] = LINTERP( t, VB->Eye[in][2], VB->Eye[out][2] );
       }
@@ -136,24 +155,68 @@ static void interpolate_aux( GLcontext* ctx,
 }
 
 
+/*
+ * Some specialized version of the interpolate_aux
+ *
+ */
+
+void interpolate_aux_color_tex2( GLcontext* ctx, GLuint space,
+				 GLuint dst, GLfloat t, GLuint in, GLuint out )
+{
+   struct vertex_buffer* VB = ctx->VB;
+
+   VB->Fcolor[dst][0] = LINTERP( t, VB->Fcolor[in][0], VB->Fcolor[out][0] );
+   VB->Fcolor[dst][1] = LINTERP( t, VB->Fcolor[in][1], VB->Fcolor[out][1] );
+   VB->Fcolor[dst][2] = LINTERP( t, VB->Fcolor[in][2], VB->Fcolor[out][2] );
+   VB->Fcolor[dst][3] = LINTERP( t, VB->Fcolor[in][3], VB->Fcolor[out][3] );
+
+   VB->Eye[dst][2] = LINTERP( t, VB->Eye[in][2], VB->Eye[out][2] );
+   VB->TexCoord[dst][0] = LINTERP(t,VB->TexCoord[in][0],VB->TexCoord[out][0]);
+   VB->TexCoord[dst][1] = LINTERP(t,VB->TexCoord[in][1],VB->TexCoord[out][1]);
+}
+
+
+void interpolate_aux_tex2( GLcontext* ctx, GLuint space,
+			   GLuint dst, GLfloat t, GLuint in, GLuint out )
+{
+   struct vertex_buffer* VB = ctx->VB;
+
+   VB->Eye[dst][2] = LINTERP( t, VB->Eye[in][2], VB->Eye[out][2] );
+   VB->TexCoord[dst][0] = LINTERP(t,VB->TexCoord[in][0],VB->TexCoord[out][0]);
+   VB->TexCoord[dst][1] = LINTERP(t,VB->TexCoord[in][1],VB->TexCoord[out][1]);
+}
+
+
+void interpolate_aux_color( GLcontext* ctx, GLuint space,
+			    GLuint dst, GLfloat t, GLuint in, GLuint out )
+{
+   struct vertex_buffer* VB = ctx->VB;
+
+   VB->Fcolor[dst][0] = LINTERP( t, VB->Fcolor[in][0], VB->Fcolor[out][0] );
+   VB->Fcolor[dst][1] = LINTERP( t, VB->Fcolor[in][1], VB->Fcolor[out][1] );
+   VB->Fcolor[dst][2] = LINTERP( t, VB->Fcolor[in][2], VB->Fcolor[out][2] );
+   VB->Fcolor[dst][3] = LINTERP( t, VB->Fcolor[in][3], VB->Fcolor[out][3] );
+}
+
+
 
 
 void gl_ClipPlane( GLcontext* ctx, GLenum plane, const GLfloat *equation )
 {
    GLint p;
 
-   p = (GLint) (plane - GL_CLIP_PLANE0);
+   p = (GLint) plane - (GLint) GL_CLIP_PLANE0;
    if (p<0 || p>=MAX_CLIP_PLANES) {
       gl_error( ctx, GL_INVALID_ENUM, "glClipPlane" );
       return;
    }
 
    /*
-    * The Equation is transformed by the transpose of the inverse of the
+    * The equation is transformed by the transpose of the inverse of the
     * current modelview matrix and stored in the resulting eye coordinates.
     */
-   if (!ctx->ModelViewInvValid) {
-      gl_compute_modelview_inverse(ctx);
+   if (ctx->NewModelViewMatrix) {
+      gl_analyze_modelview_matrix(ctx);
    }
    gl_transform_vector( ctx->Transform.ClipEquation[p], equation,
 		        ctx->ModelViewInv );
@@ -225,7 +288,6 @@ GLuint gl_viewclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
    GLfloat t, dx, dy, dz, dw;
    register GLuint ii, jj;
 
-   Space = CLIP_SPACE;
    ii = *i;
    jj = *j;
 
@@ -245,7 +307,7 @@ GLuint gl_viewclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
 	 /* new vertex put in position VB->Free */			\
          COMPUTE_INTERSECTION( VB->Free, jj, ii )             		\
 	 if (ctx->ClipMask)						\
-	    interpolate_aux( ctx, VB->Free, t, jj, ii );		\
+            ctx->ClipInterpAuxFunc( ctx, CLIP_SPACE, VB->Free, t, jj, ii );\
 	 ii = VB->Free;							\
 	 VB->Free++;							\
 	 if (VB->Free==VB_SIZE)  VB->Free = 1;				\
@@ -257,7 +319,7 @@ GLuint gl_viewclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
 	 /* new vertex put in position VB->Free */			\
          COMPUTE_INTERSECTION( VB->Free, ii, jj );            		\
 	 if (ctx->ClipMask)						\
-	    interpolate_aux( ctx, VB->Free, t, ii, jj );		\
+	    ctx->ClipInterpAuxFunc( ctx, CLIP_SPACE, VB->Free, t, ii, jj );\
 	 jj = VB->Free;							\
 	 VB->Free++;							\
 	 if (VB->Free==VB_SIZE)  VB->Free = 1;				\
@@ -405,8 +467,6 @@ GLuint gl_viewclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
    GLuint n2;
    GLdouble dx, dy, dz, dw, t, neww;
 
-   Space = CLIP_SPACE;
-
 /*
  * We use 6 instances of this code to implement clipping against the
  * 6 sides of the view volume.  Prior to each we define the macros:
@@ -440,7 +500,7 @@ GLuint gl_viewclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
 	    if (t>0.0) {						\
 	       /* interpolate aux info using the value of t */		\
 	       if (ctx->ClipMask)					\
-		  interpolate_aux( ctx, VB->Free, t, currj, prevj );	\
+		  ctx->ClipInterpAuxFunc( ctx, CLIP_SPACE, VB->Free, t, currj, prevj ); \
 	       VB->Edgeflag[VB->Free] = VB->Edgeflag[prevj];		\
 	       /* output new point */					\
 	       OUTLIST[OUTCOUNT] = VB->Free;				\
@@ -461,7 +521,7 @@ GLuint gl_viewclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
 	    if (t>0.0) {						\
 	       /* interpolate aux info using the value of t */		\
 	       if (ctx->ClipMask)					\
-		  interpolate_aux( ctx, VB->Free, t, prevj, currj );	\
+		  ctx->ClipInterpAuxFunc( ctx, CLIP_SPACE, VB->Free, t, prevj, currj ); \
 	       VB->Edgeflag[VB->Free] = VB->Edgeflag[prevj];		\
 	       /* output new point */					\
 	       OUTLIST[OUTCOUNT] = VB->Free;				\
@@ -741,8 +801,6 @@ GLuint gl_userclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
 
    GLuint p, ii, jj;
 
-   Space = EYE_SPACE;
-
    ii = *i;
    jj = *j;
 
@@ -782,7 +840,7 @@ GLuint gl_userclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
 
 	       /* Interpolate colors, indexes, and/or texture coords */
 	       if (ctx->ClipMask)
-		  interpolate_aux( ctx, VB->Free, t, jj, ii );
+		  interpolate_aux( ctx, EYE_SPACE, VB->Free, t, jj, ii );
 
 	       ii = VB->Free;
 	       VB->Free++;
@@ -813,7 +871,7 @@ GLuint gl_userclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
 
 	       /* Interpolate colors, indexes, and/or texture coords */
 	       if (ctx->ClipMask)
-		  interpolate_aux( ctx, VB->Free, t, ii, jj );
+		  interpolate_aux( ctx, EYE_SPACE, VB->Free, t, ii, jj );
 
 	       jj = VB->Free;
 	       VB->Free++;
@@ -852,8 +910,6 @@ GLuint gl_userclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
    GLuint previ, prevj;
    GLuint p;
 
-   Space = EYE_SPACE;
-
    /* initialize input vertex list */
    incount = n;
    inlist = vlist;
@@ -909,7 +965,7 @@ GLuint gl_userclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
 
 		  /* interpolate color, index, and/or texture coord */
 		  if (ctx->ClipMask) {
-		     interpolate_aux( ctx, VB->Free, t, currj, prevj);
+		     interpolate_aux( ctx, EYE_SPACE, VB->Free, t, currj, prevj);
 		  }
 		  VB->Edgeflag[VB->Free] = VB->Edgeflag[prevj];
 
@@ -949,7 +1005,7 @@ GLuint gl_userclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
 
 		  /* interpolate color, index, and/or texture coord */
 		  if (ctx->ClipMask) {
-		     interpolate_aux( ctx, VB->Free, t, prevj, currj);
+		     interpolate_aux( ctx, EYE_SPACE, VB->Free, t, prevj, currj);
 		  }
 		  VB->Edgeflag[VB->Free] = VB->Edgeflag[prevj];
 
