@@ -1,8 +1,8 @@
-/* $Id: context.c,v 1.25 1997/03/14 00:24:37 brianp Exp $ */
+/* $Id: context.c,v 1.64 1998/02/05 00:34:56 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.6
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,123 @@
 
 /*
  * $Log: context.c,v $
+ * Revision 1.64  1998/02/05 00:34:56  brianp
+ * added John Stone's initial thread modifications
+ *
+ * Revision 1.63  1998/01/06 02:40:52  brianp
+ * added DavidB's clipping interpolation optimization
+ *
+ * Revision 1.62  1997/12/31 06:10:03  brianp
+ * added Henk Kok's texture validation optimization (AnyDirty flag)
+ *
+ * Revision 1.61  1997/12/08 04:04:52  brianp
+ * changed gl_copy_context() to not use MEMCPY for PolygonStipple (John Stone)
+ *
+ * Revision 1.60  1997/12/06 18:06:50  brianp
+ * moved several static display list vars into GLcontext
+ *
+ * Revision 1.59  1997/11/25 03:37:53  brianp
+ * simple clean-ups for multi-threading (John Stone)
+ *
+ * Revision 1.58  1997/10/29 01:29:09  brianp
+ * added GL_EXT_point_parameters extension from Daniel Barrero
+ *
+ * Revision 1.57  1997/10/16 01:59:08  brianp
+ * added GL_EXT_shared_texture_palette extension
+ *
+ * Revision 1.56  1997/10/15 03:08:50  brianp
+ * don't clear depth buffer in gl_ResizeBuffersMESA()
+ *
+ * Revision 1.55  1997/09/29 22:25:28  brianp
+ * added const to a few function parameters
+ *
+ * Revision 1.54  1997/09/27 00:15:20  brianp
+ * added ctx->DirectContext flag
+ *
+ * Revision 1.53  1997/09/23 00:58:15  brianp
+ * now using hash table for texture objects
+ *
+ * Revision 1.52  1997/09/22 02:33:58  brianp
+ * display lists now implemented with hash table
+ *
+ * Revision 1.51  1997/09/17 01:39:11  brianp
+ * added update_clipmask() function from Michael Pichler
+ *
+ * Revision 1.50  1997/09/10 00:29:19  brianp
+ * set all NewState flags in gl_ResizeBuffersMESA() (Miklos Fazekas)
+ *
+ * Revision 1.49  1997/07/24 01:24:45  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.48  1997/06/20 02:19:49  brianp
+ * replaced Current.IntColor with Current.ByteColor
+ *
+ * Revision 1.47  1997/06/20 01:57:28  brianp
+ * free_shared_state() didn't free display lists
+ *
+ * Revision 1.46  1997/05/31 16:57:14  brianp
+ * added MESA_NO_RASTER env var support
+ *
+ * Revision 1.45  1997/05/28 04:06:03  brianp
+ * implemented projection near/far value stack for Driver.NearFar() function
+ *
+ * Revision 1.44  1997/05/28 03:23:48  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.43  1997/05/26 21:18:54  brianp
+ * better comments
+ *
+ * Revision 1.42  1997/05/26 21:14:13  brianp
+ * gl_create_visual() now takes red/green/blue/alpha_bits arguments
+ *
+ * Revision 1.41  1997/05/24 12:07:07  brianp
+ * removed unused vars
+ *
+ * Revision 1.40  1997/05/14 03:27:24  brianp
+ * miscellaneous alloc/free clean-ups
+ *
+ * Revision 1.39  1997/05/09 22:42:09  brianp
+ * replaced gl_init_vb() with gl_alloc_vb(), similarly for pb
+ *
+ * Revision 1.38  1997/05/01 02:07:35  brianp
+ * added shared state variable NextFreeTextureName
+ *
+ * Revision 1.37  1997/05/01 01:25:33  brianp
+ * added call to gl_init_math()
+ *
+ * Revision 1.36  1997/04/28 02:06:14  brianp
+ * also check if texturing enabled in the test for NeedNormals
+ *
+ * Revision 1.35  1997/04/26 04:34:43  brianp
+ * fixed problem with Depth/Accum/StencilBits initialization (Mark Kilgard)
+ *
+ * Revision 1.34  1997/04/24 01:49:02  brianp
+ * call gl_set_color_function() instead of directly setting pointers
+ *
+ * Revision 1.33  1997/04/24 00:18:10  brianp
+ * Proxy2D tex object was allocated twice.  reported by Randy Frank.
+ *
+ * Revision 1.32  1997/04/16 23:55:06  brianp
+ * added gl_set_api_table()
+ *
+ * Revision 1.31  1997/04/14 02:01:03  brianp
+ * #include "texstate.h" instead of "texture.h"
+ *
+ * Revision 1.30  1997/04/12 17:11:38  brianp
+ * added gl_get_current_context()
+ *
+ * Revision 1.29  1997/04/12 16:54:05  brianp
+ * new NEW_POLYGON state flag
+ *
+ * Revision 1.28  1997/04/12 16:20:51  brianp
+ * added call to Driver.Dither() in gl_update_state()
+ *
+ * Revision 1.27  1997/04/12 12:25:37  brianp
+ * added Driver.QuadFunc and Driver.RectFunc
+ *
+ * Revision 1.26  1997/04/01 04:20:22  brianp
+ * added new matrix type code
+ *
  * Revision 1.25  1997/03/14 00:24:37  brianp
  * only print runtime warnings if MESA_DEBUG env var is set
  *
@@ -110,7 +227,9 @@
  */
 
 
-
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <assert.h>
 #include <math.h>
 #include <stdio.h>
@@ -118,25 +237,29 @@
 #include <string.h>
 #include "accum.h"
 #include "alphabuf.h"
+#include "clip.h"
 #include "context.h"
 #include "depth.h"
-#include "draw.h"
 #include "eval.h"
+#include "hash.h"
 #include "light.h"
 #include "lines.h"
 #include "dlist.h"
 #include "macros.h"
+#include "mmath.h"
 #include "pb.h"
 #include "points.h"
 #include "pointers.h"
+#include "quads.h"
 #include "stencil.h"
 #include "triangle.h"
 #include "teximage.h"
 #include "texobj.h"
-#include "texture.h"
+#include "texstate.h"
 #include "types.h"
 #include "vb.h"
-#include "vertex.h"
+#include "vbfill.h"
+#endif
 
 
 
@@ -147,51 +270,23 @@
 
 #ifdef THREADS
 
-/* A different context may be bound to each thread */
+#include "mthreads.h" /* Mesa platform independent threads interface */
 
-#include <sys/types.h>
-#include <unistd.h>
+static MesaTSD mesa_ctx_tsd;
 
-#define MAX_THREADS 20
-static pid_t ThreadID[MAX_THREADS];
-static GLcontext *ThreadContext[MAX_THREADS];
-static int NumThreads = 0;
-
-GLcontext *gl_get_thread_context( void )
-{
-   pid_t id;
-   int i;
-
-   id = getpid();
-
-   for (i=0;i<MAX_THREADS;i++) {
-      if (ThreadID[i]==id) {
-         return ThreadContext[i];
-      }
-   }
-
-   /* No context for this thread */
-   return NULL;
+static void mesa_ctx_thread_init() {
+  MesaInitTSD(&mesa_ctx_tsd);
 }
 
-static void set_thread_context( GLcontext *ctx )
-{
-   pid_t id = getpid();
-   int i;
-
-   for (i=0;i<NumThreads;i++) {
-      if (ThreadID[i]==id) {
-         ThreadContext[i] = c;
-         return;
-      }
-   }
-   if (i<MAX_THREADS) {
-      ThreadID[i] = getpid();
-      ThreadContext[i] = c;
-      NumThreads++;
-   }
+GLcontext *gl_get_thread_context( void ) {
+  return (GLcontext *) MesaGetTSD(&mesa_ctx_tsd);
 }
 
+static void set_thread_context( GLcontext *ctx ) {
+  MesaSetTSD(&mesa_ctx_tsd, ctx, mesa_ctx_thread_init);
+}
+
+
 #else
 
 /* One Current Context pointer for all threads in the address space */
@@ -372,69 +467,67 @@ static void print_timings( GLcontext *ctx )
 static struct gl_shared_state *alloc_shared_state( void )
 {
    struct gl_shared_state *ss;
-   struct gl_texture_object *t1, *t2, *t3;
-   GLuint i;
 
-   ss = (struct gl_shared_state*) malloc( sizeof(struct gl_shared_state) );
+   ss = (struct gl_shared_state*) calloc( 1, sizeof(struct gl_shared_state) );
    if (!ss)
       return NULL;
-   ss->RefCount = 0;
 
-   /* Display lists */
-   for (i=0;i<MAX_DISPLAYLISTS;i++) {
-      ss->List[i] = NULL;
-   }
+   ss->DisplayList = NewHashTable();
 
-   /* Texture objects
-    * There are two default textures, one for the GL_TEXTURE_1D target and
-    * one for the GL_TEXTURE_2D target.  Both are named zero!
-    */
-   t1 = gl_alloc_texture_object();
-   if (!t1) {
+   ss->TexObjects = NewHashTable();
+
+   /* Default Texture objects */
+   ss->Default1D = gl_alloc_texture_object(ss, 0, 1);
+   ss->Default2D = gl_alloc_texture_object(ss, 0, 2);
+   ss->Default3D = gl_alloc_texture_object(ss, 0, 3);
+
+   if (!ss->DisplayList || !ss->TexObjects
+       || !ss->Default1D || !ss->Default2D || !ss->Default3D) {
+      /* Ran out of memory at some point.  Free everything and return NULL */
+      if (!ss->DisplayList)
+         DeleteHashTable(ss->DisplayList);
+      if (!ss->TexObjects)
+         DeleteHashTable(ss->TexObjects);
+      if (!ss->Default1D)
+         gl_free_texture_object(ss, ss->Default1D);
+      if (!ss->Default2D)
+         gl_free_texture_object(ss, ss->Default2D);
+      if (!ss->Default3D)
+         gl_free_texture_object(ss, ss->Default3D);
       free(ss);
       return NULL;
    }
-   t1->Dimensions = 1;
-
-   t2 = gl_alloc_texture_object();
-   if (!t2) {
-      free(ss);
-      free(t1);
-      return NULL;
+   else {
+      return ss;
    }
-   t2->Dimensions = 2;
-
-   t3 = gl_alloc_texture_object();
-   if (!t3) {
-      free(ss);
-      free(t1);
-      free(t2);
-      return NULL;
-   }
-   t3->Dimensions = 3;
-
-   ss->TexObjectList = t1;
-   ss->TexObjectList->Next = t2;
-   ss->TexObjectList->Next->Next = t3;
-
-   return ss;
 }
 
 
 /*
  * Deallocate a shared state context and all children structures.
  */
-static void free_shared_state( struct gl_shared_state *ss )
+static void free_shared_state( GLcontext *ctx, struct gl_shared_state *ss )
 {
-   struct gl_texture_object *t, *tnext;
-
-   /* Free the linked list of texture objects */
-   t = ss->TexObjectList;
-   while (t) {
-      tnext = t->Next;
-      gl_free_texture_object(t);
-      t = tnext;
+   /* Free display lists */
+   while (1) {
+      GLuint list = HashFirstEntry(ss->DisplayList);
+      if (list) {
+         gl_destroy_list(ctx, list);
+      }
+      else {
+         break;
+      }
    }
+   DeleteHashTable(ss->DisplayList);
+
+   /* Free texture objects */
+   while (ss->TexObjectList)
+   {
+      /* this function removes from linked list too! */
+      gl_free_texture_object(ss, ss->TexObjectList);
+   }
+   DeleteHashTable(ss->TexObjects);
+
    free(ss);
 }
 
@@ -507,18 +600,26 @@ static void initialize_context( GLcontext *ctx )
    GLuint i;
 
    if (ctx) {
-      /* Transformation matrices and stacks */
-      ctx->ModelViewStackDepth = 0;
+      /* Modelview matrix stuff */
+      ctx->NewModelViewMatrix = GL_FALSE;
+      ctx->ModelViewMatrixType = MATRIX_IDENTITY;
       MEMCPY( ctx->ModelViewMatrix, identity, 16*sizeof(GLfloat) );
       MEMCPY( ctx->ModelViewInv, identity, 16*sizeof(GLfloat) );
-      ctx->ModelViewInvValid = GL_TRUE;
+      ctx->ModelViewStackDepth = 0;
 
-      ctx->ProjectionStackDepth = 0;
+      /* Projection matrix stuff */
+      ctx->NewProjectionMatrix = GL_FALSE;
+      ctx->ProjectionMatrixType = MATRIX_IDENTITY;
       MEMCPY( ctx->ProjectionMatrix, identity, 16*sizeof(GLfloat) );
+      ctx->ProjectionStackDepth = 0;
+      ctx->NearFarStack[0][0] = 1.0; /* These values seem weird by make */
+      ctx->NearFarStack[0][1] = 0.0; /* sense mathematically. */
 
-      ctx->TextureStackDepth = 0;
+      /* Texture matrix stuff */
+      ctx->NewTextureMatrix = GL_FALSE;
+      ctx->TextureMatrixType = MATRIX_IDENTITY;
       MEMCPY( ctx->TextureMatrix, identity, 16*sizeof(GLfloat) );
-      ctx->IdentityTexMat = GL_TRUE;
+      ctx->TextureStackDepth = 0;
 
       /* Accumulate buffer group */
       ASSIGN_4V( ctx->Accum.ClearColor, 0.0, 0.0, 0.0, 0.0 );
@@ -548,10 +649,10 @@ static void initialize_context( GLcontext *ctx )
       /* Current group */
       ctx->Current.Index = 1;
       ASSIGN_3V( ctx->Current.Normal, 0.0, 0.0, 1.0 );
-      ctx->Current.IntColor[0] = (GLint) ctx->Visual->RedScale;
-      ctx->Current.IntColor[1] = (GLint) ctx->Visual->GreenScale;
-      ctx->Current.IntColor[2] = (GLint) ctx->Visual->BlueScale;
-      ctx->Current.IntColor[3] = (GLint) ctx->Visual->AlphaScale;
+      ctx->Current.ByteColor[0] = (GLint) ctx->Visual->RedScale;
+      ctx->Current.ByteColor[1] = (GLint) ctx->Visual->GreenScale;
+      ctx->Current.ByteColor[2] = (GLint) ctx->Visual->BlueScale;
+      ctx->Current.ByteColor[3] = (GLint) ctx->Visual->AlphaScale;
       ASSIGN_4V( ctx->Current.RasterPos, 0.0, 0.0, 0.0, 1.0 );
       ctx->Current.RasterPosValid = GL_TRUE;
       ctx->Current.RasterIndex = 1;
@@ -678,6 +779,12 @@ static void initialize_context( GLcontext *ctx )
       /* Point group */
       ctx->Point.SmoothFlag = GL_FALSE;
       ctx->Point.Size = 1.0;
+      ctx->Point.Params[0] = 1.0;
+      ctx->Point.Params[1] = 0.0;
+      ctx->Point.Params[2] = 0.0;
+      ctx->Point.MinSize = 0.0;
+      ctx->Point.MaxSize = (GLfloat) MAX_POINT_SIZE;
+      ctx->Point.Threshold = 1.0;
 
       /* Polygon group */
       ctx->Polygon.CullFlag = GL_FALSE;
@@ -734,11 +841,20 @@ static void initialize_context( GLcontext *ctx )
       ASSIGN_4V( ctx->Texture.EyePlaneR, 0.0, 0.0, 0.0, 0.0 );
       ASSIGN_4V( ctx->Texture.EyePlaneQ, 0.0, 0.0, 0.0, 0.0 );
 
-      assert( ctx->Shared->TexObjectList );
-      assert( ctx->Shared->TexObjectList->Next );
-      ctx->Texture.Current1D = ctx->Shared->TexObjectList;
-      ctx->Texture.Current2D = ctx->Shared->TexObjectList->Next;
-      ctx->Texture.Current3D = ctx->Shared->TexObjectList->Next->Next;
+      ctx->Texture.Current1D = ctx->Shared->Default1D;
+      ctx->Texture.Current2D = ctx->Shared->Default2D;
+      ctx->Texture.Current3D = ctx->Shared->Default3D;
+
+      ctx->Texture.AnyDirty = GL_FALSE;
+
+      ctx->Texture.SharedPalette = GL_FALSE;
+      ctx->Texture.Palette[0] = 255;
+      ctx->Texture.Palette[1] = 255;
+      ctx->Texture.Palette[2] = 255;
+      ctx->Texture.Palette[3] = 255;
+      ctx->Texture.PaletteSize = 1;
+      ctx->Texture.PaletteIntFormat = GL_RGBA;
+      ctx->Texture.PaletteFormat = GL_RGBA;
 
       /* Transformation group */
       ctx->Transform.MatrixMode = GL_MODELVIEW;
@@ -813,18 +929,36 @@ static void initialize_context( GLcontext *ctx )
       }
       ctx->FastDrawPixels = ctx->Visual->RGBAflag && ctx->Visual->EightBitColor;
 
+#if JUNK
       ctx->PointsFunc = NULL;
       ctx->LineFunc = NULL;
       ctx->TriangleFunc = NULL;
+#endif
 
+      ctx->DirectContext = GL_TRUE;  /* XXXX temp */
+
+      /* Display list */
       ctx->CallDepth = 0;
       ctx->ExecuteFlag = GL_TRUE;
       ctx->CompileFlag = GL_FALSE;
+      ctx->CurrentListPtr = NULL;
+      ctx->CurrentBlock = NULL;
+      ctx->CurrentListNum = 0;
+      ctx->CurrentPos = 0;
 
       ctx->ErrorValue = GL_NO_ERROR;
 
       /* For debug/development only */
       ctx->NoRaster = getenv("MESA_NO_RASTER") ? GL_TRUE : GL_FALSE;
+
+      /* Dither disable */
+      ctx->NoDither = getenv("MESA_NO_DITHER") ? GL_TRUE : GL_FALSE;
+      if (ctx->NoDither) {
+         if (getenv("MESA_DEBUG")) {
+            fprintf(stderr, "MESA_NO_DITHER set - dithering disabled\n");
+         }
+         ctx->Color.DitherFlag = GL_FALSE;
+      }
    }
 }
 
@@ -839,8 +973,12 @@ static void initialize_context( GLcontext *ctx )
  *         stencil_bits - requested minimum bits per stencil buffer value
  *         accum_bits - requested minimum bits per accum buffer component
  *         index_bits - number of bits per pixel if rgb_flag==GL_FALSE
- *         red, green, blue_scale - float-to-int color scaling
+ *         red/green/blue_scale - float-to-int color scaling
  *         alpha_scale - ignored if alpha_flag is true
+ *         red/green/blue/alpha_bits - number of bits per color component
+ *                                     in frame buffer for RGB(A) mode.
+ * Return:  pointer to new GLvisual or NULL if requested parameters can't
+ *          be met.
  */
 GLvisual *gl_create_visual( GLboolean rgb_flag,
                             GLboolean alpha_flag,
@@ -852,7 +990,11 @@ GLvisual *gl_create_visual( GLboolean rgb_flag,
                             GLfloat red_scale,
                             GLfloat green_scale,
                             GLfloat blue_scale,
-                            GLfloat alpha_scale )
+                            GLfloat alpha_scale,
+                            GLint red_bits,
+                            GLint green_bits,
+                            GLint blue_bits,
+                            GLint alpha_bits )
 {
    GLvisual *vis;
 
@@ -898,10 +1040,16 @@ GLvisual *gl_create_visual( GLboolean rgb_flag,
    if (alpha_scale) {
       vis->InvAlphaScale = 1.0F / alpha_scale;
    }
-   vis->IndexBits     = index_bits;
-   vis->DepthBits     = depth_bits;
-   vis->AccumBits     = accum_bits;
-   vis->StencilBits   = stencil_bits;
+
+   vis->RedBits   = red_bits;
+   vis->GreenBits = green_bits;
+   vis->BlueBits  = blue_bits;
+   vis->AlphaBits = alpha_flag ? 8*sizeof(GLubyte) : alpha_bits;
+
+   vis->IndexBits   = index_bits;
+   vis->DepthBits   = (depth_bits>0) ? 8*sizeof(GLdepth) : 0;
+   vis->AccumBits   = (accum_bits>0) ? 8*sizeof(GLaccum) : 0;
+   vis->StencilBits = (stencil_bits>0) ? 8*sizeof(GLstencil) : 0;
 
    if (red_scale==255.0F && green_scale==255.0F
        && blue_scale==255.0F && alpha_scale==255.0F) {
@@ -942,27 +1090,21 @@ static GLboolean alloc_proxy_textures( GLcontext *ctx )
    GLboolean out_of_memory;
    GLint i;
 
-   ctx->Texture.Proxy1D = gl_alloc_texture_object();
+   ctx->Texture.Proxy1D = gl_alloc_texture_object(NULL, 0, 1);
    if (!ctx->Texture.Proxy1D) {
       return GL_FALSE;
    }
 
-   ctx->Texture.Proxy2D = gl_alloc_texture_object();
+   ctx->Texture.Proxy2D = gl_alloc_texture_object(NULL, 0, 2);
    if (!ctx->Texture.Proxy2D) {
-      gl_free_texture_object(ctx->Texture.Proxy1D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy1D);
       return GL_FALSE;
    }
 
-   ctx->Texture.Proxy2D = gl_alloc_texture_object();
-   if (!ctx->Texture.Proxy2D) {
-      gl_free_texture_object(ctx->Texture.Proxy1D);
-      return GL_FALSE;
-   }
-
-   ctx->Texture.Proxy3D = gl_alloc_texture_object();
+   ctx->Texture.Proxy3D = gl_alloc_texture_object(NULL, 0, 3);
    if (!ctx->Texture.Proxy3D) {
-      gl_free_texture_object(ctx->Texture.Proxy1D);
-      gl_free_texture_object(ctx->Texture.Proxy2D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy1D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy2D);
       return GL_FALSE;
    }
 
@@ -989,9 +1131,9 @@ static GLboolean alloc_proxy_textures( GLcontext *ctx )
             gl_free_texture_image(ctx->Texture.Proxy3D->Image[i]);
          }
       }
-      gl_free_texture_object(ctx->Texture.Proxy1D);
-      gl_free_texture_object(ctx->Texture.Proxy2D);
-      gl_free_texture_object(ctx->Texture.Proxy3D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy1D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy2D);
+      gl_free_texture_object(NULL, ctx->Texture.Proxy3D);
       return GL_FALSE;
    }
    else {
@@ -1022,33 +1164,32 @@ GLcontext *gl_create_context( GLvisual *visual,
    assert( sizeof(GLushort) >= 2 );
    assert( sizeof(GLuint) >= 4 );
 
+   /* misc one-time initializations */
+   gl_init_math();
+   gl_init_lists();
+   gl_init_eval();
+
    ctx = (GLcontext *) calloc( 1, sizeof(GLcontext) );
    if (!ctx) {
       return NULL;
    }
 
    ctx->DriverCtx = driver_ctx;
-
    ctx->Visual = visual;
    ctx->Buffer = NULL;
 
-   gl_init_lists();
-   gl_init_eval(ctx);
-
-   ctx->VB = (struct vertex_buffer *) malloc(sizeof(struct vertex_buffer));
+   ctx->VB = gl_alloc_vb();
    if (!ctx->VB) {
       free( ctx );
       return NULL;
    }
-   gl_init_vb( ctx->VB );
 
-   ctx->PB = (struct pixel_buffer *) malloc(sizeof(struct pixel_buffer));
+   ctx->PB = gl_alloc_pb();
    if (!ctx->PB) {
       free( ctx->VB );
       free( ctx );
       return NULL;
    }
-   PB_INIT( ctx->PB, GL_BITMAP );
 
    if (share_list) {
       /* share the group of display lists of another context */
@@ -1083,7 +1224,7 @@ GLcontext *gl_create_context( GLvisual *visual,
 
 #ifdef GL_VERSION_1_1
    if (!alloc_proxy_textures(ctx)) {
-      free_shared_state(ctx->Shared);
+      free_shared_state(ctx, ctx->Shared);
       free(ctx->VB);
       free(ctx->PB);
       free(ctx);
@@ -1099,14 +1240,12 @@ GLcontext *gl_create_context( GLvisual *visual,
 
 
 
-
 /*
  * Destroy a gl_context structure.
  */
 void gl_destroy_context( GLcontext *ctx )
 {
    if (ctx) {
-      GLint i;
 
 #ifdef PROFILE
       if (getenv("MESA_PROFILE")) {
@@ -1121,18 +1260,13 @@ void gl_destroy_context( GLcontext *ctx )
       assert(ctx->Shared->RefCount>=0);
       if (ctx->Shared->RefCount==0) {
 	 /* free shared state */
-	 free_shared_state( ctx->Shared );
+	 free_shared_state( ctx, ctx->Shared );
       }
 
-      /* Free proxy texture info */
-      for (i=0;i<MAX_TEXTURE_LEVELS;i++) {
-         free( ctx->Texture.Proxy1D->Image[i] );
-         free( ctx->Texture.Proxy2D->Image[i] );
-         free( ctx->Texture.Proxy3D->Image[i] );
-      }
-      free( ctx->Texture.Proxy1D );
-      free( ctx->Texture.Proxy2D );
-      free( ctx->Texture.Proxy3D );
+      /* Free proxy texture objects */
+      gl_free_texture_object( NULL, ctx->Texture.Proxy1D );
+      gl_free_texture_object( NULL, ctx->Texture.Proxy2D );
+      gl_free_texture_object( NULL, ctx->Texture.Proxy3D );
 
       free( (void *) ctx );
 
@@ -1147,7 +1281,6 @@ void gl_destroy_context( GLcontext *ctx )
 
 
 
-
 /*
  * Create a new framebuffer.  A GLframebuffer is a struct which
  * encapsulates the depth, stencil and accum buffers and related
@@ -1198,8 +1331,6 @@ void gl_destroy_framebuffer( GLframebuffer *buffer )
 
 
 
-
-
 /*
  * Set the current context, binding the given frame buffer to the context.
  */
@@ -1225,10 +1356,18 @@ void gl_make_current( GLcontext *ctx, GLframebuffer *buffer )
 }
 
 
+/*
+ * Return current context handle.
+ */
+GLcontext *gl_get_current_context( void )
+{
+#ifdef THREADS
+   return gl_get_thread_context();
+#else
+   return CC;
+#endif
+}
 
-/**********************************************************************/
-/*****                Miscellaneous functions                     *****/
-/**********************************************************************/
 
 
 /*
@@ -1237,7 +1376,7 @@ void gl_make_current( GLcontext *ctx, GLframebuffer *buffer )
  *         dst - destination context
  *         mask - bitwise OR of GL_*_BIT flags
  */
-void gl_copy_context( GLcontext *src, GLcontext *dst, GLuint mask )
+void gl_copy_context( const GLcontext *src, GLcontext *dst, GLuint mask )
 {
    if (mask & GL_ACCUM_BUFFER_BIT) {
       MEMCPY( &dst->Accum, &src->Accum, sizeof(struct gl_accum_attrib) );
@@ -1282,7 +1421,13 @@ void gl_copy_context( GLcontext *src, GLcontext *dst, GLuint mask )
       MEMCPY( &dst->Polygon, &src->Polygon, sizeof(struct gl_polygon_attrib) );
    }
    if (mask & GL_POLYGON_STIPPLE_BIT) {
-      MEMCPY( &dst->PolygonStipple, &src->PolygonStipple, 32*sizeof(GLuint) );
+      /* Use loop instead of MEMCPY due to problem with Portland Group's
+       * C compiler.  Reported by John Stone.
+       */
+      int i;
+      for (i=0;i<32;i++) {
+         dst->PolygonStipple[i] = src->PolygonStipple[i];
+      }
    }
    if (mask & GL_SCISSOR_BIT) {
       MEMCPY( &dst->Scissor, &src->Scissor, sizeof(struct gl_scissor_attrib) );
@@ -1303,11 +1448,36 @@ void gl_copy_context( GLcontext *src, GLcontext *dst, GLuint mask )
 
 
 
+/*
+ * Someday a GLS library or OpenGL-like debugger may call this function
+ * to register it's own set of API entry points.
+ * Input: ctx - the context to set API pointers for
+ *        api - if NULL, restore original API pointers
+ *              else, set API function table to this table.
+ */
+void gl_set_api_table( GLcontext *ctx, const struct gl_api_table *api )
+{
+   if (api) {
+      MEMCPY( &ctx->API, api, sizeof(struct gl_api_table) );
+   }
+   else {
+      MEMCPY( &ctx->API, &ctx->Exec, sizeof(struct gl_api_table) );
+   }
+}
+
+
+
+
+/**********************************************************************/
+/*****                Miscellaneous functions                     *****/
+/**********************************************************************/
+
+
 /*
  * This function is called when the Mesa user has stumbled into a code
  * path which may not be implemented fully or correctly.
  */
-void gl_problem( GLcontext *ctx, const char *s )
+void gl_problem( const GLcontext *ctx, const char *s )
 {
    fprintf( stderr, "Mesa implementation error: %s\n", s );
    fprintf( stderr, "Report to Mesa author.\n" );
@@ -1320,7 +1490,7 @@ void gl_problem( GLcontext *ctx, const char *s )
  * something illogical or if there's likely a bug in their program
  * (like enabled depth testing without a depth buffer).
  */
-void gl_warning( GLcontext *ctx, const char *s )
+void gl_warning( const GLcontext *ctx, const char *s )
 {
    GLboolean debug;
 #ifdef DEBUG
@@ -1429,6 +1599,8 @@ void gl_ResizeBuffersMESA( GLcontext *ctx )
    GLint newsize;
    GLuint buf_width, buf_height;
    
+   ctx->NewState |= NEW_ALL;   /* just to be safe */
+
    /* ask device driver for size of output buffer */
    (*ctx->Driver.GetBufferSize)( ctx, &buf_width, &buf_height );
 
@@ -1443,7 +1615,8 @@ void gl_ResizeBuffersMESA( GLcontext *ctx )
    if (newsize && ctx->Visual->DepthBits>0) {
       /* reallocate depth buffer */
       (*ctx->Driver.AllocDepthBuffer)( ctx );
-      (*ctx->Driver.ClearDepthBuffer)( ctx );
+      /*** if scissoring enabled then clearing can cause a problem ***/
+      /***(*ctx->Driver.ClearDepthBuffer)( ctx );***/
    }
    if (newsize && ctx->Visual->StencilBits>0) {
       /* reallocate stencil buffer */
@@ -1576,7 +1749,7 @@ static void update_pixel_masking( GLcontext *ctx )
 
 
 /*
- * Recompute the value of ctx->RasterMask, ctx->ClipMask, etc. according to
+ * Recompute the value of ctx->RasterMask, etc. according to
  * the current context.
  */
 static void update_rasterflags( GLcontext *ctx )
@@ -1616,7 +1789,15 @@ static void update_rasterflags( GLcontext *ctx )
    else if (!ctx->Visual->RGBAflag && ctx->Color.IndexMask==0) {
       ctx->RasterMask |= NO_DRAW_BIT;
    }
+}
 
+
+/*
+ * Recompute the value of ctx->ClipMask according to the current context.
+ * ClipMask depends on Texturing and Lighting.
+ */
+static void update_clipmask(GLcontext *ctx)
+{
    /* Recompute ClipMask (what has to be interpolated when clipping) */
    ctx->ClipMask = 0;
    if (ctx->Texture.Enabled) {
@@ -1636,6 +1817,26 @@ static void update_rasterflags( GLcontext *ctx )
 	 }
       }
    }
+ 
+   switch(ctx->ClipMask) {
+      case CLIP_FCOLOR_BIT|CLIP_TEXTURE_BIT:
+         if(ctx->VB->TexCoordSize==2)
+            ctx->ClipInterpAuxFunc = interpolate_aux_color_tex2;
+         else
+            ctx->ClipInterpAuxFunc = interpolate_aux;
+         break;
+      case CLIP_TEXTURE_BIT:
+         if(ctx->VB->TexCoordSize==2)
+            ctx->ClipInterpAuxFunc = interpolate_aux_tex2;
+         else
+            ctx->ClipInterpAuxFunc = interpolate_aux;
+         break;
+      case CLIP_FCOLOR_BIT:
+         ctx->ClipInterpAuxFunc = interpolate_aux_color;
+         break;
+      default:
+         ctx->ClipInterpAuxFunc = interpolate_aux;
+   }
 }
 
 
@@ -1651,27 +1852,18 @@ void gl_update_state( GLcontext *ctx )
       update_pixel_logic(ctx);
       update_pixel_masking(ctx);
       update_rasterflags(ctx);
+      if (ctx->Driver.Dither) {
+         (*ctx->Driver.Dither)( ctx, ctx->Color.DitherFlag );
+      }
+   }
+
+   if (ctx->NewState & (NEW_RASTER_OPS | NEW_LIGHTING)) {
+      update_clipmask(ctx);
    }
 
    if (ctx->NewState & NEW_LIGHTING) {
       gl_update_lighting(ctx);
-      if (ctx->Light.ColorMaterialEnabled) {
-         ctx->Exec.Color4f = gl_ColorMat4f;
-         ctx->Exec.Color4ub = gl_ColorMat4ub;
-      }
-      else {
-         ctx->Exec.Color4f = gl_Color4f;
-         if (ctx->Visual->EightBitColor) {
-            ctx->Exec.Color4ub = gl_Color4ub8bit;
-         }
-         else {
-            ctx->Exec.Color4ub = gl_Color4ub;
-         }
-      }
-      if (!ctx->CompileFlag) {
-         ctx->API.Color4f = ctx->Exec.Color4f;
-         ctx->API.Color4ub = ctx->Exec.Color4ub;
-      }
+      gl_set_color_function(ctx);
    }
 
    if (ctx->NewState & NEW_TEXTURING) {
@@ -1680,11 +1872,12 @@ void gl_update_state( GLcontext *ctx )
 
    if (ctx->NewState & (NEW_LIGHTING | NEW_TEXTURING)) {
       /* Check if normal vectors are needed */
-      if (ctx->Light.Enabled
-          || (ctx->Texture.GenModeS==GL_SPHERE_MAP
+      GLboolean sphereGen = ctx->Texture.Enabled
+         && ((ctx->Texture.GenModeS==GL_SPHERE_MAP
               && (ctx->Texture.TexGenEnabled & S_BIT))
-          || (ctx->Texture.GenModeT==GL_SPHERE_MAP
-              && (ctx->Texture.TexGenEnabled & T_BIT))) {
+             || (ctx->Texture.GenModeT==GL_SPHERE_MAP
+                 && (ctx->Texture.TexGenEnabled & T_BIT)));
+      if (ctx->Light.Enabled || sphereGen) {
          ctx->NeedNormals = GL_TRUE;
       }
       else {
@@ -1725,7 +1918,7 @@ void gl_update_state( GLcontext *ctx )
       }
    }
 
-   if (ctx->NewState & NEW_RASTER_OPS) {
+   if (ctx->NewState & NEW_POLYGON) {
       /* Setup CullBits bitmask */
       ctx->Polygon.CullBits = 0;
       if (ctx->Polygon.CullFlag) {
@@ -1748,7 +1941,7 @@ void gl_update_state( GLcontext *ctx )
       ctx->PolygonZoffset = 0.0;
    }
 
-   if (ctx->NewState & (NEW_RASTER_OPS | NEW_LIGHTING)) {
+   if (ctx->NewState & (NEW_POLYGON | NEW_LIGHTING)) {
       /* Determine if we can directly call the triangle rasterizer */
       if (   ctx->Polygon.Unfilled
           || ctx->Polygon.OffsetAny
@@ -1810,6 +2003,13 @@ void gl_update_state( GLcontext *ctx )
       ctx->Driver.ReadDepthSpanFloat = gl_read_depth_span_float;
       ctx->Driver.ReadDepthSpanInt = gl_read_depth_span_int;
    }
+
+   ctx->Driver.PointsFunc = NULL;
+   ctx->Driver.LineFunc = NULL;
+   ctx->Driver.TriangleFunc = NULL;
+   ctx->Driver.QuadFunc = NULL;
+   ctx->Driver.RectFunc = NULL;
+
    if (ctx->Driver.UpdateState) {
       (*ctx->Driver.UpdateState)(ctx);
    }
@@ -1817,6 +2017,7 @@ void gl_update_state( GLcontext *ctx )
    gl_set_point_function(ctx);
    gl_set_line_function(ctx);
    gl_set_triangle_function(ctx);
+   gl_set_quad_function(ctx);
 
    gl_set_vertex_function(ctx);
 
