@@ -1,8 +1,8 @@
-/* $Id: depth.c,v 1.8 1997/02/27 19:58:52 brianp Exp $ */
+/* $Id: depth.c,v 1.11 1997/07/24 01:24:45 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.4
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,15 @@
 
 /*
  * $Log: depth.c,v $
+ * Revision 1.11  1997/07/24 01:24:45  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.10  1997/05/28 03:24:22  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.9  1997/04/20 19:54:15  brianp
+ * replaced abort() with gl_problem()
+ *
  * Revision 1.8  1997/02/27 19:58:52  brianp
  * don't try to clear depth buffer if there isn't one
  *
@@ -50,13 +59,14 @@
  */
 
 
-
 /*
  * Depth buffer functions
  */
 
 
-
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <stdlib.h>
 #include <string.h>
 #include "context.h"
@@ -64,6 +74,7 @@
 #include "dlist.h"
 #include "macros.h"
 #include "types.h"
+#endif
 
 
 
@@ -394,7 +405,7 @@ GLuint gl_depth_test_span_generic( GLcontext* ctx,
 	 }
 	 break;
       default:
-         abort();
+         gl_problem(ctx, "Bad depth func in gl_depth_test_span_generic");
    } /*switch*/
 
    return passed;
@@ -703,7 +714,7 @@ void gl_depth_test_pixels_generic( GLcontext* ctx,
 	 }
 	 break;
       default:
-         abort();
+         gl_problem(ctx, "Bad depth func in gl_depth_test_pixels_generic");
    } /*switch*/
 }
 
