@@ -1,8 +1,8 @@
-/* $Id: dlist.h,v 1.3 1997/02/09 18:50:18 brianp Exp $ */
+/* $Id: dlist.h,v 1.15 1997/12/06 18:06:50 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,42 @@
 
 /*
  * $Log: dlist.h,v $
+ * Revision 1.15  1997/12/06 18:06:50  brianp
+ * moved several static display list vars into GLcontext
+ *
+ * Revision 1.14  1997/10/29 01:29:09  brianp
+ * added GL_EXT_point_parameters extension from Daniel Barrero
+ *
+ * Revision 1.13  1997/09/27 00:13:44  brianp
+ * added GL_EXT_paletted_texture extension
+ *
+ * Revision 1.12  1997/06/20 01:57:57  brianp
+ * added gl_save_Color4ubv()
+ *
+ * Revision 1.11  1997/06/06 02:59:12  brianp
+ * added gl_destroy_list()
+ *
+ * Revision 1.10  1997/04/24 01:50:53  brianp
+ * optimized glColor3f, glColor3fv, glColor4fv
+ *
+ * Revision 1.9  1997/04/21 01:21:33  brianp
+ * added gl_save_Rectf()
+ *
+ * Revision 1.8  1997/04/20 16:18:15  brianp
+ * added glOrtho and glFrustum API pointers
+ *
+ * Revision 1.7  1997/04/16 23:55:33  brianp
+ * added optimized glTexCoord2f code
+ *
+ * Revision 1.6  1997/04/14 22:18:23  brianp
+ * added optimized glVertex3fv code
+ *
+ * Revision 1.5  1997/04/07 02:58:49  brianp
+ * added gl_save_Vertex[23]f() and related code
+ *
+ * Revision 1.4  1997/04/01 04:26:02  brianp
+ * added code for glLoadIdentity(), changed #include's
+ *
  * Revision 1.3  1997/02/09 18:50:18  brianp
  * added GL_EXT_texture3D support
  *
@@ -45,8 +81,7 @@
 
 extern void gl_init_lists( void );
 
-extern GLint gl_list_index( void );
-
+extern void gl_destroy_list( GLcontext *ctx, GLuint list );
 
 
 
@@ -114,17 +149,32 @@ extern void gl_save_ClearStencil( GLcontext *ctx, GLint s );
 extern void gl_save_ClipPlane( GLcontext *ctx,
                                GLenum plane, const GLfloat *equ );
 
+extern void gl_save_Color3f( GLcontext *ctx, GLfloat r, GLfloat g, GLfloat b );
+
+extern void gl_save_Color3fv( GLcontext *ctx, const GLfloat *c );
+
 extern void gl_save_Color4f( GLcontext *ctx, GLfloat r, GLfloat g,
                              GLfloat b, GLfloat a );
 
+extern void gl_save_Color4fv( GLcontext *ctx, const GLfloat *c );
+
 extern void gl_save_Color4ub( GLcontext *ctx, GLubyte r, GLubyte g,
                               GLubyte b, GLubyte a );
 
+extern void gl_save_Color4ubv( GLcontext *ctx, const GLubyte *c );
+
 extern void gl_save_ColorMask( GLcontext *ctx, GLboolean red, GLboolean green,
 			       GLboolean blue, GLboolean alpha );
 
 extern void gl_save_ColorMaterial( GLcontext *ctx, GLenum face, GLenum mode );
 
+extern void gl_save_ColorTable( GLcontext *ctx, GLenum target,
+                                GLenum internalFormat,
+                                struct gl_image *table );
+
+extern void gl_save_ColorSubTable( GLcontext *ctx, GLenum target,
+                                   GLsizei start, struct gl_image *data );
+
 extern void gl_save_CopyPixels( GLcontext *ctx, GLint x, GLint y,
 				GLsizei width, GLsizei height, GLenum type );
 
@@ -226,6 +276,8 @@ extern void gl_save_LineStipple( GLcontext *ctx, GLint factor,
 
 extern void gl_save_ListBase( GLcontext *ctx, GLuint base );
 
+extern void gl_save_LoadIdentity( GLcontext *ctx );
+
 extern void gl_save_LoadMatrixf( GLcontext *ctx, const GLfloat *m );
 
 extern void gl_save_LoadName( GLcontext *ctx, GLuint name );
@@ -264,6 +316,10 @@ extern void gl_save_Normal3fv( GLcontext *ctx, const GLfloat n[3] );
 extern void gl_save_Normal3f( GLcontext *ctx,
                               GLfloat nx, GLfloat ny, GLfloat nz );
 
+extern void gl_save_Ortho( GLcontext *ctx, GLdouble left, GLdouble right,
+                           GLdouble bottom, GLdouble top,
+                           GLdouble nearval, GLdouble farval );
+
 extern void gl_save_PassThrough( GLcontext *ctx, GLfloat token );
 
 extern void gl_save_PixelMapfv( GLcontext *ctx, GLenum map, GLint mapsize,
@@ -275,6 +331,9 @@ extern void gl_save_PixelTransferf( GLcontext *ctx,
 extern void gl_save_PixelZoom( GLcontext *ctx,
                                GLfloat xfactor, GLfloat yfactor );
 
+extern void gl_save_PointParameterfvEXT( GLcontext *ctx, GLenum pname,
+                                         const GLfloat *params );
+
 extern void gl_save_PointSize( GLcontext *ctx, GLfloat size );
 
 extern void gl_save_PolygonMode( GLcontext *ctx, GLenum face, GLenum mode );
@@ -305,6 +364,9 @@ extern void gl_save_RasterPos4f( GLcontext *ctx,
 
 extern void gl_save_ReadBuffer( GLcontext *ctx, GLenum mode );
 
+extern void gl_save_Rectf( GLcontext *ctx,
+                           GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2 );
+
 extern void gl_save_Rotatef( GLcontext *ctx, GLfloat angle,
                              GLfloat x, GLfloat y, GLfloat z );
 
@@ -323,6 +385,8 @@ extern void gl_save_StencilMask( GLcontext *ctx, GLuint mask );
 extern void gl_save_StencilOp( GLcontext *ctx,
                                GLenum fail, GLenum zfail, GLenum zpass );
 
+extern void gl_save_TexCoord2f( GLcontext *ctx, GLfloat s, GLfloat t );
+
 extern void gl_save_TexCoord4f( GLcontext *ctx, GLfloat s, GLfloat t,
                                 GLfloat r, GLfloat q );
 
@@ -378,9 +442,17 @@ extern void gl_save_TexSubImage3DEXT( GLcontext *ctx,
 extern void gl_save_Translatef( GLcontext *ctx,
                                 GLfloat x, GLfloat y, GLfloat z );
 
+extern void gl_save_Vertex2f( GLcontext *ctx,
+                              GLfloat x, GLfloat y );
+
+extern void gl_save_Vertex3f( GLcontext *ctx,
+                              GLfloat x, GLfloat y, GLfloat z );
+
 extern void gl_save_Vertex4f( GLcontext *ctx,
                               GLfloat x, GLfloat y, GLfloat z, GLfloat w );
 
+extern void gl_save_Vertex3fv( GLcontext *ctx, const GLfloat *v );
+
 extern void gl_save_Viewport( GLcontext *ctx, GLint x, GLint y,
                               GLsizei width, GLsizei height );
 
