@@ -1,8 +1,8 @@
-/* $Id: drawpix.c,v 1.6 1997/02/10 20:26:57 brianp Exp $ */
+/* $Id: drawpix.c,v 1.16 1998/02/03 23:45:02 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.4
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,36 @@
 
 /*
  * $Log: drawpix.c,v $
+ * Revision 1.16  1998/02/03 23:45:02  brianp
+ * added casts to prevent warnings with Amiga StormC compiler
+ *
+ * Revision 1.15  1997/08/26 02:10:11  brianp
+ * another bug fix related to glPixelStore()
+ *
+ * Revision 1.14  1997/08/24 02:31:23  brianp
+ * bug fix: glPixelStore() params weren't ignored during display list execute
+ *
+ * Revision 1.13  1997/07/24 01:25:01  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.12  1997/06/20 02:18:23  brianp
+ * replaced Current.IntColor with Current.ByteColor
+ *
+ * Revision 1.11  1997/05/28 03:24:22  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.10  1997/04/24 00:18:56  brianp
+ * added some missing UNDEFARRAY()s.  Reported by Randy Frank.
+ *
+ * Revision 1.9  1997/04/20 20:28:49  brianp
+ * replaced abort() with gl_problem()
+ *
+ * Revision 1.8  1997/04/11 23:24:25  brianp
+ * move call to gl_update_state() into gl_DrawPixels() from drawpixels()
+ *
+ * Revision 1.7  1997/03/18 01:55:47  brianp
+ * only generate feedback/selection if raster position is valid
+ *
  * Revision 1.6  1997/02/10 20:26:57  brianp
  * fixed memory leak in quickdraw_rgb()
  *
@@ -44,6 +74,9 @@
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <stdlib.h>
 #include <string.h>
 #include "context.h"
@@ -55,7 +88,7 @@
 #include "span.h"
 #include "stencil.h"
 #include "types.h"
-
+#endif
 
 
 
@@ -456,7 +489,8 @@ static void draw_depth_pixels( GLcontext* ctx, GLsizei width, GLsizei height,
                }
                break;
             default:
-               abort();
+               gl_problem(ctx, "Bad type in draw_depth_pixels");
+               return;
          }
 
          /* apply depth scale and bias */
@@ -637,7 +671,8 @@ static void draw_color_pixels( GLcontext* ctx,
 	    components = 4;
 	    break;
          default:
-            abort();
+            gl_problem(ctx, "Bad type in draw_color_pixels");
+            return;
       }
 
       /* process the image row by row */
@@ -824,6 +859,10 @@ static void draw_color_pixels( GLcontext* ctx,
                                  red, green, blue, alpha, GL_BITMAP );
          }
 
+         UNDEFARRAY(rf);
+         UNDEFARRAY(gf);
+         UNDEFARRAY(bf);
+         UNDEFARRAY(af);
          UNDEFARRAY(red);
          UNDEFARRAY(green);
          UNDEFARRAY(blue);
@@ -911,10 +950,6 @@ static void drawpixels( GLcontext* ctx, GLsizei width, GLsizei height,
       return;
    }
 
-   if (ctx->NewState) {
-      gl_update_state(ctx);
-   }
-
    if (ctx->RenderMode==GL_RENDER) {
       if (!ctx->Current.RasterPosValid) {
 	 return;
@@ -944,25 +979,29 @@ static void drawpixels( GLcontext* ctx, GLsizei width, GLsizei height,
       }
    }
    else if (ctx->RenderMode==GL_FEEDBACK) {
-      GLfloat color[4], texcoord[4], invq;
-      color[0] = ctx->Current.IntColor[0] * ctx->Visual->InvRedScale;
-      color[1] = ctx->Current.IntColor[1] * ctx->Visual->InvGreenScale;
-      color[2] = ctx->Current.IntColor[2] * ctx->Visual->InvBlueScale;
-      color[3] = ctx->Current.IntColor[3] * ctx->Visual->InvAlphaScale;
-      invq = 1.0F / ctx->Current.TexCoord[3];
-      texcoord[0] = ctx->Current.TexCoord[0] * invq;
-      texcoord[1] = ctx->Current.TexCoord[1] * invq;
-      texcoord[2] = ctx->Current.TexCoord[2] * invq;
-      texcoord[3] = ctx->Current.TexCoord[3];
-      FEEDBACK_TOKEN( ctx, (GLfloat) GL_DRAW_PIXEL_TOKEN );
-      gl_feedback_vertex( ctx, ctx->Current.RasterPos[0],
-			  ctx->Current.RasterPos[1],
-			  ctx->Current.RasterPos[2],
-			  ctx->Current.RasterPos[3],
-			  color, ctx->Current.Index, texcoord );
+      if (ctx->Current.RasterPosValid) {
+         GLfloat color[4], texcoord[4], invq;
+         color[0] = ctx->Current.ByteColor[0] * ctx->Visual->InvRedScale;
+         color[1] = ctx->Current.ByteColor[1] * ctx->Visual->InvGreenScale;
+         color[2] = ctx->Current.ByteColor[2] * ctx->Visual->InvBlueScale;
+         color[3] = ctx->Current.ByteColor[3] * ctx->Visual->InvAlphaScale;
+         invq = 1.0F / ctx->Current.TexCoord[3];
+         texcoord[0] = ctx->Current.TexCoord[0] * invq;
+         texcoord[1] = ctx->Current.TexCoord[1] * invq;
+         texcoord[2] = ctx->Current.TexCoord[2] * invq;
+         texcoord[3] = ctx->Current.TexCoord[3];
+         FEEDBACK_TOKEN( ctx, (GLfloat) (GLint) GL_DRAW_PIXEL_TOKEN );
+         gl_feedback_vertex( ctx, ctx->Current.RasterPos[0],
+                             ctx->Current.RasterPos[1],
+                             ctx->Current.RasterPos[2],
+                             ctx->Current.RasterPos[3],
+                             color, ctx->Current.Index, texcoord );
+      }
    }
    else if (ctx->RenderMode==GL_SELECT) {
-      gl_update_hitflag( ctx, ctx->Current.RasterPos[2] );
+      if (ctx->Current.RasterPosValid) {
+         gl_update_hitflag( ctx, ctx->Current.RasterPos[2] );
+      }
    }
 }
 
@@ -981,6 +1020,10 @@ void gl_DrawPixels( GLcontext* ctx, GLsizei width, GLsizei height,
       return;
    }
 
+   if (ctx->NewState) {
+      gl_update_state(ctx);
+   }
+
    /* Let the device driver take a crack at glDrawPixels */
    if (!ctx->CompileFlag && ctx->Driver.DrawPixels) {
       GLint x = (GLint) (ctx->Current.RasterPos[0] + 0.5F);
@@ -994,7 +1037,7 @@ void gl_DrawPixels( GLcontext* ctx, GLsizei width, GLsizei height,
 
    if (format==GL_RGB && type==GL_UNSIGNED_BYTE && ctx->FastDrawPixels
        && !ctx->CompileFlag && ctx->RenderMode==GL_RENDER
-       && ctx->RasterMask==0) {
+       && ctx->RasterMask==0 && ctx->CallDepth==0) {
       /* optimized path */
       if (quickdraw_rgb( ctx, width, height, pixels )) {
          /* success */
@@ -1003,10 +1046,19 @@ void gl_DrawPixels( GLcontext* ctx, GLsizei width, GLsizei height,
    }
 
    /* take the general path */
-   image = gl_unpack_pixels( ctx, width, height, format, type, pixels );
-   if (!image) {
-      gl_error( ctx, GL_OUT_OF_MEMORY, "glDrawPixels" );
-      return;
+
+   /* THIS IS A REAL HACK - FIX IN MESA 2.5
+    * If we're inside glCallList then we don't have to unpack the pixels again.
+    */
+   if (ctx->CallDepth == 0) {
+      image = gl_unpack_pixels( ctx, width, height, format, type, pixels );
+      if (!image) {
+         gl_error( ctx, GL_OUT_OF_MEMORY, "glDrawPixels" );
+         return;
+      }
+   }
+   else {
+      image = (GLvoid *) pixels;
    }
 
    if (ctx->CompileFlag) {
@@ -1016,9 +1068,8 @@ void gl_DrawPixels( GLcontext* ctx, GLsizei width, GLsizei height,
       drawpixels( ctx, width, height, format, type, image );
       if (!ctx->CompileFlag) {
          /* may discard unpacked image now */
-         free( image );
+         if (image!=pixels)
+            free( image );
       }
    }
 }
-
-
