@@ -1,8 +1,8 @@
-/* $Id: enable.c,v 1.13 1997/02/27 19:59:08 brianp Exp $ */
+/* $Id: enable.c,v 1.23 1997/10/29 02:23:54 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,36 @@
 
 /*
  * $Log: enable.c,v $
+ * Revision 1.23  1997/10/29 02:23:54  brianp
+ * added UseGlobalTexturePalette() dd function (David Bucciarelli v20 3dfx)
+ *
+ * Revision 1.22  1997/10/16 01:59:08  brianp
+ * added GL_EXT_shared_texture_palette extension
+ *
+ * Revision 1.21  1997/07/24 01:25:01  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.20  1997/06/20 02:20:32  brianp
+ * replaced Current.IntColor with Current.ByteColor
+ *
+ * Revision 1.19  1997/05/31 16:57:14  brianp
+ * added MESA_NO_RASTER env var support
+ *
+ * Revision 1.18  1997/05/28 03:24:22  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.17  1997/04/24 01:49:30  brianp
+ * call gl_set_color_function() instead of directly setting pointers
+ *
+ * Revision 1.16  1997/04/12 16:54:05  brianp
+ * new NEW_POLYGON state flag
+ *
+ * Revision 1.15  1997/04/12 16:20:27  brianp
+ * removed call to Driver.Dither()
+ *
+ * Revision 1.14  1997/04/02 03:10:36  brianp
+ * changed some #include's
+ *
  * Revision 1.13  1997/02/27 19:59:08  brianp
  * issue a warning if enable depth or stencil test without such a buffer
  *
@@ -65,17 +95,20 @@
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <string.h>
 #include "context.h"
 #include "depth.h"
-#include "draw.h"
 #include "enable.h"
 #include "light.h"
 #include "dlist.h"
 #include "macros.h"
 #include "stencil.h"
 #include "types.h"
-#include "vertex.h"
+#include "vbfill.h"
+#endif
 
 
 
@@ -140,38 +173,21 @@ static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
             ctx->Light.ColorMaterialEnabled = state;
             if (state) {
                GLfloat color[4];
-               color[0] = ctx->Current.IntColor[0] * ctx->Visual->InvRedScale;
-               color[1] = ctx->Current.IntColor[1] * ctx->Visual->InvGreenScale;
-               color[2] = ctx->Current.IntColor[2] * ctx->Visual->InvBlueScale;
-               color[3] = ctx->Current.IntColor[3] * ctx->Visual->InvAlphaScale;
+               color[0] = ctx->Current.ByteColor[0] * ctx->Visual->InvRedScale;
+               color[1] = ctx->Current.ByteColor[1] * ctx->Visual->InvGreenScale;
+               color[2] = ctx->Current.ByteColor[2] * ctx->Visual->InvBlueScale;
+               color[3] = ctx->Current.ByteColor[3] * ctx->Visual->InvAlphaScale;
                /* update material with current color */
                gl_set_material( ctx, ctx->Light.ColorMaterialBitmask, color );
-
-               /* update the Color4f and Color4ub API pointers */
-               ctx->Exec.Color4f = gl_ColorMat4f;
-               ctx->Exec.Color4ub = gl_ColorMat4ub;
-            }
-            else {
-               /* update the Color4f and Color4ub API pointers */
-               ctx->Exec.Color4f = gl_Color4f;
-               if (ctx->Visual->EightBitColor) {
-                  ctx->Exec.Color4ub = gl_Color4ub8bit;
-               }
-               else {
-                  ctx->Exec.Color4ub = gl_Color4ub;
-               }
-            }
-            if (!ctx->CompileFlag) {
-               ctx->API.Color4f = ctx->Exec.Color4f;
-               ctx->API.Color4ub = ctx->Exec.Color4ub;
             }
+            gl_set_color_function(ctx);
             ctx->NewState |= NEW_LIGHTING;
          }
 	 break;
       case GL_CULL_FACE:
          if (ctx->Polygon.CullFlag!=state) {
             ctx->Polygon.CullFlag = state;
-            ctx->NewState |= NEW_RASTER_OPS;
+            ctx->NewState |= NEW_POLYGON;
          }
 	 break;
       case GL_DEPTH_TEST:
@@ -185,11 +201,14 @@ static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
          }
          break;
       case GL_DITHER:
-	 ctx->Color.DitherFlag = state;
-         if (ctx->Driver.Dither) {
-            (*ctx->Driver.Dither)( ctx, state );
+         if (ctx->NoDither) {
+            /* MESA_NO_DITHER env var */
+            state = GL_FALSE;
+         }
+         if (ctx->Color.DitherFlag!=state) {
+            ctx->Color.DitherFlag = state;
+            ctx->NewState |= NEW_RASTER_OPS;
          }
-         ctx->NewState |= NEW_RASTER_OPS;
 	 break;
       case GL_FOG:
 	 if (ctx->Fog.Enabled!=state) {
@@ -316,20 +335,20 @@ static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
       case GL_POLYGON_OFFSET_POINT:
          if (ctx->Polygon.OffsetPoint!=state) {
             ctx->Polygon.OffsetPoint = state;
-            ctx->NewState |= NEW_RASTER_OPS;
+            ctx->NewState |= NEW_POLYGON;
          }
          break;
       case GL_POLYGON_OFFSET_LINE:
          if (ctx->Polygon.OffsetLine!=state) {
             ctx->Polygon.OffsetLine = state;
-            ctx->NewState |= NEW_RASTER_OPS;
+            ctx->NewState |= NEW_POLYGON;
          }
          break;
       case GL_POLYGON_OFFSET_FILL:
       /*case GL_POLYGON_OFFSET_EXT:*/
          if (ctx->Polygon.OffsetFill!=state) {
             ctx->Polygon.OffsetFill = state;
-            ctx->NewState |= NEW_RASTER_OPS;
+            ctx->NewState |= NEW_POLYGON;
          }
          break;
       case GL_SCISSOR_TEST:
@@ -338,6 +357,11 @@ static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
             ctx->NewState |= NEW_RASTER_OPS;
          }
 	 break;
+      case GL_SHARED_TEXTURE_PALETTE_EXT:
+         ctx->Texture.SharedPalette = state;
+         if (ctx->Driver.UseGlobalTexturePalette)
+            (*ctx->Driver.UseGlobalTexturePalette)( ctx, state );
+         break;
       case GL_STENCIL_TEST:
 	 if (state && ctx->Visual->StencilBits==0) {
             gl_warning(ctx, "glEnable(GL_STENCIL_TEST) but no stencil buffer");
@@ -569,6 +593,9 @@ GLboolean gl_IsEnabled( GLcontext* ctx, GLenum cap )
 	 return ctx->Polygon.OffsetFill;
       case GL_SCISSOR_TEST:
 	 return ctx->Scissor.Enabled;
+      case GL_SHARED_TEXTURE_PALETTE_EXT:
+         return ctx->Texture.SharedPalette;
+         break;
       case GL_STENCIL_TEST:
 	 return ctx->Stencil.Enabled;
       case GL_TEXTURE_1D:
