@@ -1,8 +1,8 @@
-/* $Id: feedback.c,v 1.8 1997/03/08 02:03:46 brianp Exp $ */
+/* $Id: feedback.c,v 1.12 1998/02/03 23:45:02 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.4
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,18 @@
 
 /*
  * $Log: feedback.c,v $
+ * Revision 1.12  1998/02/03 23:45:02  brianp
+ * added casts to prevent warnings with Amiga StormC compiler
+ *
+ * Revision 1.11  1997/11/07 03:38:07  brianp
+ * added stdio.h include for SunOS 4.x
+ *
+ * Revision 1.10  1997/07/24 01:25:01  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.9  1997/05/28 03:24:54  brianp
+ * added precompiled header (PCH) support
+ *
  * Revision 1.8  1997/03/08 02:03:46  brianp
  * better error checking
  *
@@ -50,13 +62,17 @@
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <assert.h>
+#include <stdio.h>
 #include "context.h"
 #include "feedback.h"
 #include "dlist.h"
 #include "macros.h"
 #include "types.h"
-
+#endif
 
 
 
@@ -132,7 +148,7 @@ void gl_PassThrough( GLcontext *ctx, GLfloat token )
    }
 
    if (ctx->RenderMode==GL_FEEDBACK) {
-      FEEDBACK_TOKEN( ctx, (GLfloat) GL_PASS_THROUGH_TOKEN );
+      FEEDBACK_TOKEN( ctx, (GLfloat) (GLint) GL_PASS_THROUGH_TOKEN );
       FEEDBACK_TOKEN( ctx, token );
    }
 }
