@@ -1,8 +1,8 @@
-/* $Id: fog.c,v 1.3 1996/09/27 01:26:52 brianp Exp $ */
+/* $Id: fog.c,v 1.9 1997/07/24 01:25:01 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.0
+ * Version:  2.4
  * Copyright (C) 1995-1996  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,24 @@
 
 /*
  * $Log: fog.c,v $
+ * Revision 1.9  1997/07/24 01:25:01  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.8  1997/06/20 04:14:47  brianp
+ * changed color components from GLfixed to GLubyte
+ *
+ * Revision 1.7  1997/05/28 03:24:54  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.6  1997/05/22 03:03:47  brianp
+ * don't apply fog to alpha values
+ *
+ * Revision 1.5  1997/04/20 20:28:49  brianp
+ * replaced abort() with gl_problem()
+ *
+ * Revision 1.4  1996/11/04 02:30:15  brianp
+ * optimized gl_fog_color_pixels() and gl_fog_index_pixels()
+ *
  * Revision 1.3  1996/09/27 01:26:52  brianp
  * added missing default cases to switches
  *
@@ -35,7 +53,9 @@
  */
 
 
-
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <math.h>
 #include <stdlib.h>
 #include "context.h"
@@ -43,6 +63,7 @@
 #include "dlist.h"
 #include "macros.h"
 #include "types.h"
+#endif
 
 
 
@@ -111,15 +132,13 @@ void gl_Fogfv( GLcontext *ctx, GLenum pname, const GLfloat *params )
  * Output:  color - the fogged colors
  */
 void gl_fog_color_vertices( GLcontext *ctx,
-                            GLuint n, GLfloat v[][4], GLfixed color[][4] )
+                            GLuint n, GLfloat v[][4], GLubyte color[][4] )
 {
    GLuint i;
    GLfloat d;
-   GLfloat shift = (GLfloat) (1 << ctx->ColorShift);
-   GLfloat fogr = ctx->Fog.Color[0] * ctx->Visual->RedScale   * shift;
-   GLfloat fogg = ctx->Fog.Color[1] * ctx->Visual->GreenScale * shift;
-   GLfloat fogb = ctx->Fog.Color[2] * ctx->Visual->BlueScale  * shift;
-   GLfloat foga = ctx->Fog.Color[3] * ctx->Visual->AlphaScale * shift;
+   GLfloat fogr = ctx->Fog.Color[0] * ctx->Visual->RedScale;
+   GLfloat fogg = ctx->Fog.Color[1] * ctx->Visual->GreenScale;
+   GLfloat fogb = ctx->Fog.Color[2] * ctx->Visual->BlueScale;
    GLfloat end = ctx->Fog.End;
 
    switch (ctx->Fog.Mode) {
@@ -131,7 +150,6 @@ void gl_fog_color_vertices( GLcontext *ctx,
             color[i][0] = f * color[i][0] + (1.0F-f) * fogr;
             color[i][1] = f * color[i][1] + (1.0F-f) * fogg;
             color[i][2] = f * color[i][2] + (1.0F-f) * fogb;
-            color[i][3] = f * color[i][3] + (1.0F-f) * foga;
          }
 	 break;
       case GL_EXP:
@@ -142,7 +160,6 @@ void gl_fog_color_vertices( GLcontext *ctx,
             color[i][0] = f * color[i][0] + (1.0F-f) * fogr;
             color[i][1] = f * color[i][1] + (1.0F-f) * fogg;
             color[i][2] = f * color[i][2] + (1.0F-f) * fogb;
-            color[i][3] = f * color[i][3] + (1.0F-f) * foga;
          }
 	 break;
       case GL_EXP2:
@@ -154,11 +171,11 @@ void gl_fog_color_vertices( GLcontext *ctx,
             color[i][0] = f * color[i][0] + (1.0F-f) * fogr;
             color[i][1] = f * color[i][1] + (1.0F-f) * fogg;
             color[i][2] = f * color[i][2] + (1.0F-f) * fogb;
-            color[i][3] = f * color[i][3] + (1.0F-f) * foga;
          }
 	 break;
       default:
-         abort();
+         gl_problem(ctx, "Bad fog mode in gl_fog_color_vertices");
+         return;
    }
 }
 
@@ -217,7 +234,8 @@ void gl_fog_index_vertices( GLcontext *ctx,
          }
 	 break;
       default:
-         abort();
+         gl_problem(ctx, "Bad fog mode in gl_fog_index_vertices");
+         return;
    }
 }
 
@@ -235,62 +253,70 @@ void gl_fog_color_pixels( GLcontext *ctx,
                           GLuint n, const GLdepth z[], GLubyte red[],
 			  GLubyte green[], GLubyte blue[], GLubyte alpha[] )
 {
-   /* TODO: optimize case when c = 1.0 and d = 0.0 */
-   GLfloat d = ctx->ProjectionMatrix[14];
    GLfloat c = ctx->ProjectionMatrix[10];
-   GLfloat winz, ndcz, eyez, f;
+   GLfloat d = ctx->ProjectionMatrix[14];
    GLuint i;
 
-   GLint fog_red   = (GLint) (ctx->Fog.Color[0] * ctx->Visual->RedScale);
-   GLint fog_green = (GLint) (ctx->Fog.Color[1] * ctx->Visual->GreenScale);
-   GLint fog_blue  = (GLint) (ctx->Fog.Color[2] * ctx->Visual->BlueScale);
-   GLint fog_alpha = (GLint) (ctx->Fog.Color[3] * ctx->Visual->AlphaScale);
+   GLfloat fog_red   = ctx->Fog.Color[0] * ctx->Visual->RedScale;
+   GLfloat fog_green = ctx->Fog.Color[1] * ctx->Visual->GreenScale;
+   GLfloat fog_blue  = ctx->Fog.Color[2] * ctx->Visual->BlueScale;
+
+   GLfloat tz = ctx->Viewport.Tz;
+   GLfloat szInv = 1.0F / ctx->Viewport.Sz;
 
    switch (ctx->Fog.Mode) {
       case GL_LINEAR:
-         for (i=0;i<n;i++) {
-	    winz = (GLfloat) z[i] / DEPTH_SCALE;
-	    ndcz = (winz - ctx->Viewport.Tz) / ctx->Viewport.Sz;
-	    eyez = -d / (c+ndcz);
-	    if (eyez < 0.0)  eyez = -eyez;
-	    f = (ctx->Fog.End - eyez) / (ctx->Fog.End - ctx->Fog.Start);
-	    f = CLAMP( f, 0.0F, 1.0F );
-	    red[i]   = (GLint) (f * (GLfloat) red[i]   + (1.0F-f) * fog_red);
-	    green[i] = (GLint) (f * (GLfloat) green[i] + (1.0F-f) * fog_green);
-	    blue[i]  = (GLint) (f * (GLfloat) blue[i]  + (1.0F-f) * fog_blue);
-	    alpha[i] = (GLint) (f * (GLfloat) alpha[i] + (1.0F-f) * fog_alpha);
-	 }
+         {
+            GLfloat fogEnd = ctx->Fog.End;
+            GLfloat fogScale = 1.0F / (ctx->Fog.End - ctx->Fog.Start);
+            for (i=0;i<n;i++) {
+               GLfloat ndcz = ((GLfloat) z[i] - tz) * szInv;
+               GLfloat eyez = -d / (c+ndcz);
+               GLfloat f, g;
+               if (eyez < 0.0)  eyez = -eyez;
+               f = (fogEnd - eyez) * fogScale;
+               f = CLAMP( f, 0.0F, 1.0F );
+               g = 1.0F - f;
+               red[i]   = (GLint) (f * (GLfloat) red[i]   + g * fog_red);
+               green[i] = (GLint) (f * (GLfloat) green[i] + g * fog_green);
+               blue[i]  = (GLint) (f * (GLfloat) blue[i]  + g * fog_blue);
+            }
+         }
 	 break;
       case GL_EXP:
 	 for (i=0;i<n;i++) {
-	    winz = (GLfloat) z[i] / DEPTH_SCALE;
-	    ndcz = (winz - ctx->Viewport.Tz) / ctx->Viewport.Sz;
-	    eyez = -d / (c+ndcz);
+	    GLfloat ndcz = ((GLfloat) z[i] - tz) * szInv;
+	    GLfloat eyez = -d / (c+ndcz);
+            GLfloat f, g;
 	    if (eyez < 0.0)  eyez = -eyez;
 	    f = exp( -ctx->Fog.Density * eyez );
 	    f = CLAMP( f, 0.0F, 1.0F );
-	    red[i]   = (GLint) (f * (GLfloat) red[i]   + (1.0F-f) * fog_red);
-	    green[i] = (GLint) (f * (GLfloat) green[i] + (1.0F-f) * fog_green);
-	    blue[i]  = (GLint) (f * (GLfloat) blue[i]  + (1.0F-f) * fog_blue);
-	    alpha[i] = (GLint) (f * (GLfloat) alpha[i] + (1.0F-f) * fog_alpha);
+            g = 1.0F - f;
+            red[i]   = (GLint) (f * (GLfloat) red[i]   + g * fog_red);
+            green[i] = (GLint) (f * (GLfloat) green[i] + g * fog_green);
+            blue[i]  = (GLint) (f * (GLfloat) blue[i]  + g * fog_blue);
 	 }
 	 break;
       case GL_EXP2:
-	 for (i=0;i<n;i++) {
-	    winz = (GLfloat) z[i] / DEPTH_SCALE;
-	    ndcz = (winz - ctx->Viewport.Tz) / ctx->Viewport.Sz;
-	    eyez = -d / (c+ndcz);
-	    if (eyez < 0.0)  eyez = -eyez;
-	    f = exp( -(ctx->Fog.Density*ctx->Fog.Density * eyez*eyez) );
-	    f = CLAMP( f, 0.0F, 1.0F );
-	    red[i]   = (GLint) (f * (GLfloat) red[i]   + (1.0F-f) * fog_red);
-	    green[i] = (GLint) (f * (GLfloat) green[i] + (1.0F-f) * fog_green);
-	    blue[i]  = (GLint) (f * (GLfloat) blue[i]  + (1.0F-f) * fog_blue);
-	    alpha[i] = (GLint) (f * (GLfloat) alpha[i] + (1.0F-f) * fog_alpha);
-	 }
+         {
+            GLfloat negDensitySquared = -ctx->Fog.Density * ctx->Fog.Density;
+            for (i=0;i<n;i++) {
+               GLfloat ndcz = ((GLfloat) z[i] - tz) * szInv;
+               GLfloat eyez = -d / (c+ndcz);
+               GLfloat f, g;
+               if (eyez < 0.0)  eyez = -eyez;
+               f = exp( negDensitySquared * eyez*eyez );
+               f = CLAMP( f, 0.0F, 1.0F );
+               g = 1.0F - f;
+               red[i]   = (GLint) (f * (GLfloat) red[i]   + g * fog_red);
+               green[i] = (GLint) (f * (GLfloat) green[i] + g * fog_green);
+               blue[i]  = (GLint) (f * (GLfloat) blue[i]  + g * fog_blue);
+            }
+         }
 	 break;
       default:
-         abort();
+         gl_problem(ctx, "Bad fog mode in gl_fog_color_pixels");
+         return;
    }
 }
 
@@ -307,29 +333,34 @@ void gl_fog_color_pixels( GLcontext *ctx,
 void gl_fog_index_pixels( GLcontext *ctx,
                           GLuint n, const GLdepth z[], GLuint index[] )
 {
-   /* TODO: optimize case when c = 1.0 and d = 0.0 */
-   GLfloat d = ctx->ProjectionMatrix[14];
    GLfloat c = ctx->ProjectionMatrix[10];
-   GLfloat winz, ndcz, eyez, f;
+   GLfloat d = ctx->ProjectionMatrix[14];
    GLuint i;
 
+   GLfloat tz = ctx->Viewport.Tz;
+   GLfloat szInv = 1.0F / ctx->Viewport.Sz;
+
    switch (ctx->Fog.Mode) {
       case GL_LINEAR:
-         for (i=0;i<n;i++) {
-	    winz = (GLfloat) z[i] / DEPTH_SCALE;
-	    ndcz = (winz - ctx->Viewport.Tz) / ctx->Viewport.Sz;
-	    eyez = -d / (c+ndcz);
-	    if (eyez < 0.0)  eyez = -eyez;
-	    f = (ctx->Fog.End - eyez) / (ctx->Fog.End - ctx->Fog.Start);
-	    f = CLAMP( f, 0.0F, 1.0F );
-	    index[i] = (GLuint) ((GLfloat) index[i] + (1.0F-f) * ctx->Fog.Index);
+         {
+            GLfloat fogEnd = ctx->Fog.End;
+            GLfloat fogScale = 1.0F / (ctx->Fog.End - ctx->Fog.Start);
+            for (i=0;i<n;i++) {
+               GLfloat ndcz = ((GLfloat) z[i] - tz) * szInv;
+               GLfloat eyez = -d / (c+ndcz);
+               GLfloat f;
+               if (eyez < 0.0)  eyez = -eyez;
+               f = (fogEnd - eyez) * fogScale;
+               f = CLAMP( f, 0.0F, 1.0F );
+               index[i] = (GLuint) ((GLfloat) index[i] + (1.0F-f) * ctx->Fog.Index);
+            }
 	 }
 	 break;
       case GL_EXP:
          for (i=0;i<n;i++) {
-	    winz = (GLfloat) z[i] / DEPTH_SCALE;
-	    ndcz = (winz - ctx->Viewport.Tz) / ctx->Viewport.Sz;
-	    eyez = -d / (c+ndcz);
+	    GLfloat ndcz = ((GLfloat) z[i] - tz) * szInv;
+	    GLfloat eyez = -d / (c+ndcz);
+            GLfloat f;
 	    if (eyez < 0.0)  eyez = -eyez;
 	    f = exp( -ctx->Fog.Density * eyez );
 	    f = CLAMP( f, 0.0F, 1.0F );
@@ -337,19 +368,22 @@ void gl_fog_index_pixels( GLcontext *ctx,
 	 }
 	 break;
       case GL_EXP2:
-         for (i=0;i<n;i++) {
-	    winz = (GLfloat) z[i] / DEPTH_SCALE;
-	    ndcz = (winz - ctx->Viewport.Tz) / ctx->Viewport.Sz;
-	    eyez = -d / (c+ndcz);
-	    if (eyez < 0.0)  eyez = -eyez;
-	    f = exp( -(ctx->Fog.Density*ctx->Fog.Density * eyez*eyez) );
-	    f = CLAMP( f, 0.0F, 1.0F );
-	    index[i] = (GLuint) ((GLfloat) index[i] + (1.0F-f) * ctx->Fog.Index);
+         {
+            GLfloat negDensitySquared = -ctx->Fog.Density * ctx->Fog.Density;
+            for (i=0;i<n;i++) {
+               GLfloat ndcz = ((GLfloat) z[i] - tz) * szInv;
+               GLfloat eyez = -d / (c+ndcz);
+               GLfloat f;
+               if (eyez < 0.0)  eyez = -eyez;
+               f = exp( negDensitySquared * eyez*eyez );
+               f = CLAMP( f, 0.0F, 1.0F );
+               index[i] = (GLuint) ((GLfloat) index[i] + (1.0F-f) * ctx->Fog.Index);
+            }
 	 }
 	 break;
       default:
-         abort();
+         gl_problem(ctx, "Bad fog mode in gl_fog_index_pixels");
+         return;
    }
-
 }
 
