@@ -1,8 +1,8 @@
-/* $Id: image.h,v 1.4 1997/02/09 20:05:03 brianp Exp $ */
+/* $Id: image.h,v 1.5 1997/09/27 00:15:39 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,9 @@
 
 /*
  * $Log: image.h,v $
+ * Revision 1.5  1997/09/27 00:15:39  brianp
+ * changed parameters to gl_unpack_image()
+ *
  * Revision 1.4  1997/02/09 20:05:03  brianp
  * new arguments for gl_pixel_addr_in_image()
  *
@@ -68,17 +71,13 @@ extern GLvoid *gl_pixel_addr_in_image( struct gl_pixelstore_attrib *packing,
 extern struct gl_image *gl_unpack_image( GLcontext *ctx,
                                   GLint width, GLint height,
                                   GLenum srcFormat, GLenum srcType,
-                                  GLenum destType,
-                                  const GLvoid *pixels,
-                                  GLboolean interleave );
+                                  const GLvoid *pixels );
 
 
 struct gl_image *gl_unpack_image3D( GLcontext *ctx,
                                     GLint width, GLint height,GLint depth,
                                     GLenum srcFormat, GLenum srcType,
-                                    GLenum destType,
-                                    const GLvoid *pixels,
-                                    GLboolean interleave );
+                                    const GLvoid *pixels );
 
 
 extern void gl_free_image( struct gl_image *image );
