@@ -1,8 +1,8 @@
-/* $Id: osmesa.c,v 1.10 1997/03/06 01:10:29 brianp Exp $ */
+/* $Id: osmesa.c,v 1.20 1998/02/05 00:35:12 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.6
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,37 @@
 
 /*
  * $Log: osmesa.c,v $
+ * Revision 1.20  1998/02/05 00:35:12  brianp
+ * added John Stone's initial thread modifications
+ *
+ * Revision 1.19  1997/07/24 01:24:11  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.18  1997/07/16 03:07:28  brianp
+ * added optimized blended lines (Randy Frank)
+ *
+ * Revision 1.17  1997/07/11 23:08:15  brianp
+ * added OSMesaGetDepthBuffer() function (Randy Frank)
+ *
+ * Revision 1.16  1997/06/20 02:48:54  brianp
+ * changed color components from GLfixed to GLubyte
+ * fixed bug involving rowlength in OSMesaMakeCurrent()
+ *
+ * Revision 1.15  1997/05/28 03:25:43  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.14  1997/05/26 21:15:37  brianp
+ * now pass red/green/blue/alpha bits to gl_create_visual()
+ *
+ * Revision 1.13  1997/03/21 01:57:49  brianp
+ * added RendererString() function
+ *
+ * Revision 1.12  1997/03/16 02:41:20  brianp
+ * did some clean-up in osmesa_setup_DD_pointers()
+ *
+ * Revision 1.11  1997/03/16 02:37:05  brianp
+ * changed line functions to use linetemp.h
+ *
  * Revision 1.10  1997/03/06 01:10:29  brianp
  * added Randy Frank's optimized line drawing code
  *
@@ -62,6 +93,9 @@
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <stdlib.h>
 #include <string.h>
 #include "GL/osmesa.h"
@@ -71,7 +105,7 @@
 #include "matrix.h"
 #include "types.h"
 #include "vb.h"
-#include "bresenhm.h"
+#endif
 
 
 struct osmesa_context {
@@ -84,6 +118,7 @@ struct osmesa_context {
    GLuint pixel;		/* current color index or RGBA pixel value */
    GLuint clearpixel;		/* pixel for clearing the color buffer */
    GLint rowlength;		/* number of pixels per row */
+   GLint userRowLength;		/* user-specified number of pixels per row */
    GLint rshift, gshift;	/* bit shifts for RGBA formats */
    GLint bshift, ashift;
    GLint rind, gind, bind;	/* index offsets for RGBA formats */
@@ -95,8 +130,24 @@ struct osmesa_context {
 
 
 #ifdef THREADS
-   /* A context handle for each thread */
-   /* TODO: an array/table of contexts indexed by thread IDs */
+
+#include "mthreads.h" /* Mesa platform independent threads interface */
+
+static MesaTSD osmesa_ctx_tsd;
+
+static void osmesa_ctx_thread_init() {
+  MesaInitTSD(&osmesa_ctx_tsd);
+}
+
+static OSMesaContext osmesa_get_thread_context( void ) {
+  return (OSMesaContext) MesaGetTSD(&osmesa_ctx_tsd);
+}
+
+static void osmesa_set_thread_context( OSMesaContext ctx ) {
+  MesaSetTSD(&osmesa_ctx_tsd, ctx, osmesa_ctx_thread_init);
+}
+
+
 #else
    /* One current context for address space, all threads */
    static OSMesaContext Current = NULL;
@@ -109,6 +160,10 @@ static void osmesa_setup_DD_pointers( GLcontext *ctx );
 
 
 
+/**********************************************************************/
+/*****                    Public Functions                        *****/
+/**********************************************************************/
+
 
 /*
  * Create an Off-Screen Mesa rendering context.  The only attribute needed is
@@ -231,7 +286,8 @@ OSMesaContext OSMesaCreateContext( GLenum format, OSMesaContext sharelist )
                                             STENCIL_BITS,
                                             ACCUM_BITS,
                                             index_bits,
-                                            rscale, gscale, bscale, ascale );
+                                            rscale, gscale, bscale, ascale,
+                                            8, 8, 8, 0 );
       if (!osmesa->gl_visual) {
          return NULL;
       }
@@ -257,6 +313,7 @@ OSMesaContext OSMesaCreateContext( GLenum format, OSMesaContext sharelist )
       osmesa->height = 0;
       osmesa->pixel = 0;
       osmesa->clearpixel = 0;
+      osmesa->userRowLength = 0;
       osmesa->rowlength = 0;
       osmesa->yup = GL_TRUE;
       osmesa->rshift = rshift;
@@ -389,15 +446,16 @@ GLboolean OSMesaMakeCurrent( OSMesaContext ctx, void *buffer, GLenum type,
    ctx->buffer = buffer;
    ctx->width = width;
    ctx->height = height;
-   if (ctx->rowlength==0) {
+   if (ctx->userRowLength)
+      ctx->rowlength = ctx->userRowLength;
+   else
       ctx->rowlength = width;
-   }
 
    osmesa_setup_DD_pointers( ctx->gl_ctx );
 
 #ifdef THREADS
    /* Set current context for the calling thread */
-   /* TODO */
+   osmesa_set_thread_context(ctx);
 #else
    /* Set current context for the address space, all threads */
    Current = ctx;
@@ -423,6 +481,7 @@ OSMesaContext OSMesaGetCurrentContext( void )
 {
 #ifdef THREADS
    /* Return current handle for the calling thread */
+   return osmesa_get_thread_context();
 #else
    /* Return current handle for the address space, all threads */
    return Current;
@@ -442,6 +501,7 @@ void OSMesaPixelStore( GLint pname, GLint value )
                       "OSMesaPixelStore(value)" );
             return;
          }
+         ctx->userRowLength = value;
          ctx->rowlength = value;
          break;
       case OSMESA_Y_UP:
@@ -487,6 +547,36 @@ void OSMesaGetIntegerv( GLint pname, GLint *value )
 
 
 
+/*
+ * Return the depth buffer associated with an OSMesa context.
+ * Input:  c - the OSMesa context
+ * Output:  width, height - size of buffer in pixels
+ *          bytesPerValue - bytes per depth value (2 or 4)
+ *          buffer - pointer to depth buffer values
+ * Return:  GL_TRUE or GL_FALSE to indicate success or failure.
+ */
+GLboolean OSMesaGetDepthBuffer( OSMesaContext c, GLint *width, GLint *height,
+                                GLint *bytesPerValue, void **buffer )
+{
+   if ((!c->gl_buffer) || (!c->gl_buffer->Depth)) {
+      *width = 0;
+      *height = 0;
+      *bytesPerValue = 0;
+      *buffer = 0;
+      return GL_FALSE;
+   }
+   else {
+      *width = c->gl_buffer->Width;
+      *height = c->gl_buffer->Height;
+      *bytesPerValue = sizeof(GLdepth);
+      *buffer = c->gl_buffer->Depth;
+      return GL_TRUE;
+   }
+}
+
+
+
+
 /**********************************************************************/
 /*** Device Driver Functions                                        ***/
 /**********************************************************************/
@@ -1013,83 +1103,150 @@ static void read_color_pixels3( GLcontext *ctx,
 /*****                   Optimized line rendering                 *****/
 /**********************************************************************/
 
-/*
- * Despite being clipped to the view volume, the line's window coordinates
- * may just lie outside the window bounds.  That is, if the legal window
- * coordinates are [0,W-1][0,H-1], it's possible for x==W and/or y==H.
- * These quick and dirty macros take care of that possibility.
- */
- 
-#define WINCLIP_X(X1,X2)		\
-   {					\
-      GLint w = ctx->Buffer->Width;	\
-      if ((X1==w) | (X2==w)) {		\
-         if ((X1==w) & (X2==w))  return;\
-         X1 -= (X1==w);   X2 -= (X2==w);\
-      }					\
-   }
-
-#define WINCLIP_Y(Y1,Y2)		\
-   {					\
-      GLint h = ctx->Buffer->Height;	\
-      if ((Y1==h) | (Y2==h)) {		\
-         if ((Y1==h) & (Y2==h))  return;\
-         Y1 -= (Y1==h);   Y2 -= (Y2==h);\
-      }					\
-   }
 
 /*
  * Draw a flat-shaded, RGB line into an osmesa buffer.
  */
 static void flat_color_line( GLcontext *ctx,
-                                 GLuint v0, GLuint v1, GLuint pv )
+                             GLuint vert0, GLuint vert1, GLuint pvert )
 {
    OSMesaContext osmesa = (OSMesaContext) ctx->DriverCtx;
-   struct vertex_buffer *VB = ctx->VB;
-   GLint x1 = (GLint) VB->Win[v0][0], y1 = (GLint) VB->Win[v0][1];
-   GLint x2 = (GLint) VB->Win[v1][0], y2 = (GLint) VB->Win[v1][1];
-   unsigned long pixel = PACK_RGBA( VB->Color[pv][0], VB->Color[pv][1],
-                                   VB->Color[pv][2], 0 );
-   WINCLIP_X(x1,x2);
-   WINCLIP_Y(y1,y2);
+   GLubyte *color = ctx->VB->Color[pvert];
+   unsigned long pixel = PACK_RGBA( color[0], color[1], color[2], color[3] );
 
-#define BRESENHAM_PLOT(X,Y)					\
-   { GLuint *ptr4 = PIXELADDR4(X,Y); *ptr4 = pixel; }
+#define INTERP_XY 1
+#define CLIP_HACK 1
+#define PLOT(X,Y) { GLuint *ptr4 = PIXELADDR4(X,Y); *ptr4 = pixel; }
 
-   BRESENHAM( x1, y1, x2, y2 );
-
-#undef BRESENHAM_PLOT
+#include "linetemp.h"
 }
 
+
 /*
  * Draw a flat-shaded, Z-less, RGB line into an osmesa buffer.
  */
 static void flat_color_z_line( GLcontext *ctx,
-                                   GLuint v0, GLuint v1, GLuint pv )
+                               GLuint vert0, GLuint vert1, GLuint pvert )
+{
+   OSMesaContext osmesa = (OSMesaContext) ctx->DriverCtx;
+   GLubyte *color = ctx->VB->Color[pvert];
+   unsigned long pixel = PACK_RGBA( color[0], color[1], color[2], color[3] );
+
+#define INTERP_XY 1
+#define INTERP_Z 1
+#define CLIP_HACK 1
+#define PLOT(X,Y)				\
+	if (Z < *zPtr) {			\
+	   GLuint *ptr4 = PIXELADDR4(X,Y);	\
+	   *ptr4 = pixel;			\
+	   *zPtr = Z;				\
+	}
+
+#include "linetemp.h"
+}
+
+
+/*
+ * Draw a flat-shaded, alpha-blended, RGB line into an osmesa buffer.
+ */
+static void flat_blend_color_line( GLcontext *ctx,
+                                 GLuint vert0, GLuint vert1, GLuint pvert )
 {
    OSMesaContext osmesa = (OSMesaContext) ctx->DriverCtx;
    struct vertex_buffer *VB = ctx->VB;
-   GLint x1 = (GLint) VB->Win[v0][0], x2 = (GLint) VB->Win[v1][0];
-   GLint y1 = (GLint) VB->Win[v0][1], y2 = (GLint) VB->Win[v1][1];
-   GLint z1 = (GLint) (VB->Win[v0][2] + ctx->LineZoffset);
-   GLint z2 = (GLint) (VB->Win[v1][2] + ctx->LineZoffset);
-   unsigned long pixel = PACK_RGBA( VB->Color[pv][0], VB->Color[pv][1],
-                                   VB->Color[pv][2], 0 );
+   GLint rshift = osmesa->rshift;
+   GLint gshift = osmesa->gshift;
+   GLint bshift = osmesa->bshift;
+   GLint avalue = VB->Color[pvert][3];
+   GLint msavalue = 255 - avalue;
+   GLint rvalue = VB->Color[pvert][0]*avalue;
+   GLint gvalue = VB->Color[pvert][1]*avalue;
+   GLint bvalue = VB->Color[pvert][2]*avalue;
 
-   WINCLIP_X(x1,x2);
-   WINCLIP_Y(y1,y2);
+#define INTERP_XY 1
+#define CLIP_HACK 1
+#define PLOT(X,Y)					\
+   { GLuint *ptr4 = PIXELADDR4(X,Y); \
+     GLuint  pixel = 0; \
+     pixel |=((((((*ptr4) >> rshift) & 0xff)*msavalue+rvalue)>>8) << rshift);\
+     pixel |=((((((*ptr4) >> gshift) & 0xff)*msavalue+gvalue)>>8) << gshift);\
+     pixel |=((((((*ptr4) >> bshift) & 0xff)*msavalue+bvalue)>>8) << bshift);\
+     *ptr4 = pixel; \
+   }
+   
+#include "linetemp.h"
+}
 
-#define BRESENHAM_PLOT(X,Y,Z,ZBUF)		\
-	if (Z < *ZBUF) {			\
-          { GLuint *ptr4 = PIXELADDR4(X,Y); *ptr4 = pixel; } \
-	   *ZBUF = Z;				\
+/*
+ * Draw a flat-shaded, Z-less, alpha-blended, RGB line into an osmesa buffer.
+ */
+static void flat_blend_color_z_line( GLcontext *ctx,
+                                   GLuint vert0, GLuint vert1, GLuint pvert )
+{
+   OSMesaContext osmesa = (OSMesaContext) ctx->DriverCtx;
+   struct vertex_buffer *VB = ctx->VB;
+   GLint rshift = osmesa->rshift;
+   GLint gshift = osmesa->gshift;
+   GLint bshift = osmesa->bshift;
+   GLint avalue = VB->Color[pvert][3];
+   GLint msavalue = 256 - avalue;
+   GLint rvalue = VB->Color[pvert][0]*avalue;
+   GLint gvalue = VB->Color[pvert][1]*avalue;
+   GLint bvalue = VB->Color[pvert][2]*avalue;
+
+#define INTERP_XY 1
+#define INTERP_Z 1
+#define CLIP_HACK 1
+#define PLOT(X,Y)				\
+	if (Z < *zPtr) {			\
+   { GLuint *ptr4 = PIXELADDR4(X,Y); \
+     GLuint  pixel = 0; \
+     pixel |=((((((*ptr4) >> rshift) & 0xff)*msavalue+rvalue)>>8) << rshift);\
+     pixel |=((((((*ptr4) >> gshift) & 0xff)*msavalue+gvalue)>>8) << gshift);\
+     pixel |=((((((*ptr4) >> bshift) & 0xff)*msavalue+bvalue)>>8) << bshift);\
+     *ptr4 = pixel; \
+   } \
 	}
 
-   BRESENHAM_Z( ctx, x1, y1, z1, x2, y2, z2 );
-
-#undef BRESENHAM_PLOT
+#include "linetemp.h"
 }
 
+/*
+ * Draw a flat-shaded, Z-less, alpha-blended, RGB line into an osmesa buffer.
+ */
+static void flat_blend_color_z_line_write( GLcontext *ctx,
+                                   GLuint vert0, GLuint vert1, GLuint pvert )
+{
+   OSMesaContext osmesa = (OSMesaContext) ctx->DriverCtx;
+   struct vertex_buffer *VB = ctx->VB;
+   GLint rshift = osmesa->rshift;
+   GLint gshift = osmesa->gshift;
+   GLint bshift = osmesa->bshift;
+   GLint avalue = VB->Color[pvert][3];
+   GLint msavalue = 256 - avalue;
+   GLint rvalue = VB->Color[pvert][0]*avalue;
+   GLint gvalue = VB->Color[pvert][1]*avalue;
+   GLint bvalue = VB->Color[pvert][2]*avalue;
+
+#define INTERP_XY 1
+#define INTERP_Z 1
+#define CLIP_HACK 1
+#define PLOT(X,Y)				\
+	if (Z < *zPtr) {			\
+   { GLuint *ptr4 = PIXELADDR4(X,Y); \
+     GLuint  pixel = 0; \
+     pixel |=((((((*ptr4) >> rshift) & 0xff)*msavalue+rvalue)>>8) << rshift);\
+     pixel |=((((((*ptr4) >> gshift) & 0xff)*msavalue+gvalue)>>8) << gshift);\
+     pixel |=((((((*ptr4) >> bshift) & 0xff)*msavalue+bvalue)>>8) << bshift);\
+     *ptr4 = pixel; \
+   } \
+	   *zPtr = Z;				\
+	}
+
+#include "linetemp.h"
+}
+
+
 /*
  * Analyze context state to see if we can provide a fast line drawing
  * function, like those in lines.c.  Otherwise, return NULL.
@@ -1102,12 +1259,13 @@ static line_func choose_line_function( GLcontext *ctx )
    if (ctx->Texture.Enabled)              return NULL;
    if (ctx->Light.ShadeModel!=GL_FLAT)    return NULL;
 
-   if (ctx->RasterMask==DEPTH_BIT
-       && ctx->Depth.Func==GL_LESS
-       && ctx->Depth.Mask==GL_TRUE
-       && ctx->Line.Width==1.0F
+   if (ctx->Line.Width==1.0F
        && ctx->Line.StippleFlag==GL_FALSE) {
-       switch(osmesa->format) {
+
+       if (ctx->RasterMask==DEPTH_BIT
+           && ctx->Depth.Func==GL_LESS
+           && ctx->Depth.Mask==GL_TRUE) {
+           switch(osmesa->format) {
        		case OSMESA_RGBA:
        		case OSMESA_BGRA:
        		case OSMESA_ARGB:
@@ -1116,12 +1274,11 @@ static line_func choose_line_function( GLcontext *ctx )
        		default:
        			return NULL;
        			break;
+           }
        }
-   }
-   if (ctx->RasterMask==0
-       && ctx->Line.Width==1.0F
-       && ctx->Line.StippleFlag==GL_FALSE) {
-       switch(osmesa->format) {
+
+       if (ctx->RasterMask==0) {
+           switch(osmesa->format) {
        		case OSMESA_RGBA:
        		case OSMESA_BGRA:
        		case OSMESA_ARGB:
@@ -1130,11 +1287,66 @@ static line_func choose_line_function( GLcontext *ctx )
        		default:
        			return NULL;
        			break;
+           }
        }
+
+       if (ctx->RasterMask==(DEPTH_BIT|BLEND_BIT)
+           && ctx->Depth.Func==GL_LESS
+           && ctx->Depth.Mask==GL_TRUE
+           && ctx->Color.BlendSrc==GL_SRC_ALPHA
+           && ctx->Color.BlendDst==GL_ONE_MINUS_SRC_ALPHA
+           && ctx->Color.BlendEquation==GL_FUNC_ADD_EXT) {
+           switch(osmesa->format) {
+       		case OSMESA_RGBA:
+       		case OSMESA_BGRA:
+       		case OSMESA_ARGB:
+       			return flat_blend_color_z_line_write;
+       			break;
+       		default:
+       			return NULL;
+       			break;
+           }
+       }
+
+       if (ctx->RasterMask==(DEPTH_BIT|BLEND_BIT)
+           && ctx->Depth.Func==GL_LESS
+           && ctx->Depth.Mask==GL_FALSE
+           && ctx->Color.BlendSrc==GL_SRC_ALPHA
+           && ctx->Color.BlendDst==GL_ONE_MINUS_SRC_ALPHA
+           && ctx->Color.BlendEquation==GL_FUNC_ADD_EXT) {
+           switch(osmesa->format) {
+       		case OSMESA_RGBA:
+       		case OSMESA_BGRA:
+       		case OSMESA_ARGB:
+       			return flat_blend_color_z_line;
+       			break;
+       		default:
+       			return NULL;
+       			break;
+           }
+       }
+
+       if (ctx->RasterMask==BLEND_BIT
+           && ctx->Color.BlendSrc==GL_SRC_ALPHA
+           && ctx->Color.BlendDst==GL_ONE_MINUS_SRC_ALPHA
+           && ctx->Color.BlendEquation==GL_FUNC_ADD_EXT) {
+           switch(osmesa->format) {
+       		case OSMESA_RGBA:
+       		case OSMESA_BGRA:
+       		case OSMESA_ARGB:
+       			return flat_blend_color_line;
+       			break;
+       		default:
+       			return NULL;
+       			break;
+           }
+       }
+
    }
    return NULL;
 }
 
+
 /**********************************************************************/
 /*****                 Optimized triangle rendering               *****/
 /**********************************************************************/
@@ -1237,10 +1449,17 @@ static triangle_func choose_triangle_function( GLcontext *ctx )
 
 
 
+static const char *renderer_string(void)
+{
+   return "OffScreen";
+}
+
+
 static void osmesa_setup_DD_pointers( GLcontext *ctx )
 {
    OSMesaContext osmesa = (OSMesaContext) ctx->DriverCtx;
    
+   ctx->Driver.RendererString = renderer_string;
    ctx->Driver.UpdateState = osmesa_setup_DD_pointers;
 
    ctx->Driver.SetBuffer = set_buffer;
@@ -1255,29 +1474,30 @@ static void osmesa_setup_DD_pointers( GLcontext *ctx )
    ctx->Driver.PointsFunc = NULL;
    ctx->Driver.LineFunc = choose_line_function( ctx );
    ctx->Driver.TriangleFunc = choose_triangle_function( ctx );
-   
-   ctx->Driver.WriteColorSpan = write_color_span;
-   ctx->Driver.WriteColorPixels = write_color_pixels;
-   ctx->Driver.WriteIndexSpan = write_index_span;
-   ctx->Driver.WriteMonocolorSpan = write_monocolor_span;
-   ctx->Driver.WriteMonoindexSpan = write_monoindex_span;
-   ctx->Driver.WriteIndexPixels = write_index_pixels;
-   ctx->Driver.WriteMonocolorPixels = write_monocolor_pixels;
-   ctx->Driver.WriteMonoindexPixels = write_monoindex_pixels;
 
-   ctx->Driver.ReadColorSpan = read_color_span;
-   ctx->Driver.ReadIndexSpan = read_index_span;
-   ctx->Driver.ReadColorPixels = read_color_pixels;
-   ctx->Driver.ReadIndexPixels = read_index_pixels;
-   
-   if ((osmesa->format==OSMESA_RGB)||(osmesa->format==OSMESA_BGR)) {
+   /* RGB(A) span/pixel functions */
+   if ((osmesa->format==OSMESA_RGB) || (osmesa->format==OSMESA_BGR)) {
       ctx->Driver.WriteColorSpan = write_color_span3;
       ctx->Driver.WriteColorPixels = write_color_pixels3;
       ctx->Driver.WriteMonocolorSpan = write_monocolor_span3;
       ctx->Driver.WriteMonocolorPixels = write_monocolor_pixels3;
-
       ctx->Driver.ReadColorSpan = read_color_span3;
       ctx->Driver.ReadColorPixels = read_color_pixels3;
    }
+   else {
+      ctx->Driver.WriteColorSpan = write_color_span;
+      ctx->Driver.WriteColorPixels = write_color_pixels;
+      ctx->Driver.WriteMonocolorSpan = write_monocolor_span;
+      ctx->Driver.WriteMonocolorPixels = write_monocolor_pixels;
+      ctx->Driver.ReadColorSpan = read_color_span;
+      ctx->Driver.ReadColorPixels = read_color_pixels;
+   }
 
+   /* CI span/pixel functions */
+   ctx->Driver.WriteIndexSpan = write_index_span;
+   ctx->Driver.WriteMonoindexSpan = write_monoindex_span;
+   ctx->Driver.WriteIndexPixels = write_index_pixels;
+   ctx->Driver.WriteMonoindexPixels = write_monoindex_pixels;
+   ctx->Driver.ReadIndexSpan = read_index_span;
+   ctx->Driver.ReadIndexPixels = read_index_pixels;
 }
