@@ -1,8 +1,8 @@
-/* $Id: pb.h,v 1.2 1997/02/09 18:43:14 brianp Exp $ */
+/* $Id: pb.h,v 1.4 1997/11/13 02:16:48 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.5
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,12 @@
 
 /*
  * $Log: pb.h,v $
+ * Revision 1.4  1997/11/13 02:16:48  brianp
+ * added lambda array, initialized to zeros
+ *
+ * Revision 1.3  1997/05/09 22:40:19  brianp
+ * added gl_alloc_pb()
+ *
  * Revision 1.2  1997/02/09 18:43:14  brianp
  * added GL_EXT_texture3D support
  *
@@ -58,6 +64,7 @@ struct pixel_buffer {
 	GLfloat s[PB_SIZE];	/* Texture S coordinate */
 	GLfloat t[PB_SIZE];	/* Texture T coordinate */
 	GLfloat u[PB_SIZE];	/* Texture R coordinate */
+	GLfloat lambda[PB_SIZE];/* Texture lambda value */
 	GLint color[4];		/* Mono color, integers! */
 	GLuint index;		/* Mono index */
 	GLuint count;		/* Number of pixels in buffer */
@@ -163,6 +170,8 @@ struct pixel_buffer {
 	}
 
 
+extern struct pixel_buffer *gl_alloc_pb(void);
+
 extern void gl_flush_pb( GLcontext *ctx );
 
 #endif
