@@ -1,9 +1,9 @@
-/* $Id: pixel.c,v 1.3 1996/09/26 22:35:10 brianp Exp $ */
+/* $Id: pixel.c,v 1.8 1997/07/24 01:23:44 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.0
- * Copyright (C) 1995-1996  Brian Paul
+ * Version:  2.4
+ * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +23,21 @@
 
 /*
  * $Log: pixel.c,v $
+ * Revision 1.8  1997/07/24 01:23:44  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.7  1997/05/28 03:26:02  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.6  1997/02/09 20:05:03  brianp
+ * new arguments for gl_pixel_addr_in_image()
+ *
+ * Revision 1.5  1997/02/09 18:52:37  brianp
+ * added GL_EXT_texture3D support
+ *
+ * Revision 1.4  1996/11/06 04:09:37  brianp
+ * added a missing return after a gl_error() call
+ *
  * Revision 1.3  1996/09/26 22:35:10  brianp
  * fixed a few compiler warnings from IRIX 6 -n32 and -64 compiler
  *
@@ -40,7 +55,9 @@
  */
 
 
-
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -53,6 +70,7 @@
 #include "span.h"
 #include "stencil.h"
 #include "types.h"
+#endif
 
 
 
@@ -526,6 +544,7 @@ void gl_PixelMapfv( GLcontext *ctx,
       }
       if (!ok) {
 	 gl_error( ctx, GL_INVALID_VALUE, "glPixelMapfv(mapsize)" );
+         return;
       }
    }
 
@@ -894,8 +913,8 @@ GLvoid *gl_unpack_pixels( GLcontext *ctx,
       width_in_bytes = CEILING( width, 8 );
       dst = buffer;
       for (i=0;i<height;i++) {
-         src = gl_pixel_addr_in_image( ctx, pixels, width, height,
-                                       format, type, GL_FALSE, i, 0 );
+         src = gl_pixel_addr_in_image( &ctx->Unpack, pixels, width, height,
+                                       format, type, 0, i, 0 );
          if (!src) {
             free(buffer);
             return NULL;
@@ -928,8 +947,8 @@ GLvoid *gl_unpack_pixels( GLcontext *ctx,
       /* Copy/unpack pixel data to buffer */
       dst = buffer;
       for (i=0;i<height;i++) {
-         src = gl_pixel_addr_in_image( ctx, pixels, width, height,
-                                       format, type, GL_FALSE, i, 0 );
+         src = gl_pixel_addr_in_image( &ctx->Unpack, pixels, width, height,
+                                       format, type, 0, i, 0 );
          if (!src) {
             free(buffer);
             return NULL;
