@@ -1,8 +1,8 @@
-/* $Id: readpix.c,v 1.7 1997/02/03 20:31:15 brianp Exp $ */
+/* $Id: readpix.c,v 1.10 1997/07/24 01:25:18 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.4
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,15 @@
 
 /*
  * $Log: readpix.c,v $
+ * Revision 1.10  1997/07/24 01:25:18  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.9  1997/05/28 03:26:18  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.8  1997/05/08 01:43:50  brianp
+ * added error check to gl_ReadPixels() for inside glBegin/glEnd
+ *
  * Revision 1.7  1997/02/03 20:31:15  brianp
  * added a few DEFARRAY macros for BeOS
  *
@@ -47,6 +56,9 @@
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
@@ -61,7 +73,7 @@
 #include "span.h"
 #include "stencil.h"
 #include "types.h"
-
+#endif
 
 
 
@@ -872,6 +884,11 @@ void gl_ReadPixels( GLcontext *ctx,
                     GLint x, GLint y, GLsizei width, GLsizei height,
 		    GLenum format, GLenum type, GLvoid *pixels )
 {
+   if (INSIDE_BEGIN_END(ctx)) {
+      gl_error( ctx, GL_INVALID_OPERATION, "glReadPixels" );
+      return;
+   }
+
    (void) (*ctx->Driver.SetBuffer)( ctx, ctx->Pixel.ReadBuffer );
 
    switch (format) {
