@@ -1,8 +1,8 @@
-/* $Id: stencil.c,v 1.3 1997/02/27 19:58:35 brianp Exp $ */
+/* $Id: stencil.c,v 1.8 1998/01/01 00:52:11 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
+ * Version:  2.6
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,21 @@
 
 /*
  * $Log: stencil.c,v $
+ * Revision 1.8  1998/01/01 00:52:11  brianp
+ * added some tests to prevent crashing if Driver.DepthTestPixels not defined
+ *
+ * Revision 1.7  1997/07/24 01:21:56  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.6  1997/05/28 03:26:29  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.5  1997/04/29 01:26:37  brianp
+ * fixed a few return statements which were missing values
+ *
+ * Revision 1.4  1997/04/20 20:29:11  brianp
+ * replaced abort() with gl_problem()
+ *
  * Revision 1.3  1997/02/27 19:58:35  brianp
  * don't try to clear stencil buffer if there isn't one
  *
@@ -35,6 +50,9 @@
  */
 
 
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <stdlib.h>
 #include <string.h>
 #include "context.h"
@@ -43,7 +61,7 @@
 #include "pb.h"
 #include "stencil.h"
 #include "types.h"
-
+#endif
 
 
 /*
@@ -302,7 +320,7 @@ static void apply_stencil_op_to_span( GLcontext *ctx,
 	 }
 	 break;
       default:
-         abort();
+         gl_problem(ctx, "Bad stencilop in apply_stencil_op_to_span");
    }
 }
 
@@ -473,7 +491,8 @@ GLint gl_stencil_span( GLcontext *ctx,
 	 }
 	 break;
       default:
-         abort();
+         gl_problem(ctx, "Bad stencil func in gl_stencil_span");
+         return 0;
    }
 
    apply_stencil_op_to_span( ctx, n, x, y, ctx->Stencil.FailFunc, fail );
@@ -516,7 +535,8 @@ void gl_depth_stencil_span( GLcontext *ctx,
       }
 
       /* apply the depth test */
-      (*ctx->Driver.DepthTestSpan)( ctx, n, x, y, z, mask );
+      if (ctx->Driver.DepthTestSpan)
+         (*ctx->Driver.DepthTestSpan)( ctx, n, x, y, z, mask );
 
       /* set the stencil pass/fail flags according to result of depth test */
       for (i=0;i<n;i++) {
@@ -664,8 +684,7 @@ static void apply_stencil_op_to_pixels( GLcontext *ctx,
 	 }
 	 break;
       default:
-         abort();
-	 return;
+         gl_problem(ctx, "Bad stencilop in apply_stencil_op_to_pixels");
    }
 }
 
@@ -840,7 +859,8 @@ GLint gl_stencil_pixels( GLcontext *ctx,
 	 }
 	 break;
       default:
-         abort();
+         gl_problem(ctx, "Bad stencil func in gl_stencil_pixels");
+         return 0;
    }
 
    apply_stencil_op_to_pixels( ctx, n, x, y, ctx->Stencil.FailFunc, fail );
@@ -883,7 +903,8 @@ void gl_depth_stencil_pixels( GLcontext *ctx,
       }
 
       /* apply the depth test */
-      (*ctx->Driver.DepthTestPixels)( ctx, n, x, y, z, mask );
+      if (ctx->Driver.DepthTestPixels)
+         (*ctx->Driver.DepthTestPixels)( ctx, n, x, y, z, mask );
 
       /* set the stencil pass/fail flags according to result of depth test */
       for (i=0;i<n;i++) {
