@@ -1,9 +1,9 @@
-/* $Id: xform.c,v 1.5 1997/01/03 23:54:17 brianp Exp $ */
+/* $Id: xform.c,v 1.10 1997/10/30 06:00:06 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.2
- * Copyright (C) 1995-1996  Brian Paul
+ * Version:  2.5
+ * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +23,21 @@
 
 /*
  * $Log: xform.c,v $
+ * Revision 1.10  1997/10/30 06:00:06  brianp
+ * added Intel X86 assembly optimzations (Josh Vanderhoof)
+ *
+ * Revision 1.9  1997/07/24 01:25:54  brianp
+ * changed precompiled header symbol from PCH to PC_HEADER
+ *
+ * Revision 1.8  1997/05/28 03:27:03  brianp
+ * added precompiled header (PCH) support
+ *
+ * Revision 1.7  1997/05/01 01:40:51  brianp
+ * replaced sqrt() with GL_SQRT()
+ *
+ * Revision 1.6  1997/04/02 03:15:02  brianp
+ * removed gl_xform_texcoords_4fv()
+ *
  * Revision 1.5  1997/01/03 23:54:17  brianp
  * changed length threshold in gl_xform_normals_3fv() to 1E-30 per Jeroen
  *
@@ -53,12 +68,14 @@
  */
 
 
-
+#ifdef PC_HEADER
+#include "all.h"
+#else
 #include <math.h>
+#include "mmath.h"
 #include "types.h"
 #include "xform.h"
-
-
+#endif
 
 
 
@@ -167,6 +184,7 @@ void gl_xform_points_3fv( GLuint n, GLfloat q[][4], const GLfloat m[16],
 
 
 
+#ifndef USE_ASM
 /*
  * Apply a transformation matrix to an array of normal vectors:
  *   for i in 0 to n-1 do  v[i] = u[i] * m
@@ -193,7 +211,7 @@ void gl_xform_normals_3fv( GLuint n, GLfloat v[][3], const GLfloat m[16],
          }
          {
             GLdouble len, scale;
-            len = sqrt( tx*tx + ty*ty + tz*tz );
+            len = GL_SQRT( tx*tx + ty*ty + tz*tz );
             scale = (len>1E-30) ? (1.0 / len) : 1.0;
             v[i][0] = tx * scale;
             v[i][1] = ty * scale;
@@ -215,7 +233,7 @@ void gl_xform_normals_3fv( GLuint n, GLfloat v[][3], const GLfloat m[16],
       }
    }
 }
-
+#endif
 
 
 /*
@@ -238,25 +256,3 @@ void gl_transform_vector( GLfloat u[4], const GLfloat v[4], const GLfloat m[16]
 #undef M
 }
 
-
-
-/*
- * Apply a transformation matrix to an array of [S T R Q] texture coordinates:
- *   for i in 0 to n-1 do   tc[i] = m * tc[i]
- */
-void gl_xform_texcoords_4fv( GLuint n, GLfloat tc[][4], const GLfloat m[16] )
-{
-   GLfloat m0 = m[0], m4 = m[4],  m8 = m[ 8], m12 = m[12];
-   GLfloat m1 = m[1], m5 = m[5],  m9 = m[ 9], m13 = m[13];
-   GLfloat m2 = m[2], m6 = m[6], m10 = m[10], m14 = m[14];
-   GLfloat m3 = m[3], m7 = m[7], m11 = m[11], m15 = m[15];
-   GLint i;
-
-   for (i=0;i<n;i++) {
-      GLfloat t0 = tc[i][0], t1 = tc[i][1], t2 = tc[i][2], t3 = tc[i][3];
-      tc[i][0] = m0 * t0 + m4 * t1 +  m8 * t2 + m12 * t3;
-      tc[i][1] = m1 * t0 + m5 * t1 +  m9 * t2 + m13 * t3;
-      tc[i][2] = m2 * t0 + m6 * t1 + m10 * t2 + m14 * t3;
-      tc[i][3] = m3 * t0 + m7 * t1 + m11 * t2 + m15 * t3;
-   }
-}
