@@ -1,4 +1,4 @@
-/* $Id: xform.h,v 1.3 1996/11/08 02:20:39 brianp Exp $ */
+/* $Id: xform.h,v 1.4 1997/04/02 03:15:02 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -23,6 +23,9 @@
 
 /*
  * $Log: xform.h,v $
+ * Revision 1.4  1997/04/02 03:15:02  brianp
+ * removed gl_xform_texcoords_4fv()
+ *
  * Revision 1.3  1996/11/08 02:20:39  brianp
  * added gl_xform_texcoords_4fv()
  *
@@ -79,8 +82,4 @@ extern void gl_transform_vector( GLfloat u[4],
 				 const GLfloat v[4], const GLfloat m[16] );
 
 
-extern void gl_xform_texcoords_4fv( GLuint n, GLfloat tc[][4],
-                                    const GLfloat m[16] );
-
-
 #endif
