@@ -1,8 +1,8 @@
-/* $Id: xmesa2.c,v 1.6 1996/10/22 02:46:29 brianp Exp $ */
+/* $Id: xmesa2.c,v 1.11 1997/05/26 20:34:40 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.0
+ * Version:  2.3
  * Copyright (C) 1995-1996  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,21 @@
 
 /*
  * $Log: xmesa2.c,v $
+ * Revision 1.11  1997/05/26 20:34:40  brianp
+ * added PF_TRUEDITHER pixel format
+ *
+ * Revision 1.10  1997/04/20 20:29:11  brianp
+ * replaced abort() with gl_problem()
+ *
+ * Revision 1.9  1997/04/12 16:24:07  brianp
+ * removed several xmesa_setup_DD_pointers calls, removed die()
+ *
+ * Revision 1.8  1997/04/04 05:23:57  brianp
+ * fixed problem with black/white being reversed on monochrome displays
+ *
+ * Revision 1.7  1997/03/21 01:58:32  brianp
+ * added RendererString() function
+ *
  * Revision 1.6  1996/10/22 02:46:29  brianp
  * now use DITHER_SETUP and XDITHER macros
  * use array indexing instead of pointer dereferencing in inner loops
@@ -58,22 +73,13 @@
 #include <string.h>
 #include <X11/Xlib.h>
 #include "GL/xmesa.h"
+#include "context.h"
 #include "macros.h"
 #include "types.h"
 #include "xmesaP.h"
 
 
 
-/*
- * Abort with an error message.
- */
-static void die( char *s )
-{
-   fprintf( stderr, "%s\n", s );
-   abort();
-}
-
-
 
 /*
  * The following functions are used to trap XGetImage() calls which
@@ -233,7 +239,9 @@ static GLboolean set_buffer( GLcontext *ctx, GLenum mode )
    if (mode==GL_FRONT) {
       /* read/write front buffer */
       xmesa->xm_buffer->buffer = xmesa->xm_buffer->frontbuffer;
-      xmesa_setup_DD_pointers( ctx );
+      ctx->NewState |= NEW_RASTER_OPS;
+      gl_update_state(ctx);
+      /*xmesa_setup_DD_pointers( ctx );*/
       return GL_TRUE;
    }
    else if (mode==GL_BACK && xmesa->xm_buffer->db_state) {
@@ -245,10 +253,12 @@ static GLboolean set_buffer( GLcontext *ctx, GLenum mode )
          xmesa->xm_buffer->buffer = None;
       }
       else {
-         /* just in case, probably a serious error? */
+         /* just in case there wasn't enough memory for back buffer */
          xmesa->xm_buffer->buffer = xmesa->xm_buffer->frontbuffer;
       }
-      xmesa_setup_DD_pointers( ctx );
+      ctx->NewState |= NEW_RASTER_OPS;
+      gl_update_state(ctx);
+      /*xmesa_setup_DD_pointers( ctx );*/
       return GL_TRUE;
    }
    else {
@@ -373,7 +383,6 @@ static GLboolean logicop( GLcontext *ctx, GLenum op )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
    int func;
-   if (!XMesa)  return GL_FALSE;
    if ((xmesa->xm_buffer->buffer==XIMAGE) && op!=GL_COPY) {
       /* X can't do logic ops in Ximages, except for GL_COPY */
       return GL_FALSE;
@@ -415,7 +424,6 @@ static void dither( GLcontext *ctx, GLboolean enable )
    else {
       xmesa->pixelformat = xmesa->xm_visual->undithered_pf;
    }
-   xmesa_setup_DD_pointers( ctx );
 }
 
 
@@ -633,7 +641,9 @@ static void write_span_TRUECOLOR_pixmap( COLOR_SPAN_ARGS )
    if (mask) {
       for (i=0;i<n;i++,x++) {
          if (mask[i]) {
-            XSetForeground( dpy, gc, PACK_RGB( red[i], green[i], blue[i] ) );
+            unsigned long p;
+            PACK_TRUECOLOR( p, red[i], green[i], blue[i] );
+            XSetForeground( dpy, gc, p );
             XDrawPoint( dpy, buffer, gc, (int) x, (int) y );
          }
       }
@@ -642,13 +652,49 @@ static void write_span_TRUECOLOR_pixmap( COLOR_SPAN_ARGS )
       /* draw all pixels */
       XImage *rowimg = xmesa->xm_buffer->rowimage;
       for (i=0;i<n;i++) {
-         XPutPixel( rowimg, i, 0, PACK_RGB( red[i], green[i], blue[i] ) );
+         unsigned long p;
+         PACK_TRUECOLOR( p, red[i], green[i], blue[i] );
+         XPutPixel( rowimg, i, 0, p );
       }
       XPutImage( dpy, buffer, gc, rowimg, 0, 0, x, y, n, 1 );
    }
 }
 
 
+/*
+ * Write a span of PF_TRUEDITHER pixels to a pixmap.
+ */
+static void write_span_TRUEDITHER_pixmap( COLOR_SPAN_ARGS )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   Display *dpy = xmesa->xm_visual->display;
+   Drawable buffer = xmesa->xm_buffer->buffer;
+   GC gc = xmesa->xm_buffer->gc2;
+   register GLuint i;
+   y = FLIP( y );
+   if (mask) {
+      for (i=0;i<n;i++,x++) {
+         if (mask[i]) {
+            unsigned long p;
+            PACK_TRUEDITHER(p, x, y, red[i], green[i], blue[i]);
+            XSetForeground( dpy, gc, p );
+            XDrawPoint( dpy, buffer, gc, (int) x, (int) y );
+         }
+      }
+   }
+   else {
+      XImage *rowimg = xmesa->xm_buffer->rowimage;
+      for (i=0;i<n;i++) {
+         unsigned long p;
+         PACK_TRUEDITHER(p, x+i, y, red[i], green[i], blue[i]);
+         XPutPixel( rowimg, i, 0, p );
+      }
+      XPutImage( dpy, buffer, gc, rowimg, 0, 0, x, y, n, 1 );
+   }
+}
+
+
+
 /*
  * Write a span of PF_8A8B8G8R pixels to a pixmap.
  */
@@ -784,6 +830,7 @@ static void write_span_1BIT_pixmap( COLOR_SPAN_ARGS )
    Drawable buffer = xmesa->xm_buffer->buffer;
    GC gc = xmesa->xm_buffer->gc2;
    register GLuint i;
+   SETUP_1BIT;
    y = FLIP( y );
    if (mask) {
       for (i=0;i<n;i++,x++) {
@@ -898,7 +945,6 @@ static void write_span_GRAYSCALE_pixmap( COLOR_SPAN_ARGS )
 }
 
 
-
 /*
  * Write a span of PF_TRUECOLOR pixels to an XImage.
  */
@@ -911,14 +957,47 @@ static void write_span_TRUECOLOR_ximage( COLOR_SPAN_ARGS )
    if (mask) {
       for (i=0;i<n;i++,x++) {
          if (mask[i]) {
-            XPutPixel( img, x, y, PACK_RGB( red[i], green[i], blue[i] ) );
+            unsigned long p;
+            PACK_TRUECOLOR( p, red[i], green[i], blue[i] );
+            XPutPixel( img, x, y, p );
          }
       }
    }
    else {
       /* draw all pixels */
       for (i=0;i<n;i++,x++) {
-         XPutPixel( img, x, y, PACK_RGB( red[i], green[i], blue[i] ) );
+         unsigned long p;
+         PACK_TRUECOLOR( p, red[i], green[i], blue[i] );
+         XPutPixel( img, x, y, p );
+      }
+   }
+}
+
+
+/*
+ * Write a span of PF_TRUEDITHER pixels to an XImage.
+ */
+static void write_span_TRUEDITHER_ximage( COLOR_SPAN_ARGS )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   XImage *img = xmesa->xm_buffer->backimage;
+   register GLuint i;
+   y = FLIP(y);
+   if (mask) {
+      for (i=0;i<n;i++,x++) {
+         if (mask[i]) {
+            unsigned long p;
+            PACK_TRUEDITHER(p, x, y, red[i], green[i], blue[i]);
+            XPutPixel( img, x, y, p );
+         }
+      }
+   }
+   else {
+      /* draw all pixels */
+      for (i=0;i<n;i++,x++) {
+         unsigned long p;
+         PACK_TRUEDITHER(p, x, y, red[i], green[i], blue[i]);
+         XPutPixel( img, x, y, p );
       }
    }
 }
@@ -1056,6 +1135,7 @@ static void write_span_1BIT_ximage( COLOR_SPAN_ARGS )
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
    XImage *img = xmesa->xm_buffer->backimage;
    register GLuint i;
+   SETUP_1BIT;
    y = FLIP(y);
    if (mask) {
       for (i=0;i<n;i++,x++) {
@@ -1174,6 +1254,7 @@ static void write_span_GRAYSCALE_ximage( COLOR_SPAN_ARGS )
 }
 
 
+
 /*
  * Write a span of 8-bit PF_GRAYSCALE pixels to an XImage.
  */
@@ -1224,13 +1305,36 @@ static void write_pixels_TRUECOLOR_pixmap( COLOR_PIXEL_ARGS )
    register GLuint i;
    for (i=0;i<n;i++) {
       if (mask[i]) {
-	 XSetForeground( dpy, gc, PACK_RGB( red[i], green[i], blue[i] ) );
+         unsigned long p;
+         PACK_TRUECOLOR( p, red[i], green[i], blue[i] );
+	 XSetForeground( dpy, gc, p );
 	 XDrawPoint( dpy, buffer, gc, (int) x[i], (int) FLIP(y[i]) );
       }
    }
 }
 
 
+/*
+ * Write an array of PF_TRUEDITHER pixels to a pixmap.
+ */
+static void write_pixels_TRUEDITHER_pixmap( COLOR_PIXEL_ARGS )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   Display *dpy = xmesa->xm_visual->display;
+   Drawable buffer = xmesa->xm_buffer->buffer;
+   GC gc = xmesa->xm_buffer->gc2;
+   register GLuint i;
+   for (i=0;i<n;i++) {
+      if (mask[i]) {
+         unsigned long p;
+         PACK_TRUEDITHER(p, x[i], y[i], red[i], green[i], blue[i]);
+         XSetForeground( dpy, gc, p );
+         XDrawPoint( dpy, buffer, gc, (int) x[i], (int) FLIP(y[i]) );
+      }
+   }
+}
+
+
 /*
  * Write an array of PF_8A8B8G8R pixels to a pixmap.
  */
@@ -1320,6 +1424,7 @@ static void write_pixels_1BIT_pixmap( COLOR_PIXEL_ARGS )
    Drawable buffer = xmesa->xm_buffer->buffer;
    GC gc = xmesa->xm_buffer->gc2;
    register GLuint i;
+   SETUP_1BIT;
    for (i=0;i<n;i++) {
       if (mask[i]) {
 	 XSetForeground( dpy, gc,
@@ -1399,8 +1504,27 @@ static void write_pixels_TRUECOLOR_ximage( COLOR_PIXEL_ARGS )
    register GLuint i;
    for (i=0;i<n;i++) {
       if (mask[i]) {
-	 XPutPixel( img, x[i], FLIP(y[i]),
-                    PACK_RGB( red[i], green[i], blue[i] ) );
+         unsigned long p;
+         PACK_TRUECOLOR( p, red[i], green[i], blue[i] );
+	 XPutPixel( img, x[i], FLIP(y[i]), p );
+      }
+   }
+}
+
+
+/*
+ * Write an array of PF_TRUEDITHER pixels to an XImage.
+ */
+static void write_pixels_TRUEDITHER_ximage( COLOR_PIXEL_ARGS )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   XImage *img = xmesa->xm_buffer->backimage;
+   register GLuint i;
+   for (i=0;i<n;i++) {
+      if (mask[i]) {
+         unsigned long p;
+         PACK_TRUEDITHER(p, x[i], FLIP(y[i]), red[i], green[i], blue[i]);
+	 XPutPixel( img, x[i], FLIP(y[i]), p );
       }
    }
 }
@@ -1497,6 +1621,7 @@ static void write_pixels_1BIT_ximage( COLOR_PIXEL_ARGS )
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
    XImage *img = xmesa->xm_buffer->backimage;
    register GLuint i;
+   SETUP_1BIT;
    for (i=0;i<n;i++) {
       if (mask[i]) {
 	 XPutPixel( img, x[i], FLIP(y[i]),
@@ -1632,6 +1757,33 @@ static void write_span_mono_pixmap( MONO_SPAN_ARGS )
 }
 
 
+/*
+ * Write a span of PF_TRUEDITHER pixels to a pixmap.  The pixel value is
+ * the one set by DD.color() or DD.index().
+ */
+static void write_span_mono_TRUEDITHER_pixmap( MONO_SPAN_ARGS )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   Display *dpy = xmesa->xm_visual->display;
+   Drawable buffer = xmesa->xm_buffer->buffer;
+   GC gc = xmesa->xm_buffer->gc2;
+   register GLuint i;
+   register GLubyte r, g, b;
+   int yy = FLIP( y );
+   r = xmesa->red;
+   g = xmesa->green;
+   b = xmesa->blue;
+   for (i=0;i<n;i++,x++) {
+      if (mask[i]) {
+         unsigned long p;
+         PACK_TRUEDITHER(p, x, yy, r, g, b);
+         XSetForeground( dpy, gc, p );
+         XDrawPoint( dpy, buffer, gc, (int) x, (int) yy );
+      }
+   }
+}
+
+
 /*
  * Write a span of PF_DITHER pixels to a pixmap.  The pixel value is
  * the one set by DD.color() or DD.index().
@@ -1670,6 +1822,7 @@ static void write_span_mono_1BIT_pixmap( MONO_SPAN_ARGS )
    GC gc = xmesa->xm_buffer->gc2;
    register GLuint i;
    register GLubyte r, g, b;
+   SETUP_1BIT;
    r = xmesa->red;
    g = xmesa->green;
    b = xmesa->blue;
@@ -1702,6 +1855,29 @@ static void write_span_mono_ximage( MONO_SPAN_ARGS )
 }
 
 
+/*
+ * Write a span of identical PF_TRUEDITHER pixels to an XImage.  The pixel
+ * value is the one set by DD.color().
+ */
+static void write_span_mono_TRUEDITHER_ximage( MONO_SPAN_ARGS )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   XImage *img = xmesa->xm_buffer->backimage;
+   GLuint i;
+   GLint r = xmesa->red;
+   GLint g = xmesa->green;
+   GLint b = xmesa->blue;
+   y = FLIP( y );
+   for (i=0;i<n;i++) {
+      if (mask[i]) {
+         unsigned long p;
+         PACK_TRUEDITHER( p, x+i, y, r, g, b);
+	 XPutPixel( img, x+i, y, p );
+      }
+   }
+}
+
+
 /*
  * Write a span of identical 8A8B8G8R pixels to an XImage.  The pixel
  * value is the one set by DD.color().
@@ -1811,6 +1987,7 @@ static void write_span_mono_1BIT_ximage( MONO_SPAN_ARGS )
    XImage *img = xmesa->xm_buffer->backimage;
    register GLuint i;
    register GLubyte r, g, b;
+   SETUP_1BIT;
    r = xmesa->red;
    g = xmesa->green;
    b = xmesa->blue;
@@ -1864,7 +2041,6 @@ static void write_span_mono_GRAYSCALE8_ximage( MONO_SPAN_ARGS )
 
 
 
-
 /**********************************************************************/
 /*** Write MONO COLOR PIXELS functions                              ***/
 /**********************************************************************/
@@ -1892,6 +2068,32 @@ static void write_pixels_mono_pixmap( MONO_PIXEL_ARGS )
 }
 
 
+/*
+ * Write an array of PF_TRUEDITHER pixels to a pixmap.  The pixel value is
+ * the one set by DD.color() or DD.index.
+ */
+static void write_pixels_mono_TRUEDITHER_pixmap( MONO_PIXEL_ARGS )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   Display *dpy = xmesa->xm_visual->display;
+   Drawable buffer = xmesa->xm_buffer->buffer;
+   GC gc = xmesa->xm_buffer->gc2;
+   register GLuint i;
+   register GLubyte r, g, b;
+   r = xmesa->red;
+   g = xmesa->green;
+   b = xmesa->blue;
+   for (i=0;i<n;i++) {
+      if (mask[i]) {
+         unsigned long p;
+         PACK_TRUEDITHER(p, x[i], y[i], r, g, b);
+         XSetForeground( dpy, gc, p );
+	 XDrawPoint( dpy, buffer, gc, (int) x[i], (int) FLIP(y[i]) );
+      }
+   }
+}
+
+
 /*
  * Write an array of PF_DITHER pixels to a pixmap.  The pixel value is
  * the one set by DD.color() or DD.index.
@@ -1929,6 +2131,7 @@ static void write_pixels_mono_1BIT_pixmap( MONO_PIXEL_ARGS )
    GC gc = xmesa->xm_buffer->gc2;
    register GLuint i;
    register GLubyte r, g, b;
+   SETUP_1BIT;
    r = xmesa->red;
    g = xmesa->green;
    b = xmesa->blue;
@@ -1959,6 +2162,28 @@ static void write_pixels_mono_ximage( MONO_PIXEL_ARGS )
 }
 
 
+/*
+ * Write an array of identical TRUEDITHER pixels to an XImage.
+ * The pixel value is the one set by DD.color() or DD.index.
+ */
+static void write_pixels_mono_TRUEDITHER_ximage( MONO_PIXEL_ARGS )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   XImage *img = xmesa->xm_buffer->backimage;
+   register GLuint i;
+   int r = xmesa->red;
+   int g = xmesa->green;
+   int b = xmesa->blue;
+   for (i=0;i<n;i++) {
+      if (mask[i]) {
+         unsigned long p;
+         PACK_TRUEDITHER(p, x[i], FLIP(y[i]), r, g, b);
+	 XPutPixel( img, x[i], FLIP(y[i]), p );
+      }
+   }
+}
+
+
 
 /*
  * Write an array of identical 8A8B8G8R pixels to an XImage.  The pixel value
@@ -2069,6 +2294,7 @@ static void write_pixels_mono_1BIT_ximage( MONO_PIXEL_ARGS )
    XImage *img = xmesa->xm_buffer->backimage;
    register GLuint i;
    register GLubyte r, g, b;
+   SETUP_1BIT;
    r = xmesa->red;
    g = xmesa->green;
    b = xmesa->blue;
@@ -2284,15 +2510,23 @@ static void read_color_span( GLcontext *ctx,
       if (span && !error) {
 	 switch (xmesa->pixelformat) {
 	    case PF_TRUECOLOR:
+	    case PF_TRUEDITHER:
+            case PF_5R6G5B:
                {
-                  GLint rshift = xmesa->xm_visual->rshift, rmult = xmesa->xm_visual->rmult;
-                  GLint gshift = xmesa->xm_visual->gshift, gmult = xmesa->xm_visual->gmult;
-                  GLint bshift = xmesa->xm_visual->bshift, bmult = xmesa->xm_visual->bmult;
+                  unsigned long rMask = xmesa->xm_visual->visinfo->red_mask;
+                  unsigned long gMask = xmesa->xm_visual->visinfo->green_mask;
+                  unsigned long bMask = xmesa->xm_visual->visinfo->blue_mask;
+                  GLubyte *pixelToR = xmesa->xm_visual->PixelToR;
+                  GLubyte *pixelToG = xmesa->xm_visual->PixelToG;
+                  GLubyte *pixelToB = xmesa->xm_visual->PixelToB;
+                  GLint rShift = xmesa->xm_visual->rshift;
+                  GLint gShift = xmesa->xm_visual->gshift;
+                  GLint bShift = xmesa->xm_visual->bshift;
                   for (i=0;i<n;i++) {
                      unsigned long p = XGetPixel( span, i, 0 );
-                     red[i]   = (GLubyte) ((p >> rshift) & rmult);
-                     green[i] = (GLubyte) ((p >> gshift) & gmult);
-                     blue[i]  = (GLubyte) ((p >> bshift) & bmult);
+                     red[i]   = pixelToR[(p & rMask) >> rShift];
+                     green[i] = pixelToG[(p & gMask) >> gShift];
+                     blue[i]  = pixelToB[(p & bMask) >> bShift];
                      alpha[i] = 255;
                   }
                }
@@ -2321,18 +2555,6 @@ static void read_color_span( GLcontext *ctx,
                   }
 	       }
 	       break;
-            case PF_5R6G5B:
-               {
-                  GLushort *ptr2 = (GLushort *) span->data;
-                  for (i=0;i<n;i++) {
-                     GLushort p2 = *ptr2++;
-                     red[i]   = (GLubyte) ((p2 >> 11) & 0x1f);
-                     green[i] = (GLubyte) ((p2 >> 5)  & 0x3f);
-                     blue[i]  = (GLubyte) ( p2        & 0x1f);
-                     alpha[i] = 255;
-                  }
-	       }
-               break;
             case PF_HPCR:
                {
                   GLubyte *ptr1 = (GLubyte *) span->data;
@@ -2374,16 +2596,20 @@ static void read_color_span( GLcontext *ctx,
                }
 	       break;
 	    case PF_1BIT:
-	       for (i=0;i<n;i++) {
-		  unsigned long p = XGetPixel( span, i, 0 );
-		  red[i]   = (GLubyte) (p * 255);
-		  green[i] = (GLubyte) (p * 255);
-		  blue[i]  = (GLubyte) (p * 255);
-		  alpha[i] = 255;
-	       }
+               {
+                  int bitFlip = xmesa->xm_visual->bitFlip;
+                  for (i=0;i<n;i++) {
+                     unsigned long p = XGetPixel( span, i, 0 ) ^ bitFlip;
+                     red[i]   = (GLubyte) (p * 255);
+                     green[i] = (GLubyte) (p * 255);
+                     blue[i]  = (GLubyte) (p * 255);
+                     alpha[i] = 255;
+                  }
+               }
 	       break;
 	    default:
-	       die("Problem in DD.read_color_span (1)");
+	       gl_problem(NULL,"Problem in DD.read_color_span (1)");
+               return;
 	 }
       }
       else {
@@ -2398,22 +2624,30 @@ static void read_color_span( GLcontext *ctx,
    }
    else if (xmesa->xm_buffer->backimage) {
       switch (xmesa->pixelformat) {
-	 case PF_TRUECOLOR:
+         case PF_TRUECOLOR:
+         case PF_TRUEDITHER:
+         case PF_5R6G5B:
             {
-               GLint rshift = xmesa->xm_visual->rshift, rmult = xmesa->xm_visual->rmult;
-               GLint gshift = xmesa->xm_visual->gshift, gmult = xmesa->xm_visual->gmult;
-               GLint bshift = xmesa->xm_visual->bshift, bmult = xmesa->xm_visual->bmult;
+               unsigned long rMask = xmesa->xm_visual->visinfo->red_mask;
+               unsigned long gMask = xmesa->xm_visual->visinfo->green_mask;
+               unsigned long bMask = xmesa->xm_visual->visinfo->blue_mask;
+               GLubyte *pixelToR = xmesa->xm_visual->PixelToR;
+               GLubyte *pixelToG = xmesa->xm_visual->PixelToG;
+               GLubyte *pixelToB = xmesa->xm_visual->PixelToB;
+               GLint rShift = xmesa->xm_visual->rshift;
+               GLint gShift = xmesa->xm_visual->gshift;
+               GLint bShift = xmesa->xm_visual->bshift;
                XImage *img = xmesa->xm_buffer->backimage;
                y = FLIP(y);
-               for (i=0;i<n;i++,x++) {
-                  unsigned long p = XGetPixel( img, x, y );
-                  red[i]   = (GLubyte) ((p >> rshift) & rmult);
-                  green[i] = (GLubyte) ((p >> gshift) & gmult);
-                  blue[i]  = (GLubyte) ((p >> bshift) & bmult);
+               for (i=0;i<n;i++) {
+                  unsigned long p = XGetPixel( img, x+i, y );
+                  red[i]   = pixelToR[(p & rMask) >> rShift];
+                  green[i] = pixelToG[(p & gMask) >> gShift];
+                  blue[i]  = pixelToB[(p & bMask) >> bShift];
                   alpha[i] = 255;
                }
             }
-	    break;
+            break;
 	 case PF_8A8B8G8R:
             {
                GLuint *ptr4 = PIXELADDR4( x, y );
@@ -2438,18 +2672,6 @@ static void read_color_span( GLcontext *ctx,
                }
             }
 	    break;
-         case PF_5R6G5B:
-            {
-               GLushort *ptr2 = PIXELADDR2( x, y );
-               for (i=0;i<n;i++) {
-                  GLushort p2 = *ptr2++;
-                  red[i]   = (GLubyte) ((p2 >> 11) & 0x1f);
-                  green[i] = (GLubyte) ((p2 >> 5)  & 0x3f);
-                  blue[i]  = (GLubyte) ( p2        & 0x1f);
-                  alpha[i] = 255;
-               }
-            }
-            break;
          case PF_HPCR:
             {
                GLubyte *ptr1 = PIXELADDR1( x, y );
@@ -2495,9 +2717,10 @@ static void read_color_span( GLcontext *ctx,
 	 case PF_1BIT:
             {
                XImage *img = xmesa->xm_buffer->backimage;
+               int bitFlip = xmesa->xm_visual->bitFlip;
                y = FLIP(y);
                for (i=0;i<n;i++,x++) {
-                  unsigned long p = XGetPixel( img, x, y );
+                  unsigned long p = XGetPixel( img, x, y ) ^ bitFlip;
                   red[i]   = (GLubyte) (p * 255);
                   green[i] = (GLubyte) (p * 255);
                   blue[i]  = (GLubyte) (p * 255);
@@ -2506,7 +2729,8 @@ static void read_color_span( GLcontext *ctx,
 	    }
 	    break;
 	 default:
-	    die("Problem in DD.read_color_span (2)");
+	    gl_problem(NULL,"Problem in DD.read_color_span (2)");
+            return;
       }
    }
 }
@@ -2557,22 +2781,30 @@ static void read_color_pixels( GLcontext *ctx,
    if (xmesa->xm_buffer->buffer) {
       switch (xmesa->pixelformat) {
 	 case PF_TRUECOLOR:
+         case PF_TRUEDITHER:
+         case PF_5R6G5B:
             {
-               GLint rshift = xmesa->xm_visual->rshift, rmult = xmesa->xm_visual->rmult;
-               GLint gshift = xmesa->xm_visual->gshift, gmult = xmesa->xm_visual->gmult;
-               GLint bshift = xmesa->xm_visual->bshift, bmult = xmesa->xm_visual->bmult;
+               unsigned long rMask = xmesa->xm_visual->visinfo->red_mask;
+               unsigned long gMask = xmesa->xm_visual->visinfo->green_mask;
+               unsigned long bMask = xmesa->xm_visual->visinfo->blue_mask;
+               GLubyte *pixelToR = xmesa->xm_visual->PixelToR;
+               GLubyte *pixelToG = xmesa->xm_visual->PixelToG;
+               GLubyte *pixelToB = xmesa->xm_visual->PixelToB;
+               GLint rShift = xmesa->xm_visual->rshift;
+               GLint gShift = xmesa->xm_visual->gshift;
+               GLint bShift = xmesa->xm_visual->bshift;
                for (i=0;i<n;i++) {
-                  if (mask[i] ) {
+                  if (mask[i]) {
                      unsigned long p = read_pixel( dpy, buffer,
                                                    x[i], FLIP(y[i]) );
-                     red[i]   = (GLubyte) ((p >> rshift) & rmult);
-                     green[i] = (GLubyte) ((p >> gshift) & gmult);
-                     blue[i]  = (GLubyte) ((p >> bshift) & bmult);
+                     red[i]   = pixelToR[(p & rMask) >> rShift];
+                     green[i] = pixelToG[(p & gMask) >> gShift];
+                     blue[i]  = pixelToB[(p & bMask) >> bShift];
                      alpha[i] = 255;
                   }
                }
-	    }
-	    break;
+            }
+            break;
 	 case PF_8A8B8G8R:
 	    for (i=0;i<n;i++) {
                if (mask[i]) {
@@ -2597,18 +2829,6 @@ static void read_color_pixels( GLcontext *ctx,
                }
 	    }
 	    break;
-         case PF_5R6G5B:
-            for (i=0;i<n;i++) {
-               if (mask[i]) {
-                  unsigned long p = read_pixel( dpy, buffer,
-                                                x[i], FLIP(y[i]) );
-                  red[i]   = (GLubyte) ((p >> 11) & 0x1f);
-                  green[i] = (GLubyte) ((p >> 5)  & 0x3f);
-                  blue[i]  = (GLubyte) ( p        & 0x1f);
-                  alpha[i] = 255;
-               }
-            }
-            break;
          case PF_HPCR:
             {
                for (i=0;i<n;i++) {
@@ -2643,41 +2863,52 @@ static void read_color_pixels( GLcontext *ctx,
 	    }
 	    break;
 	 case PF_1BIT:
-	    for (i=0;i<n;i++) {
-               if (mask[i]) {
-                  unsigned long p = read_pixel( dpy, buffer,
-                                                x[i], FLIP(y[i]) );
-                  red[i]   = (GLubyte) (p * 255);
-                  green[i] = (GLubyte) (p * 255);
-                  blue[i]  = (GLubyte) (p * 255);
-                  alpha[i] = 255;
-               }
-	    }
-	    break;
-	 default:
-	    die("Problem in DD.read_color_pixels (1)");
-      }
-   }
-   else if (xmesa->xm_buffer->backimage) {
-      switch (xmesa->pixelformat) {
-	 case PF_TRUECOLOR:
             {
-               GLint rshift = xmesa->xm_visual->rshift, rmult = xmesa->xm_visual->rmult;
-               GLint gshift = xmesa->xm_visual->gshift, gmult = xmesa->xm_visual->gmult;
-               GLint bshift = xmesa->xm_visual->bshift, bmult = xmesa->xm_visual->bmult;
-               XImage *img = xmesa->xm_buffer->backimage;
+               int bitFlip = xmesa->xm_visual->bitFlip;
                for (i=0;i<n;i++) {
                   if (mask[i]) {
-                     unsigned long p;
-                     p = XGetPixel( img, x[i], FLIP(y[i]) );
-                     red[i]   = (GLubyte) ((p >> rshift) & rmult);
-                     green[i] = (GLubyte) ((p >> gshift) & gmult);
-                     blue[i]  = (GLubyte) ((p >> bshift) & bmult);
+                     unsigned long p = read_pixel( dpy, buffer,
+                                                   x[i], FLIP(y[i])) ^ bitFlip;
+                     red[i]   = (GLubyte) (p * 255);
+                     green[i] = (GLubyte) (p * 255);
+                     blue[i]  = (GLubyte) (p * 255);
                      alpha[i] = 255;
                   }
                }
 	    }
 	    break;
+	 default:
+	    gl_problem(NULL,"Problem in DD.read_color_pixels (1)");
+            return;
+      }
+   }
+   else if (xmesa->xm_buffer->backimage) {
+      switch (xmesa->pixelformat) {
+	 case PF_TRUECOLOR:
+         case PF_TRUEDITHER:
+         case PF_5R6G5B:
+            {
+               unsigned long rMask = xmesa->xm_visual->visinfo->red_mask;
+               unsigned long gMask = xmesa->xm_visual->visinfo->green_mask;
+               unsigned long bMask = xmesa->xm_visual->visinfo->blue_mask;
+               GLubyte *pixelToR = xmesa->xm_visual->PixelToR;
+               GLubyte *pixelToG = xmesa->xm_visual->PixelToG;
+               GLubyte *pixelToB = xmesa->xm_visual->PixelToB;
+               GLint rShift = xmesa->xm_visual->rshift;
+               GLint gShift = xmesa->xm_visual->gshift;
+               GLint bShift = xmesa->xm_visual->bshift;
+               XImage *img = xmesa->xm_buffer->backimage;
+               for (i=0;i<n;i++) {
+                  if (mask[i]) {
+                     unsigned long p = XGetPixel( img, x[i], FLIP(y[i]) );
+                     red[i]   = pixelToR[(p & rMask) >> rShift];
+                     green[i] = pixelToG[(p & gMask) >> gShift];
+                     blue[i]  = pixelToB[(p & bMask) >> bShift];
+                     alpha[i] = 255;
+                  }
+               }
+            }
+            break;
 	 case PF_8A8B8G8R:
 	    for (i=0;i<n;i++) {
 	       if (mask[i]) {
@@ -2702,18 +2933,6 @@ static void read_color_pixels( GLcontext *ctx,
                }
 	    }
 	    break;
-         case PF_5R6G5B:
-            for (i=0;i<n;i++) {
-               if (mask[i]) {
-                  GLushort *ptr2 = PIXELADDR2( x[i], y[i] );
-                  GLushort p2 = *ptr2;
-                  red[i]   = (GLubyte) ((p2 >> 11) & 0x1f);
-                  green[i] = (GLubyte) ((p2 >> 5)  & 0x3f);
-                  blue[i]  = (GLubyte) ( p2        & 0x1f);
-                  alpha[i] = 255;
-               }
-            }
-            break;
          case PF_HPCR:
             for (i=0;i<n;i++) {
                if (mask[i]) {
@@ -2749,10 +2968,11 @@ static void read_color_pixels( GLcontext *ctx,
 	 case PF_1BIT:
             {
                XImage *img = xmesa->xm_buffer->backimage;
+               int bitFlip = xmesa->xm_visual->bitFlip;
                for (i=0;i<n;i++) {
                   if (mask[i]) {
                      unsigned long p;
-                     p = XGetPixel( img, x[i], FLIP(y[i]) );
+                     p = XGetPixel( img, x[i], FLIP(y[i]) ) ^ bitFlip;
                      red[i]   = (GLubyte) (p * 255);
                      green[i] = (GLubyte) (p * 255);
                      blue[i]  = (GLubyte) (p * 255);
@@ -2762,12 +2982,17 @@ static void read_color_pixels( GLcontext *ctx,
 	    }
 	    break;
 	 default:
-	    die("Problem in DD.read_color_pixels (1)");
+	    gl_problem(NULL,"Problem in DD.read_color_pixels (1)");
+            return;
       }
    }
 }
 
 
+static const char *renderer_string(void)
+{
+   return "X11";
+}
 
 
 /*
@@ -2780,6 +3005,7 @@ void xmesa_setup_DD_pointers( GLcontext *ctx )
    /*
     * Always the same:
     */
+   ctx->Driver.RendererString = renderer_string;
    ctx->Driver.UpdateState = xmesa_setup_DD_pointers;
    ctx->Driver.GetBufferSize = get_buffer_size;
    ctx->Driver.Flush = flush;
@@ -2796,9 +3022,12 @@ void xmesa_setup_DD_pointers( GLcontext *ctx )
    ctx->Driver.LogicOp = logicop;
    ctx->Driver.Dither = dither;
 
-   ctx->Driver.PointsFunc = xmesa_get_points_func( ctx );
-   ctx->Driver.LineFunc = xmesa_get_line_func( ctx );
-   ctx->Driver.TriangleFunc = xmesa_get_triangle_func( ctx );
+   if (!ctx->Driver.PointsFunc)
+      ctx->Driver.PointsFunc = xmesa_get_points_func( ctx );
+   if (!ctx->Driver.LineFunc)
+      ctx->Driver.LineFunc = xmesa_get_line_func( ctx );
+   if (!ctx->Driver.TriangleFunc)
+      ctx->Driver.TriangleFunc = xmesa_get_triangle_func( ctx );
 
    /*
     * These drawing functions depend on color buffer config:
@@ -2819,6 +3048,12 @@ void xmesa_setup_DD_pointers( GLcontext *ctx )
 	    ctx->Driver.WriteColorPixels     = write_pixels_TRUECOLOR_pixmap;
 	    ctx->Driver.WriteMonocolorPixels = write_pixels_mono_pixmap;
 	    break;
+	 case PF_TRUEDITHER:
+	    ctx->Driver.WriteColorSpan       = write_span_TRUEDITHER_pixmap;
+	    ctx->Driver.WriteMonocolorSpan   = write_span_mono_TRUEDITHER_pixmap;
+	    ctx->Driver.WriteColorPixels     = write_pixels_TRUEDITHER_pixmap;
+	    ctx->Driver.WriteMonocolorPixels = write_pixels_mono_TRUEDITHER_pixmap;
+	    break;
 	 case PF_8A8B8G8R:
 	    ctx->Driver.WriteColorSpan       = write_span_8A8B8G8R_pixmap;
 	    ctx->Driver.WriteMonocolorSpan   = write_span_mono_pixmap;
@@ -2868,7 +3103,8 @@ void xmesa_setup_DD_pointers( GLcontext *ctx )
             ctx->Driver.WriteMonocolorPixels = write_pixels_mono_pixmap;
             break;
 	 default:
-	    die("Bad pixel format in xmesa_setup_DD_pointers (1)");
+	    gl_problem(NULL,"Bad pixel format in xmesa_setup_DD_pointers (1)");
+            return;
       }
    }
    else if (xmesa->xm_buffer->buffer==XIMAGE) {
@@ -2908,6 +3144,12 @@ void xmesa_setup_DD_pointers( GLcontext *ctx )
 	    ctx->Driver.WriteColorPixels     = write_pixels_TRUECOLOR_ximage;
 	    ctx->Driver.WriteMonocolorPixels = write_pixels_mono_ximage;
 	    break;
+	 case PF_TRUEDITHER:
+	    ctx->Driver.WriteColorSpan       = write_span_TRUEDITHER_ximage;
+	    ctx->Driver.WriteMonocolorSpan   = write_span_mono_TRUEDITHER_ximage;
+	    ctx->Driver.WriteColorPixels     = write_pixels_TRUEDITHER_ximage;
+	    ctx->Driver.WriteMonocolorPixels = write_pixels_mono_TRUEDITHER_ximage;
+	    break;
 	 case PF_8A8B8G8R:
 	    ctx->Driver.WriteColorSpan       = write_span_8A8B8G8R_ximage;
 	    ctx->Driver.WriteMonocolorSpan   = write_span_mono_8A8B8G8R_ximage;
@@ -2981,7 +3223,8 @@ void xmesa_setup_DD_pointers( GLcontext *ctx )
 	    }
 	    break;
 	 default:
-	    die("Bad pixel format in xmesa_setup_DD_pointers (2)");
+	    gl_problem(NULL,"Bad pixel format in xmesa_setup_DD_pointers (2)");
+            return;
       }
    }
 
